@@ -1,20 +1,23 @@
 // BreakControl.cc
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: BreakControl.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: BreakControl.cc 1513 2008-03-26 18:16:44Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -42,8 +45,11 @@ static const char rcsid[] = "$Id: BreakControl.cc 1298 2007-08-30 14:18:04Z rcae
 
 #ifdef HAVE_DBUS
 #include "DBus.hh"
+#include "DBusWorkrave.hh"
 #endif
 
+using namespace std;
+
 //! Construct a new Break Controller.
 /*!
  *  \param id ID of the break this BreakControl controls.
@@ -52,9 +58,8 @@ static const char rcsid[] = "$Id: BreakControl.cc 1298 2007-08-30 14:18:04Z rcae
  *          windows.
  *  \param timer pointer to the interface of the timer that belongs to this break.
  */
-BreakControl::BreakControl(BreakId id, Core *c, IApp *app, Timer *timer) :
+BreakControl::BreakControl(BreakId id, IApp *app, Timer *timer) :
   break_id(id),
-  core(c),
   application(app),
   break_timer(timer),
   break_stage(STAGE_NONE),
@@ -63,19 +68,16 @@ BreakControl::BreakControl(BreakId id, Core *c, IApp *app, Timer *timer) :
   user_initiated(false),
   prelude_count(0),
   postponable_count(0),
-  reached_max_postpone(false),
   max_number_of_preludes(2),
-  max_number_of_postpones(2),
-  ignorable_break(true),
   fake_break(false),
   fake_break_count(0),
   user_abort(false),
   delayed_abort(false)
 {
-  set_ignorable_break(ignorable_break);
-
   assert(break_timer != NULL);
   assert(application != NULL);
+
+  core = Core::get_instance();
 }
 
 
@@ -100,8 +102,8 @@ BreakControl::heartbeat()
     {
       // Prefer the running state of the break timer as input for
       // our current activity.
-      ITimer::TimerState tstate = break_timer->get_state();
-      is_idle = (tstate == ITimer::STATE_STOPPED);
+      TimerState tstate = break_timer->get_state();
+      is_idle = (tstate == STATE_STOPPED);
     }
   else
     {
@@ -216,14 +218,14 @@ BreakControl::goto_stage(BreakStage stage)
         IActivityMonitor *monitor = core->get_activity_monitor();
         monitor->set_listener(this);
 
-        break_timer->set_insensitive_mode(Timer::MODE_IDLE_ON_LIMIT_REACHED);
+        break_timer->set_insensitive_mode(MODE_IDLE_ON_LIMIT_REACHED);
       }
       break;
 
     case STAGE_NONE:
       {
         // Teminate the break.
-        break_timer->set_insensitive_mode(Timer::MODE_IDLE_ON_LIMIT_REACHED);
+        break_timer->set_insensitive_mode(MODE_IDLE_ON_LIMIT_REACHED);
         application->hide_break_window();
         core->defrost();
 
@@ -272,8 +274,7 @@ BreakControl::goto_stage(BreakStage stage)
 
     case STAGE_PRELUDE:
       {
-        break_timer->set_insensitive_mode(Timer::MODE_FOLLOW_IDLE);
-
+        break_timer->set_insensitive_mode(MODE_FOLLOW_IDLE);
         prelude_count++;
         postponable_count++;
         prelude_time = 0;
@@ -288,8 +289,8 @@ BreakControl::goto_stage(BreakStage stage)
     case STAGE_TAKING:
       {
         // Break timer should always idle.
-        // Previous revisions set ITimer::MODE_IDLE_ON_LIMIT_REACHED
-        break_timer->set_insensitive_mode(Timer::MODE_IDLE_ALWAYS );
+        // Previous revisions set MODE_IDLE_ON_LIMIT_REACHED
+        break_timer->set_insensitive_mode(MODE_IDLE_ALWAYS );
 
         // Remove the prelude window, if necessary.
         application->hide_break_window();
@@ -300,7 +301,6 @@ BreakControl::goto_stage(BreakStage stage)
 
         // Start the break.
         break_window_start();
-        application->refresh_break_window();
 
         // Play sound
         if (!user_initiated)
@@ -385,10 +385,8 @@ BreakControl::start_break()
   delayed_abort = false;
 
   reached_max_prelude = max_number_of_preludes >= 0 && prelude_count + 1 >= max_number_of_preludes;
-  reached_max_postpone = max_number_of_postpones >= 0 && postponable_count + 1 >= max_number_of_postpones;
 
-  if ((max_number_of_preludes >= 0 && prelude_count >= max_number_of_preludes) ||
-      (max_number_of_postpones >= 0 && postponable_count >= max_number_of_postpones))
+  if (max_number_of_preludes >= 0 && prelude_count >= max_number_of_preludes)
     {
       // Forcing break without prelude.
       goto_stage(STAGE_TAKING);
@@ -514,7 +512,7 @@ BreakControl::get_break_state()
     }
   else if (break_stage == STAGE_SNOOZED)
     {
-      ret = BREAK_INACTIVE; // FIXME: actually: SUSPENDED;
+      ret = BREAK_INACTIVE;
     }
   else
     {
@@ -532,24 +530,27 @@ BreakControl::get_break_state()
 void
 BreakControl::postpone_break()
 {
-  if (!user_initiated)
+  if (break_stage == STAGE_TAKING)
     {
-      if (!fake_break)
+      if (!user_initiated)
         {
-          // Snooze the timer.
-          break_timer->snooze_timer();
+          if (!fake_break)
+            {
+              // Snooze the timer.
+              break_timer->snooze_timer();
+            }
+
+          // Update stats.
+          Statistics *stats = core->get_statistics();
+          stats->increment_break_counter(break_id, Statistics::STATS_BREAKVALUE_POSTPONED);
         }
 
-      // Update stats.
-      Statistics *stats = core->get_statistics();
-      stats->increment_break_counter(break_id, Statistics::STATS_BREAKVALUE_POSTPONED);
+      // This is to avoid a skip sound...
+      user_abort = true;
+
+      // and stop the break.
+      stop_break(true);
     }
-
-  // This is to avoid a skip sound...
-  user_abort = true;
-
-  // and stop the break.
-  stop_break(true);
 }
 
 
@@ -561,25 +562,28 @@ void
 BreakControl::skip_break()
 {
   // This is to avoid a skip sound...
-  user_abort = true;
-
-  if (break_id == BREAK_ID_DAILY_LIMIT)
+  if (break_stage == STAGE_TAKING)
     {
-      // Make sure the daily limit remains silent after skipping.
-      break_timer->inhibit_snooze();
-    }
-  else
-    {
-      // Reset the restbreak timer.
-      break_timer->reset_timer();
-    }
+      user_abort = true;
 
-  // Update stats.
-  Statistics *stats = core->get_statistics();
-  stats->increment_break_counter(break_id, Statistics::STATS_BREAKVALUE_SKIPPED);
+      if (break_id == BREAK_ID_DAILY_LIMIT)
+        {
+          // Make sure the daily limit remains silent after skipping.
+          break_timer->inhibit_snooze();
+        }
+      else
+        {
+          // Reset the restbreak timer.
+          break_timer->reset_timer();
+        }
 
-  // and stop the break.
-  stop_break(false);
+      // Update stats.
+      Statistics *stats = core->get_statistics();
+      stats->increment_break_counter(break_id, Statistics::STATS_BREAKVALUE_SKIPPED);
+
+      // and stop the break.
+      stop_break(false);
+    }
 }
 
 void
@@ -599,35 +603,16 @@ BreakControl::set_max_preludes(int m)
   max_number_of_preludes = m;
 }
 
-//! Sets the maximum number of preludes before making the break not ignorable
-void
-BreakControl::set_max_postpone(int m)
-{
-  max_number_of_postpones = m;
-}
-
-
-//! Sets the ignorable-break flags
-/*!
- *  A break that is ignorable has a skip/postpone button.
- */
-void
-BreakControl::set_ignorable_break(bool i)
-{
-  ignorable_break = i;
-}
-
-
 //! Creates and shows the break window.
 void
 BreakControl::break_window_start()
 {
   TRACE_ENTER_MSG("BreakControl::break_window_start", break_id);
 
-  application->start_break_window(break_id,
-                                  user_initiated ? true : ignorable_break);
-
+  application->create_break_window(break_id, user_initiated);
   update_break_window();
+  application->show_break_window();
+  
   TRACE_EXIT();
 }
 
@@ -637,7 +622,7 @@ BreakControl::prelude_window_start()
 {
   TRACE_ENTER_MSG("BreakControl::prelude_window_start", break_id);
 
-  application->start_prelude_window(break_id);
+  application->create_prelude_window(break_id);
 
   application->set_prelude_stage(IApp::STAGE_INITIAL);
 
@@ -652,6 +637,8 @@ BreakControl::prelude_window_start()
 
   update_prelude_window();
 
+  application->show_break_window();
+  
   TRACE_EXIT();
 }
 
@@ -679,7 +666,6 @@ BreakControl::set_state_data(bool active, const BreakStateData &data)
             " stage = " <<  data.break_stage <<
             " final = " << reached_max_prelude <<
             " total preludes = " << postponable_count <<
-            " force ignorable break = " << reached_max_postpone <<
             " time = " << data.prelude_time);
 
   // TODO: check application->hide_break_window();
@@ -765,7 +751,6 @@ BreakControl::get_state_data(BreakStateData &data)
   data.reached_max_prelude = reached_max_prelude;
   data.prelude_time = prelude_time;
   data.postponable_count = postponable_count;
-  data.reached_max_postpone = reached_max_postpone;
 }
 
 
@@ -789,7 +774,7 @@ BreakControl::send_signal(BreakStage stage)
   (void) stage;
 
 #ifdef HAVE_DBUS
-  char *progress = NULL;
+  const char *progress = NULL;
 
   switch (stage)
     {
@@ -816,7 +801,32 @@ BreakControl::send_signal(BreakStage stage)
 
   if (progress != NULL)
     {
-      workrave_service_send_break_stage_signal(break_id, progress);
+      DBus *dbus = core->get_dbus();
+      org_workrave_CoreInterface *iface = org_workrave_CoreInterface::instance(dbus);
+
+      if (iface != NULL)
+        {
+          switch (break_id)
+            {
+            case BREAK_ID_MICRO_BREAK:
+              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
+                                       progress);
+              break;
+          
+            case BREAK_ID_REST_BREAK:
+              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
+                                       progress);
+              break;
+          
+            case BREAK_ID_DAILY_LIMIT:
+              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
+                                       progress);
+              break;
+          
+            default:
+              break;
+            }
+        }
     }
 
 #endif
