@@ -1,20 +1,23 @@
 // Core.cc --- The main controller
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: Core.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: Core.cc 1578 2008-07-11 19:32:10Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,6 +25,13 @@ static const char rcsid[] = "$Id: Core.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 
 #include "debug.hh"
 
+// FIXME: remove
+#if defined(PLATFORM_OS_WIN32)
+#define BACKEND
+#include "w32debug.hh"
+#endif
+
+#include <stdlib.h>
 #include <assert.h>
 #include <iostream>
 #include <fstream>
@@ -31,17 +41,20 @@ static const char rcsid[] = "$Id: Core.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 
 #include "Util.hh"
 #include "IApp.hh"
-#include "CoreEventListener.hh"
+#include "ICoreEventListener.hh"
 #include "ActivityMonitor.hh"
 #include "TimerActivityMonitor.hh"
 #include "Break.hh"
+#include "ConfiguratorFactory.hh"
 #include "Configurator.hh"
+#include "CoreConfig.hh"
 #include "Statistics.hh"
 #include "BreakControl.hh"
 #include "Timer.hh"
 #include "TimePredFactory.hh"
 #include "TimePred.hh"
 #include "TimeSource.hh"
+#include "InputMonitorFactory.hh"
 
 #ifdef HAVE_DISTRIBUTION
 #include "DistributionManager.hh"
@@ -58,6 +71,7 @@ static const char rcsid[] = "$Id: Core.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 
 #ifdef HAVE_DBUS
 #include "DBus.hh"
+#include "DBusException.hh"
 #endif
 
 Core *Core::instance = NULL;
@@ -65,12 +79,8 @@ Core *Core::instance = NULL;
 const char *WORKRAVESTATE="WorkRaveState";
 const int SAVESTATETIME = 60;
 
-const string Core::CFG_KEY_MONITOR = "monitor";
-const string Core::CFG_KEY_MONITOR_NOISE = "monitor/noise";
-const string Core::CFG_KEY_MONITOR_ACTIVITY = "monitor/activity";
-const string Core::CFG_KEY_MONITOR_IDLE = "monitor/idle";
-const string Core::CFG_KEY_GENERAL_DATADIR = "general/datadir";
-const string Core::CFG_KEY_OPERATION_MODE = "gui/operation-mode";
+#define DBUS_PATH_WORKRAVE         "/org/workrave/Workrave/Core"
+#define DBUS_SERVICE_WORKRAVE      "org.workrave.Workrave"
 
 //! Constructs a new Core.
 Core::Core() :
@@ -97,9 +107,7 @@ Core::Core() :
   idlelog_manager(NULL)
 #  ifndef NDEBUG
   ,
-  fake_monitor(NULL),
-  script_count(-1),
-  script_start_time(-1)
+  fake_monitor(NULL)
 #  endif
 #endif
 {
@@ -127,9 +135,6 @@ Core::~Core()
 
   delete statistics;
   delete monitor;
-
-  // Set whatever values need to be set before quitting.
-  configurator->set_values_now_we_are_quitting();
   delete configurator;
 
 #ifdef HAVE_DISTRIBUTION
@@ -156,7 +161,7 @@ Core::~Core()
 
 //! Initializes the core.
 void
-Core::init(int argc, char **argv, IApp *app, char *display_name)
+Core::init(int argc, char **argv, IApp *app, const string &display_name)
 {
   application = app;
   this->argc = argc;
@@ -171,14 +176,10 @@ Core::init(int argc, char **argv, IApp *app, char *display_name)
 
   init_breaks();
   init_statistics();
+  init_bus();
 
   load_state();
   load_misc();
-
-#ifdef HAVE_DBUS
-  // FIXME: move..
-  workrave_dbus_server_init(this);
-#endif
 }
 
 
@@ -190,24 +191,24 @@ Core::init_configurator()
 
   if (Util::file_exists(ini_file))
     {
-      configurator = Configurator::create(Configurator::FormatIni);
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatIni);
       configurator->load(ini_file);
     }
   else
     {
-#if defined(HAVE_REGISTRY)
-      configurator = Configurator::create(Configurator::FormatNative);
+#if defined(PLATFORM_OS_WIN32) || defined(PLATFORM_OS_OSX)
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
 
 #elif defined(HAVE_GCONF)
       gconf_init(argc, argv, NULL);
       g_type_init();
-      configurator = Configurator::create(Configurator::FormatNative);
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
 
 #elif defined(HAVE_GDOME)
       string configFile = Util::complete_directory("config.xml", Util::SEARCH_PATH_CONFIG);
-      configurator = Configurator::create(Configurator::FormatXml);
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatXml);
 
-#  if defined(HAVE_X)
+#  if defined(PLATFORM_OS_UNIX)
       if (configFile == "" || configFile == "config.xml")
         {
           configFile = Util::get_home_directory() + "config.xml";
@@ -218,22 +219,61 @@ Core::init_configurator()
           configurator->load(configFile);
         }
 #elif defined(HAVE_QT)
-      configurator = Configurator::create(Configurator::FormatNative);
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
 #else
-#error No configuator configured
+      ini_file = Util::get_home_directory() + "workrave.ini";
+      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatIni);
+      configurator->load(ini_file);
+      configurator->save(ini_file);
 #endif
     }
 
   string home;
-  if (configurator->get_value(CFG_KEY_GENERAL_DATADIR, &home))
+  if (configurator->get_value(CoreConfig::CFG_KEY_GENERAL_DATADIR, home))
     {
       Util::set_home_directory(home);
     }
 }
 
+//! Initializes the communication bus.
+void
+Core::init_bus()
+{
+#ifdef HAVE_DBUS
+  try
+    {
+      dbus = new DBus();
+
+      dbus->init();
+
+      string name = DBUS_SERVICE_WORKRAVE;
+      char *env = getenv("WORKRAVE_DBUS_NAME");
+      if (env != NULL)
+        {
+          name = env;
+        }      
+      dbus->register_service(name);
+      dbus->register_object_path(DBUS_PATH_WORKRAVE);
+
+      extern void init_DBusWorkrave(DBus *dbus);
+      init_DBusWorkrave(dbus);
+  
+      dbus->connect(DBUS_PATH_WORKRAVE, "org.workrave.CoreInterface", this);
+      
+      dbus->connect(DBUS_PATH_WORKRAVE,
+                    "org.workrave.ConfigInterface",
+                    configurator);
+    }
+  catch (DBusException &e)
+    {
+    }
+#endif
+}
+
+
 //! Initializes the activity monitor.
 void
-Core::init_monitor(char *display_name)
+Core::init_monitor(const string &display_name)
 {
 #ifdef HAVE_DISTRIBUTION
 #ifndef NDEBUG
@@ -246,10 +286,12 @@ Core::init_monitor(char *display_name)
 #endif
 #endif
 
-  monitor = new ActivityMonitor(display_name);
+  InputMonitorFactory::init(display_name);
+  
+  monitor = new ActivityMonitor();
   load_monitor_config();
 
-  configurator->add_listener(CFG_KEY_MONITOR, this);
+  configurator->add_listener(CoreConfig::CFG_KEY_MONITOR, this);
 }
 
 
@@ -279,9 +321,6 @@ Core::init_distribution_manager()
   dist_manager->register_client_message(DCM_MONITOR, DCMT_MASTER, this);
   dist_manager->register_client_message(DCM_IDLELOG, DCMT_SIGNON, this);
   dist_manager->register_client_message(DCM_BREAKCONTROL, DCMT_PASSIVE, this);
-#ifndef NDEBUG
-  dist_manager->register_client_message(DCM_SCRIPT, DCMT_PASSIVE, this);
-#endif
 
   dist_manager->add_listener(this);
 
@@ -300,7 +339,6 @@ Core::init_statistics()
 }
 
 
-
 //! Loads the configuration of the monitor.
 void
 Core::load_monitor_config()
@@ -314,22 +352,31 @@ Core::load_monitor_config()
   assert(configurator != NULL);
   assert(monitor != NULL);
 
-  if (! configurator->get_value(CFG_KEY_MONITOR_NOISE, &noise))
+  if (! configurator->get_value(CoreConfig::CFG_KEY_MONITOR_NOISE, noise))
     noise = 9000;
-  if (! configurator->get_value(CFG_KEY_MONITOR_ACTIVITY, &activity))
+  if (! configurator->get_value(CoreConfig::CFG_KEY_MONITOR_ACTIVITY, activity))
     activity = 1000;
-  if (! configurator->get_value(CFG_KEY_MONITOR_IDLE, &idle))
+  if (! configurator->get_value(CoreConfig::CFG_KEY_MONITOR_IDLE, idle))
     idle = 5000;
 
   // Pre 1.0 compatibility...
   if (noise < 50)
-    noise *= 1000;
+    {
+      noise *= 1000;
+      configurator->set_value(CoreConfig::CFG_KEY_MONITOR_NOISE, noise);
+    }
 
   if (activity < 50)
-    activity *= 1000;
+    {
+      activity *= 1000;
+      configurator->set_value(CoreConfig::CFG_KEY_MONITOR_ACTIVITY, activity);
+    }
 
   if (idle < 50)
-    idle *= 1000;
+    {
+      idle *= 1000;
+      configurator->set_value(CoreConfig::CFG_KEY_MONITOR_IDLE, idle);
+    }
 
   TRACE_MSG("Monitor config = " << noise << " " << activity << " " << idle);
 
@@ -340,7 +387,7 @@ Core::load_monitor_config()
 
 //! Notification that the configuration has changed.
 void
-Core::config_changed_notify(string key)
+Core::config_changed_notify(const string &key)
 {
   string::size_type pos = key.find('/');
   string path;
@@ -348,10 +395,9 @@ Core::config_changed_notify(string key)
   if (pos != string::npos)
     {
       path = key.substr(0, pos);
-      key = key.substr(pos + 1);
     }
 
-  if (path == CFG_KEY_MONITOR)
+  if (path == CoreConfig::CFG_KEY_MONITOR)
     {
       load_monitor_config();
     }
@@ -436,6 +482,18 @@ Core::get_break(BreakId id)
   return &breaks[id];
 }
 
+Break *
+Core::get_break(std::string name)
+{
+  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+    {
+      if (breaks[i].get_name() == name)
+        {
+          return &breaks[i];
+        }
+    }
+  return NULL;
+}
 
 #ifdef HAVE_DISTRIBUTION
 //! Returns the distribution manager.
@@ -457,44 +515,70 @@ Core::get_operation_mode()
 
 //! Sets the operation mode
 OperationMode
-Core::set_operation_mode(OperationMode mode)
+Core::set_operation_mode(OperationMode mode, bool persistent)
 {
-  OperationMode previous_mode = operation_mode;
 
+#ifdef PLATFORM_OS_WIN32
+// FIXME: debug, remove later
+if(mode == OPERATION_MODE_NORMAL)
+APPEND_TIME("Core::set_operation_mode()", "OPERATION_MODE_NORMAL ")
+else if (mode == OPERATION_MODE_SUSPENDED)
+APPEND_TIME("Core::set_operation_mode()", "OPERATION_MODE_SUSPENDED ")
+else if (mode == OPERATION_MODE_QUIET)
+APPEND_TIME("Core::set_operation_mode()", "OPERATION_MODE_QUIET ")
+#endif
+
+  OperationMode previous_mode = operation_mode;
+  
   if (operation_mode != mode)
     {
       operation_mode = mode;
-
+      
       if (operation_mode == OPERATION_MODE_SUSPENDED)
         {
           force_idle();
           monitor->suspend();
+          stop_all_breaks();
+          
+          for( int i = 0; i < BREAK_ID_SIZEOF; ++i )
+            {
+              if( breaks[ i ].is_enabled() )
+                {
+                  breaks[ i ].get_timer()->set_insensitive_mode( MODE_IDLE_ALWAYS );
+                  //breaks[ i ].get_timer()->freeze_timer( true );
+                }
+            }
         }
       else if (previous_mode == OPERATION_MODE_SUSPENDED)
         {
+          // stop_all_breaks again will reset insensitive mode (that is good)
+          stop_all_breaks();
           monitor->resume();
         }
-
-      if (operation_mode == OPERATION_MODE_SUSPENDED ||
-          operation_mode == OPERATION_MODE_QUIET)
+      
+      if( operation_mode == OPERATION_MODE_QUIET )
         {
           stop_all_breaks();
         }
 
-      get_configurator()->set_value(CFG_KEY_OPERATION_MODE, mode);
+      if (persistent)
+        {
+          get_configurator()->set_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode);
+        }
+      
       if (core_event_listener != NULL)
         {
           core_event_listener->core_event_operation_mode_changed(mode);
         }
     }
-
+  
   return previous_mode;
 }
 
 
 //! Sets the listener for core events.
 void
-Core::set_core_events_listener(CoreEventListener *l)
+Core::set_core_events_listener(ICoreEventListener *l)
 {
   core_event_listener = l;
 }
@@ -505,11 +589,13 @@ void
 Core::force_break(BreakId id, bool initiated_by_user)
 {
   do_force_break(id, initiated_by_user);
+
 #ifdef HAVE_DISTRIBUTION
   send_break_control_message_bool_param(id, BCM_START_BREAK, initiated_by_user);
 #endif
 }
 
+
 //! Forces the start of the specified break.
 void
 Core::do_force_break(BreakId id, bool initiated_by_user)
@@ -531,6 +617,26 @@ Core::do_force_break(BreakId id, bool initiated_by_user)
 }
 
 
+//! Announces a change in time.
+void
+Core::time_changed()
+{
+  TRACE_ENTER("Core::time_changed");
+
+  // In case out timezone changed..
+  tzset();
+
+  // A change of system time idle handled by process_timewarp.
+  // This is used to handle a change in timezone on windows.
+  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+    {
+      breaks[i].get_timer()->shift_time(0);
+    }
+  
+  TRACE_EXIT();
+}
+
+
 //! Announces a powersave state.
 void
 Core::set_powersave(bool down)
@@ -539,7 +645,7 @@ Core::set_powersave(bool down)
   if (down)
     {
       // Computer is going down
-      powersave_operation_mode = set_operation_mode(OPERATION_MODE_SUSPENDED);
+      powersave_operation_mode = set_operation_mode(OPERATION_MODE_SUSPENDED, false);
       powersave_resume_time = 0;
       powersave = true;
 
@@ -553,10 +659,10 @@ Core::set_powersave(bool down)
       // or until some time has passed
       if (powersave_resume_time == 0)
         {
-          powersave_resume_time = current_time;
+          powersave_resume_time = current_time ? current_time : 1;
         }
 
-      set_operation_mode(powersave_operation_mode);
+      set_operation_mode(powersave_operation_mode, false);
     }
   TRACE_EXIT();
 }
@@ -615,7 +721,7 @@ Core::force_idle()
 
 
 /********************************************************************************/
-/**** Break Resonse                                                        ******/
+/**** Break Response                                                        ******/
 /********************************************************************************/
 
 //! User postpones the specified break.
@@ -623,10 +729,12 @@ void
 Core::postpone_break(BreakId break_id)
 {
   do_postpone_break(break_id);
+
 #ifdef HAVE_DISTRIBUTION
   send_break_control_message(break_id, BCM_POSTPONE);
 #endif
 
+  /* FIXME: move to frontend
   if (resume_break != BREAK_ID_NONE &&
       !breaks[resume_break].get_break_ignorable())
     {
@@ -639,6 +747,7 @@ Core::postpone_break(BreakId break_id)
           resume_break = BREAK_ID_NONE;
         }
     }
+  */
 }
 
 
@@ -647,10 +756,12 @@ void
 Core::skip_break(BreakId break_id)
 {
   do_skip_break(break_id);
+
 #ifdef HAVE_DISTRIBUTION
   send_break_control_message(break_id, BCM_SKIP);
 #endif
 
+  /* FIXME: move to frontend
   if (resume_break != BREAK_ID_NONE &&
       !breaks[resume_break].get_break_ignorable())
     {
@@ -663,6 +774,7 @@ Core::skip_break(BreakId break_id)
           resume_break = BREAK_ID_NONE;
         }
     }
+  */
 }
 
 
@@ -672,9 +784,11 @@ Core::stop_prelude(BreakId break_id)
 {
   TRACE_ENTER_MSG("Core::stop_prelude", break_id);
   do_stop_prelude(break_id);
+
 #ifdef HAVE_DISTRIBUTION
   send_break_control_message(break_id, BCM_ABORT_PRELUDE);
 #endif
+
   TRACE_EXIT();
 }
 
@@ -731,6 +845,9 @@ Core::heartbeat()
   // Set current time.
   current_time = time(NULL);
 
+  // Process configuration
+  configurator->heartbeat();
+
   // Perform distribution processing.
   process_distribution();
 
@@ -777,12 +894,6 @@ Core::process_distribution()
   master_node = true;
 
 #ifdef HAVE_DISTRIBUTION
-#ifndef NDEBUG
-  if (script_start_time != -1 && current_time >= script_start_time)
-    {
-      do_script();
-    }
-#endif
 
   // Retrieve State.
   ActivityState state = monitor->get_current_state();
@@ -828,6 +939,25 @@ Core::process_state()
 {
   // Default
   local_state = monitor->get_current_state();
+
+  map<std::string, int>::iterator i = external_activity.begin();
+  while (i != external_activity.end())
+    {
+      map<std::string, int>::iterator next = i;
+      next++;
+
+      if (i->second >= current_time)
+        {
+          local_state = ACTIVITY_ACTIVE;
+        }
+      else
+        {
+          external_activity.erase(i);
+        }
+
+      i = next;
+    }
+
   monitor_state = local_state;
 
 #if defined(HAVE_DISTRIBUTION) and !defined(NDEBUG)
@@ -858,6 +988,46 @@ Core::process_state()
 }
 
 
+void
+Core::report_external_activity(std::string who, bool act)
+{
+  TRACE_ENTER_MSG("Core::report_external_activity", who << " " << act);
+  if (act)
+    {
+      external_activity[who] = current_time + 10;
+    }
+  else
+    {
+      external_activity.erase(who);
+    }
+  TRACE_EXIT();
+}
+
+
+void
+Core::is_timer_running(BreakId id, bool &value)
+{
+  Timer *timer = get_timer(id);
+  value = timer->get_state() == STATE_RUNNING;
+}
+
+
+void
+Core::get_timer_idle(BreakId id, int *value)
+{
+  Timer *timer = get_timer(id);
+  *value = timer->get_elapsed_idle_time();
+}
+
+
+void
+Core::get_timer_elapsed(BreakId id, int *value)
+{
+  Timer *timer = get_timer(id);
+  *value = timer->get_elapsed_time();
+}
+
+
 //! Processes all timers.
 void
 Core::process_timers()
@@ -918,8 +1088,9 @@ Core::process_timers()
   TRACE_EXIT();
 }
 
+#ifdef PLATFORM_OS_WIN32
 
-//! Process a possible timewarp
+//! Process a possible timewarp on Win32
 void
 Core::process_timewarp()
 {
@@ -933,9 +1104,9 @@ Core::process_timewarp()
             {
               TRACE_MSG("Time warp of " << gap << " seconds. Correcting");
 
+
               force_idle();
 
-#ifdef WIN32
               monitor->shift_time(gap);
               for (int i = 0; i < BREAK_ID_SIZEOF; i++)
                 {
@@ -943,22 +1114,12 @@ Core::process_timewarp()
                 }
 
               monitor_state = ACTIVITY_IDLE;
-#else
-              /* Assume it is a powersave... */
-              int save_current_time = current_time;
-
-              current_time = last_process_time + 1;
-              monitor_state = ACTIVITY_IDLE;
-
-              process_timers();
-
-              current_time = save_current_time;
-#endif
             }
           else
             {
               TRACE_MSG("Time warp of " << gap << " seconds because of powersave");
 
+
               // In case the windows message was lost. some people reported that
               // workrave never restarted the timers...
               set_operation_mode(powersave_operation_mode);
@@ -967,6 +1128,8 @@ Core::process_timewarp()
       if (powersave && powersave_resume_time != 0 && current_time > powersave_resume_time + 30)
         {
           TRACE_MSG("End of time warp after powersave");
+
+
           powersave = false;
           powersave_resume_time = 0;
 
@@ -975,6 +1138,64 @@ Core::process_timewarp()
   TRACE_EXIT();
 }
 
+#else
+
+//! Process a possible timewarp On Non-Windows
+void
+Core::process_timewarp()
+{
+  TRACE_ENTER("Core::process_timewarp");
+  if (last_process_time != 0)
+    {
+      int gap = current_time - 1 - last_process_time;
+      bool do_powersafe = false;
+      bool do_timeshift = false;
+
+      if (gap < 0 || (gap > 5 && gap < 600))
+        {
+          // Negative gap are always intrepreted as timechanges.
+          // Positive gaps between 5..600 are also considered timechanges
+          do_timeshift = true;
+        }
+      else if (gap >= 600)
+        {
+          // Gaps > 600 are considered powersaves.
+          do_powersafe = true;
+        }
+      
+      if (do_powersafe)
+        {
+          TRACE_MSG("Time warp of " << gap << " seconds. Powersafe");
+          
+          force_idle();
+
+          /* Assume it is a powersave... */
+          int save_current_time = current_time;
+
+          current_time = last_process_time + 1;
+          monitor_state = ACTIVITY_IDLE;
+          
+          process_timers();
+
+          current_time = save_current_time;
+        }
+
+      if (do_timeshift)
+        {
+          TRACE_MSG("Time warp of " << gap << " seconds. Timeshift");
+
+          monitor->shift_time(gap);
+          for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+            {
+              breaks[i].get_timer()->shift_time(gap);
+            }
+        }
+    }
+  
+  TRACE_EXIT();
+}
+
+#endif
 
 //! Notication of a timer action.
 /*!
@@ -1128,7 +1349,7 @@ Core::stop_all_breaks()
 /**** Misc                                                                 ******/
 /********************************************************************************/
 
-//! Performs a reset when the dailt limit is reached.
+//! Performs a reset when the daily limit is reached.
 void
 Core::daily_reset()
 {
@@ -1146,8 +1367,7 @@ Core::daily_reset()
       t->daily_reset_timer();
     }
 
-  monitor->reset_statistics();
-
+  
 #ifdef HAVE_DISTRIBUTION
   idlelog_manager->reset();
 #endif
@@ -1166,7 +1386,7 @@ Core::save_state() const
 
   ofstream stateFile(ss.str().c_str());
 
-  stateFile << "WorkRaveState 2"  << endl
+  stateFile << "WorkRaveState 3"  << endl
             << get_time() << endl;
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
@@ -1185,11 +1405,11 @@ void
 Core::load_misc()
 {
   int mode;
-  if (! get_configurator()->get_value(CFG_KEY_OPERATION_MODE, &mode))
+  if (! get_configurator()->get_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode))
     {
       mode = OPERATION_MODE_NORMAL;
     }
-  set_operation_mode(OperationMode(mode));
+  set_operation_mode(OperationMode(mode), false);
 }
 
 
@@ -1203,6 +1423,7 @@ Core::load_state()
 
   ifstream stateFile(ss.str().c_str());
 
+  int version = 0;
   bool ok = stateFile;
 
   if (ok)
@@ -1215,10 +1436,9 @@ Core::load_state()
 
   if (ok)
     {
-      int version;
       stateFile >> version;
 
-      ok = (version == 1 || version == 2);
+      ok = (version >= 1 && version <= 3);
     }
 
   if (ok)
@@ -1239,7 +1459,7 @@ Core::load_state()
               string state;
               getline(stateFile, state);
 
-              breaks[i].get_timer()->deserialize_state(state);
+              breaks[i].get_timer()->deserialize_state(state, version);
               break;
             }
         }
@@ -1247,7 +1467,7 @@ Core::load_state()
 }
 
 
-//! Posts an event.
+//! Post an event.
 void
 Core::post_event(CoreEvent event)
 {
@@ -1258,7 +1478,7 @@ Core::post_event(CoreEvent event)
 }
 
 
-//! Excecutes the insist policy.
+//! Excecute the insist policy.
 void
 Core::freeze()
 {
@@ -1293,7 +1513,7 @@ Core::freeze()
 }
 
 
-//! Undoes the insist policy.
+//! Undo the insist policy.
 void
 Core::defrost()
 {
@@ -1326,53 +1546,18 @@ Core::defrost()
 }
 
 
+//! Is the user currently active?
+bool
+Core::is_user_active() const
+{
+  return monitor_state == ACTIVITY_ACTIVE;
+}
+
+
 /********************************************************************************/
 /**** Distribution                                                         ******/
 /********************************************************************************/
 
-#ifndef NDEBUG
-void
-Core::test_me()
-{
-  TRACE_ENTER("Core::test_me");
-
-#if 0
-  script_count = 0;
-  script_start_time = current_time + 5;
-
-  PacketBuffer buffer;
-  buffer.create();
-
-  buffer.pack_ushort(1);
-  buffer.pack_ushort(SCRIPT_START);
-  buffer.pack_ushort(script_start_time);
-
-  dist_manager->broadcast_client_message(DCM_SCRIPT, buffer);
-
-#else
-#ifdef HAVE_DISTRIBUTION
-  if (fake_monitor != NULL)
-    {
-      ActivityState state = fake_monitor->get_current_state();
-
-      if (state == ACTIVITY_ACTIVE)
-        {
-          TRACE_MSG("Setting idle");
-          fake_monitor->set_state(ACTIVITY_IDLE);
-        }
-      else
-        {
-          TRACE_MSG("Setting master");
-          fake_monitor->set_state(ACTIVITY_ACTIVE);
-        }
-    }
-#endif
-#endif
-  TRACE_EXIT();
-}
-#endif
-
-
 #ifdef HAVE_DISTRIBUTION
 //! The distribution manager requests a client message.
 bool
@@ -1421,6 +1606,8 @@ Core::client_message(DistributionClientMessageID id, bool master, const char *cl
 {
   bool ret = false;
 
+  (void) client_id;
+  
   switch (id)
     {
     case DCM_BREAKS:
@@ -1448,11 +1635,6 @@ Core::client_message(DistributionClientMessageID id, bool master, const char *cl
       ret = true;
       break;
 
-#ifndef NDEBUG
-    case DCM_SCRIPT:
-      ret = script_message(master, client_id, buffer);
-      break;
-#endif
     default:
       break;
     }
@@ -1778,89 +1960,24 @@ Core::set_break_control(PacketBuffer &buffer)
   return true;
 }
 
-
-#ifndef NDEBUG
-bool
-Core::script_message(bool master, const char *client_id, PacketBuffer &buffer)
-{
-  TRACE_ENTER("Core::script_message");
-
-  (void) master;
-  (void) client_id;
-
-  int cmd_size = buffer.unpack_ushort();
-  for (int i = 0; i < cmd_size; i++)
-    {
-      ScriptCommand type = (ScriptCommand) buffer.unpack_ushort();
-
-      switch (type)
-        {
-        case SCRIPT_START:
-          {
-            script_start_time = buffer.unpack_ulong();
-            script_count = 0;
-          }
-          break;
-        }
-    }
-
-  TRACE_EXIT();
-  return true;
-}
-
-
-void
-Core::do_script()
-{
-  string id = dist_manager->get_my_id();
-  PacketBuffer packet;
-
-  if (id == "192.168.0.42:2701")
-    {
-      if (script_count == 0)
-        {
-          fake_monitor->set_state(ACTIVITY_ACTIVE);
-        }
-      else if (script_count == 5)
-        {
-          fake_monitor->set_state(ACTIVITY_IDLE);
-        }
-      else if (script_count == 40)
-        {
-          fake_monitor->set_state(ACTIVITY_ACTIVE);
-        }
-      else if (script_count == 50)
-        {
-          fake_monitor->set_state(ACTIVITY_IDLE);
-          dist_manager->disconnect("192.168.0.42:2702");
-        }
-      else if (script_count == 85)
-        {
-          dist_manager->connect("tcp://192.168.0.42:2702");
-        }
-    }
-  else if (id == "192.168.0.42:2702")
-    {
-      if (script_count == 51)
-        {
-          fake_monitor->set_state(ACTIVITY_ACTIVE);
-        }
-      else if (script_count == 61)
-        {
-          fake_monitor->set_state(ACTIVITY_IDLE);
-        }
-      else if (script_count == 71)
-        {
-          fake_monitor->set_state(ACTIVITY_ACTIVE);
-        }
-      else if (script_count == 81)
-        {
-          fake_monitor->set_state(ACTIVITY_IDLE);
-        }
-    }
-
-  script_count++;
-}
-
-#endif // NDEBUG
 #endif // HAVE_DISTRIBUTION
+
+namespace workrave
+{
+  std::string operator%(const string &key, BreakId id)
+  {
+    IBreak *b = Core::get_instance()->get_break(id);
+  
+    string str = key;
+    string::size_type pos = 0;
+    string name = b->get_name();
+  
+    while ((pos = str.find("%b", pos)) != string::npos)
+      {
+        str.replace(pos, 2, name);
+        pos++;
+      }
+  
+    return str;
+  }
+}
