@@ -1,32 +1,35 @@
 // CoreFactory.cc
 //
-// Copyright (C) 2001, 2002, 2003, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2006, 2007 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: CoreFactory.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: CoreFactory.cc 1409 2008-01-08 19:38:35Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "debug.hh"
 #include <assert.h>
 
 #include "CoreFactory.hh"
 #include "Configurator.hh"
 #include "Core.hh"
 
+//! Returns the interface to the core.
 ICore *
 CoreFactory::get_core()
 {
@@ -34,6 +37,7 @@ CoreFactory::get_core()
 }
 
 
+//! Returns the interface to the configurator
 IConfigurator *
 CoreFactory::get_configurator()
 {
@@ -42,3 +46,18 @@ CoreFactory::get_configurator()
 
   return core->get_configurator();
 }
+
+
+//! Returns the interface to the D-BUS facility
+DBus *
+CoreFactory::get_dbus()
+{
+#ifdef HAVE_DBUS
+  Core *core = Core::get_instance();
+  assert(core != NULL);
+
+  return core->get_dbus();
+#else
+  return NULL;
+#endif
+}
