@@ -1,6 +1,6 @@
 // DistributionManager.cc
 //
-// Copyright (C) 2002, 2003, 2004, 2006 Rob Caelers <robc@krandor.org>
+// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008 Rob Caelers <robc@krandor.org>
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: DistributionManager.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: DistributionManager.cc 1467 2008-03-08 22:28:03Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -33,26 +33,19 @@ static const char rcsid[] = "$Id: DistributionManager.cc 1298 2007-08-30 14:18:0
 #include "DistributionLogListener.hh"
 #include "DistributionListener.hh"
 #include "Configurator.hh"
+#include "CoreConfig.hh"
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
 #endif
 
-const string DistributionManager::CFG_KEY_DISTRIBUTION = "distribution";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_ENABLED = "/enabled";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_PEERS = "/peers";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP = "distribution/tcp";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP_PORT = "/port";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP_USERNAME = "/username";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP_PASSWORD = "/password";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP_ATTEMPTS = "/reconnect_attempts";
-const string DistributionManager::CFG_KEY_DISTRIBUTION_TCP_INTERVAL = "/reconnect_interval";
-
+#define MAX_LOG_LEN (256)
 
 //! Constructs a new DistributionManager.
 DistributionManager::DistributionManager() :
-  distribution_enabled(false),
+  network_enabled(false),
+  server_enabled(false),
   link(NULL),
   state(NODE_ACTIVE)
 {
@@ -81,7 +74,7 @@ DistributionManager::init(Configurator *conf)
 
   // Read configuration.
   read_configuration();
-  configurator->add_listener(CFG_KEY_DISTRIBUTION, this);
+  configurator->add_listener(CoreConfig::CFG_KEY_DISTRIBUTION, this);
 }
 
 
@@ -509,11 +502,13 @@ DistributionManager::read_configuration()
   bool is_set;
 
   // Distributed operation enabled or not.
-  distribution_enabled = get_enabled();
+  network_enabled = get_enabled();
+  server_enabled = get_listening();
 
   // Enable/Disable link.
   assert(link != NULL);
-  link->set_enabled(distribution_enabled);
+  link->set_network_enabled(network_enabled);
+  link->set_server_enabled(server_enabled);
 
   // Peers
   const char *env = getenv("WORKRAVE_URL");
@@ -524,7 +519,7 @@ DistributionManager::read_configuration()
   else
     {
       string peer ;
-      is_set = configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_PEERS, &peer);
+      is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_PEERS, peer);
       if (is_set)
         {
           parse_peers(peer);
@@ -548,14 +543,14 @@ DistributionManager::write_peers()
       peers += (*i);
     }
 
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_PEERS, peers);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_PEERS, peers);
   TRACE_EXIT();
 }
 
 
 //! Notification that the specified configuration key has changed.
 void
-DistributionManager::config_changed_notify(string key)
+DistributionManager::config_changed_notify(const string &key)
 {
   TRACE_ENTER_MSG("DistributionManager:config_changed_notify", key);
 
@@ -565,7 +560,7 @@ DistributionManager::config_changed_notify(string key)
 }
 
 
-//! Notification that the specified configuration key has changed.
+//! 
 void
 DistributionManager::log(char *fmt, ...)
 {
@@ -576,17 +571,15 @@ DistributionManager::log(char *fmt, ...)
   time_t current_time = time (NULL);
   struct tm *lt = localtime(&current_time);
 
-  char str[256];
-  snprintf(str, 255, "[%02d/%02d/%02d %02d:%02d:%02d] ",
-          lt->tm_mday, lt->tm_mon + 1, lt->tm_year + 1900,
-          lt->tm_hour, lt->tm_min, lt->tm_sec);
+  char log_str[MAX_LOG_LEN];
+  vsnprintf(log_str, MAX_LOG_LEN - 1, fmt, va);
+  log_str[MAX_LOG_LEN - 1] = '\0';
 
-  char *ptr = str + strlen(str);
-  vsnprintf(ptr, 255 - strlen(str), fmt, va);
-
-  ptr = str + strlen(str);
-  ptr[0] = '\n';
-  ptr[1] = 0;
+  char str[MAX_LOG_LEN];
+  snprintf(str, MAX_LOG_LEN - 1, "[%02d/%02d/%02d %02d:%02d:%02d] %s\n",
+           lt->tm_mday, lt->tm_mon + 1, lt->tm_year + 1900,
+           lt->tm_hour, lt->tm_min, lt->tm_sec, log_str);
+  str[MAX_LOG_LEN - 1] = '\0';
 
   log_messages.push_back(str);
 
@@ -763,8 +756,7 @@ bool
 DistributionManager::get_enabled() const
 {
   bool ret = true;
-  bool is_set = configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_ENABLED,
-                                        &ret);
+  bool is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_ENABLED, ret);
   if (!is_set)
     {
       ret = false;
@@ -777,7 +769,29 @@ DistributionManager::get_enabled() const
 void
 DistributionManager::set_enabled(bool b)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_ENABLED, b);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_ENABLED, b);
+}
+
+
+bool
+DistributionManager::get_listening() const
+{
+  bool ret = true;
+  bool is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_LISTENING,
+                                        ret);
+  if (!is_set)
+    {
+      ret = false;
+    }
+
+  return ret;
+}
+
+
+void
+DistributionManager::set_listening(bool b)
+{
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_LISTENING, b);
 }
 
 
@@ -785,8 +799,7 @@ string
 DistributionManager::get_username() const
 {
   string ret;
-  configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_USERNAME,
-                          &ret);
+  configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_USERNAME, ret);
   return ret;
 }
 
@@ -794,7 +807,7 @@ DistributionManager::get_username() const
 void
 DistributionManager::set_username(string name)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_USERNAME, name);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION + CoreConfig::CFG_KEY_DISTRIBUTION_TCP_USERNAME, name);
 }
 
 
@@ -802,8 +815,7 @@ string
 DistributionManager::get_password() const
 {
   string ret;
-  configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_PASSWORD,
-                          &ret);
+  configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_PASSWORD, ret);
   return ret;
 }
 
@@ -811,7 +823,7 @@ DistributionManager::get_password() const
 void
 DistributionManager::set_password(string name)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_PASSWORD, name);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_PASSWORD, name);
 }
 
 
@@ -819,8 +831,7 @@ int
 DistributionManager::get_port() const
 {
   int ret;
-  bool is_set = configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_PORT,
-                                        &ret);
+  bool is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_PORT, ret);
   if (!is_set)
     {
       ret = DEFAULT_PORT;
@@ -833,7 +844,7 @@ DistributionManager::get_port() const
 void
 DistributionManager::set_port(int v)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_PORT, v);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_PORT, v);
 }
 
 
@@ -841,8 +852,7 @@ int
 DistributionManager::get_reconnect_attempts() const
 {
   int ret;
-  bool is_set = configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_ATTEMPTS,
-                                        &ret);
+  bool is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_ATTEMPTS, ret);
   if (!is_set)
     {
       ret = DEFAULT_ATTEMPTS;
@@ -855,7 +865,7 @@ DistributionManager::get_reconnect_attempts() const
 void
 DistributionManager::set_reconnect_attempts(int v)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_ATTEMPTS, v);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_ATTEMPTS, v);
 }
 
 
@@ -863,8 +873,7 @@ int
 DistributionManager::get_reconnect_interval() const
 {
   int ret;
-  bool is_set = configurator->get_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_INTERVAL,
-                                        &ret);
+  bool is_set = configurator->get_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_INTERVAL, ret);
   if (!is_set)
     {
       ret = DEFAULT_INTERVAL;
@@ -877,5 +886,5 @@ DistributionManager::get_reconnect_interval() const
 void
 DistributionManager::set_reconnect_interval(int v)
 {
-  configurator->set_value(CFG_KEY_DISTRIBUTION + CFG_KEY_DISTRIBUTION_TCP_INTERVAL, v);
+  configurator->set_value(CoreConfig::CFG_KEY_DISTRIBUTION_TCP_INTERVAL, v);
 }
