@@ -1,6 +1,6 @@
 // DistributionSocketLink.cc
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers <robc@krandor.org>
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers <robc@krandor.org>
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
 //
 //
 
-static const char rcsid[] = "$Id: DistributionSocketLink.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: DistributionSocketLink.cc 1467 2008-03-08 22:28:03Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,6 +36,8 @@ static const char rcsid[] = "$Id: DistributionSocketLink.cc 1298 2007-08-30 14:1
 #include <signal.h>
 
 #include "Configurator.hh"
+#include "CoreConfig.hh"
+
 #include "DistributionManager.hh"
 #include "DistributionLink.hh"
 #include "DistributionSocketLink.hh"
@@ -59,6 +61,7 @@ DistributionSocketLink::DistributionSocketLink(Configurator *conf) :
   myid(NULL),
   server_port(DEFAULT_PORT),
   server_socket(NULL),
+  network_enabled(false),
   server_enabled(false),
   reconnect_attempts(DEFAULT_ATTEMPTS),
   reconnect_interval(DEFAULT_INTERVAL),
@@ -95,7 +98,7 @@ DistributionSocketLink::init()
 
   // Read all tcp link configuration.
   read_configuration();
-  configurator->add_listener(DistributionManager::CFG_KEY_DISTRIBUTION_TCP, this);
+  configurator->add_listener(CoreConfig::CFG_KEY_DISTRIBUTION_TCP, this);
 
   TRACE_EXIT();
 }
@@ -124,7 +127,8 @@ DistributionSocketLink::heartbeat()
               c->reconnect_count--;
               c->reconnect_time = 0;
 
-              dist_manager->log(_("Reconnecting to %s:%d."), c->hostname, c->port);
+              dist_manager->log(_("Reconnecting to %s:%d."),
+                                c->hostname == NULL ? "Unknown" : c->hostname, c->port);
               socket_driver->connect(c->hostname, c->port, c);
             }
           i++;
@@ -179,9 +183,8 @@ DistributionSocketLink::get_number_of_peers()
 void
 DistributionSocketLink::connect(string url)
 {
-  if (server_enabled)
+  if (network_enabled)
     {
-#ifdef HAVE_GNET2
       GURI *client_url = gnet_uri_new(url.c_str());
 
       if (client_url != NULL)
@@ -190,15 +193,6 @@ DistributionSocketLink::connect(string url)
         }
       gnet_uri_delete(client_url);
 
-#else
-      GURL *client_url = gnet_url_new(url.c_str());
-
-      if (client_url != NULL)
-        {
-          add_client(NULL, client_url->hostname, client_url->port, CLIENTTYPE_DIRECT);
-        }
-      gnet_url_delete(client_url);
-#endif
     }
 }
 
@@ -324,13 +318,10 @@ DistributionSocketLink::set_distribution_manager(DistributionManager *dll)
 }
 
 
-//! Enable/disable distributed operation.
+//! Enable/disable connecting to distributed operation.
 bool
-DistributionSocketLink::set_enabled(bool enabled)
+DistributionSocketLink::set_network_enabled(bool enabled)
 {
-  TRACE_ENTER_MSG("DistributionSocketLink:set_enabled", enabled);
-  bool ret = server_enabled;
-
   if (enabled)
     {
       if (myname != NULL)
@@ -349,7 +340,6 @@ DistributionSocketLink::set_enabled(bool enabled)
       if (myname != NULL)
         {
           myid = g_strdup_printf("%s:%d", myname, server_port);
-          TRACE_MSG(myid);
         }
       else
         {
@@ -358,6 +348,33 @@ DistributionSocketLink::set_enabled(bool enabled)
         }
     }
 
+  if (network_enabled && !enabled)
+    {
+      set_server_enabled(false);
+      set_me_master();
+    }
+  else if (!network_enabled && enabled)
+    {
+      set_server_enabled(server_enabled);
+    }
+
+  network_enabled = enabled;
+  return network_enabled;
+}
+
+
+//! Enable/disable listening for distributed operation.
+bool
+DistributionSocketLink::set_server_enabled(bool enabled)
+{
+  TRACE_ENTER_MSG("DistributionSocketLink:set_server_enabled", enabled);
+  bool ret = server_enabled;
+
+  if (!network_enabled) {
+    // Don't start the server if the network is not enabled
+    return ret;
+  }
+
   if (!server_enabled && enabled)
     {
       // Switching from disabled to enabled;
@@ -379,7 +396,6 @@ DistributionSocketLink::set_enabled(bool enabled)
         }
       server_socket = NULL;
       disconnect_all();
-      set_me_master();
     }
 
   server_enabled = enabled;
@@ -529,7 +545,8 @@ DistributionSocketLink::add_client(gchar *id, gchar *host, gint port, ClientType
         }
 
       c->type = type;
-      dist_manager->log(_("Connecting to %s:%d."), host, port);
+      dist_manager->log(_("Connecting to %s:%d."),
+                        host == NULL ? "Unknown" : host, port);
       socket_driver->connect(host, port, c);
       skip = true;
     }
@@ -556,7 +573,8 @@ DistributionSocketLink::add_client(gchar *id, gchar *host, gint port, ClientType
 
       if (type == CLIENTTYPE_DIRECT)
         {
-          dist_manager->log(_("Connecting to %s:%d."), host, port);
+          dist_manager->log(_("Connecting to %s:%d."),
+                            host == NULL ? "Unknown" : host, port);
           socket_driver->connect(host, port, client);
         }
     }
@@ -898,7 +916,8 @@ DistributionSocketLink::set_master_by_id(gchar *id)
   if (c != NULL)
     {
       // It's a remote client. mark it master.
-      dist_manager->log(_("Client %s:%d is now master."), c->hostname, c->port);
+      dist_manager->log(_("Client %s:%d is now master."),
+                        c->hostname == NULL ? "Unknown" : c->hostname, c->port);
       set_master(c);
     }
   else if (strcmp(id, myid) == 0)
@@ -1235,7 +1254,8 @@ DistributionSocketLink::handle_hello(PacketBuffer &packet, Client *client)
   gchar *name = packet.unpack_string();
   gint port = packet.unpack_ushort();
 
-  dist_manager->log(_("Client %s:%d saying hello."), name, port);
+  dist_manager->log(_("Client %s:%d saying hello."),
+                    name == NULL ? "Unknown" : name, port);
 
   if ( (username == NULL || (user != NULL && strcmp(username, user) == 0)) &&
        (password == NULL || (pass != NULL && strcmp(password, pass) == 0)))
@@ -1250,7 +1270,8 @@ DistributionSocketLink::handle_hello(PacketBuffer &packet, Client *client)
       else
         {
           // Duplicate client. inform client that it's bogus and close.
-          dist_manager->log(_("Client %s:%d is duplicate."), name, port);
+          dist_manager->log(_("Client %s:%d is duplicate."),
+                            name == NULL ? "Unknown" : name, port);
 
           send_duplicate(client);
           remove_client(client);
@@ -1259,7 +1280,8 @@ DistributionSocketLink::handle_hello(PacketBuffer &packet, Client *client)
   else
     {
       // Incorrect password.
-      dist_manager->log(_("Client %s:%d access denied."), name, port);
+      dist_manager->log(_("Client %s:%d access denied."),
+                        name == NULL ? "Unknown" : name, port);
       remove_client(client);
     }
 
@@ -1417,7 +1439,8 @@ DistributionSocketLink::handle_welcome(PacketBuffer &packet, Client *client)
   gchar *name = packet.unpack_string();
   gint port = packet.unpack_ushort();
 
-  dist_manager->log(_("Client %s:%d is welcoming us."), name, port);
+  dist_manager->log(_("Client %s:%d is welcoming us."),
+                    name == NULL ? "Unknown" : name, port);
 
   bool ok = set_client_id(client, id, name, port);
 
@@ -1655,7 +1678,8 @@ DistributionSocketLink::send_claim(Client *client)
     {
       PacketBuffer packet;
 
-      dist_manager->log(_("Requesting master status from %s:%d."), client->hostname, client->port);
+      dist_manager->log(_("Requesting master status from %s:%d."),
+                        client->hostname == NULL ? "Unknown" : client->hostname, client->port);
 
       packet.create();
       init_packet(packet, PACKET_CLAIM);
@@ -1668,7 +1692,8 @@ DistributionSocketLink::send_claim(Client *client)
 
       if (client->claim_count >= 3)
         {
-          dist_manager->log(_("Client timeout from %s:%d."), client->hostname, client->port);
+          dist_manager->log(_("Client timeout from %s:%d."),
+                            client->hostname == NULL ? "Unknown" : client->hostname, client->port);
 
           close_client(client, client->outbound);
         }
@@ -1690,13 +1715,13 @@ DistributionSocketLink::handle_claim(PacketBuffer &packet, Client *client)
   if (i_am_master && master_locked)
     {
       dist_manager->log(_("Rejecting master request from client %s:%d."),
-                        client->hostname, client->port);
+                        client->hostname == NULL ? "Unknown" : client->hostname, client->port);
       send_claim_reject(client);
     }
   else
     {
       dist_manager->log(_("Acknowledging master request from client %s:%d."),
-                        client->hostname, client->port);
+                        client->hostname == NULL ? "Unknown" : client->hostname, client->port);
 
       bool was_master = i_am_master;
 
@@ -1747,12 +1772,12 @@ DistributionSocketLink::handle_claim_reject(PacketBuffer &packet, Client *client
   if (client != master_client)
     {
       dist_manager->log(_("Non-master client %s:%d rejected master request."),
-                        client->hostname, client->port);
+                        client->hostname == NULL ? "Unknown" : client->hostname, client->port);
     }
   else
     {
       dist_manager->log(_("Client %s:%d rejected master request, delaying."),
-                        client->hostname, client->port);
+                        client->hostname == NULL ? "Unknown" : client->hostname, client->port);
       client->reject_count++;
       int count = client->reject_count;
 
@@ -1822,7 +1847,8 @@ DistributionSocketLink::handle_new_master(PacketBuffer &packet, Client *client)
   gchar *id = packet.unpack_string();
   /* gint count = */ packet.unpack_ushort();
 
-  dist_manager->log(_("Client %s is now the new master."), id);
+  dist_manager->log(_("Client %s is now the new master."),
+                    id == NULL ? "Unknown" : id);
 
   if (client->id != NULL)
     {
@@ -2141,8 +2167,8 @@ DistributionSocketLink::read_configuration()
 
   if (old_port != server_port && server_enabled)
     {
-      set_enabled(false);
-      set_enabled(true);
+      set_server_enabled(false);
+      set_server_enabled(true);
     }
 
   reconnect_interval = dist_manager->get_reconnect_interval();
@@ -2159,7 +2185,7 @@ DistributionSocketLink::read_configuration()
 
 //! Notification from the configurator that the configuration has changed.
 void
-DistributionSocketLink::config_changed_notify(string key)
+DistributionSocketLink::config_changed_notify(const string &key)
 {
   (void) key;
   read_configuration();
