@@ -1,28 +1,33 @@
 // Statistics.cc
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2002 - 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: Statistics.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: Statistics.cc 1585 2008-07-13 08:59:32Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 
+#include <cstring>
 #include <sstream>
 #include <assert.h>
+#include <math.h>
 
 #include "debug.hh"
 
@@ -32,6 +37,9 @@ static const char rcsid[] = "$Id: Statistics.cc 1298 2007-08-30 14:18:04Z rcaele
 #include "Util.hh"
 #include "Timer.hh"
 #include "TimePred.hh"
+#include "InputMonitorFactory.hh"
+#include "IInputMonitor.hh"
+#include "timeutil.h"
 
 #ifdef HAVE_DISTRIBUTION
 #include "DistributionManager.hh"
@@ -45,8 +53,14 @@ const int STATSVERSION = 4;
 Statistics::Statistics() :
   core(NULL),
   current_day(NULL),
-  been_active(false)
+  been_active(false),
+  prev_x(-1),
+  prev_y(-1),
+  click_x(-1),
+  click_y(-1)
 {
+  last_mouse_time.tv_sec = 0;
+  last_mouse_time.tv_usec = 0;
 }
 
 
@@ -61,6 +75,11 @@ Statistics::~Statistics()
     }
 
   delete current_day;
+
+  if (input_monitor != NULL)
+    {
+      input_monitor->unsubscribe_statistics(this);
+    }
 }
 
 
@@ -70,6 +89,12 @@ Statistics::init(Core *control)
 {
   core = control;
 
+  input_monitor = InputMonitorFactory::get_monitor(IInputMonitorFactory::CAPABILITY_STATISTICS);
+  if (input_monitor != NULL)
+    {
+      input_monitor->subscribe_statistics(this);
+    }
+  
 #ifdef HAVE_DISTRIBUTION
   init_distribution_manager();
 #endif
@@ -81,7 +106,6 @@ Statistics::init(Core *control)
       start_new_day();
     }
 
-  update_enviromnent();
   load_history();
 }
 
@@ -97,7 +121,7 @@ Statistics::update()
 
   if (state == ACTIVITY_ACTIVE && !been_active)
     {
-      const time_t now = time(NULL);
+      const time_t now = core->get_time();
       struct tm *tmnow = localtime(&now);
 
       current_day->start = *tmnow;
@@ -117,17 +141,15 @@ void
 Statistics::start_new_day()
 {
   TRACE_ENTER("Statistics::start_new_day");
-  const time_t now = time(NULL);
+  const time_t now = core->get_time();
   struct tm *tmnow = localtime(&now);
 
   if (current_day == NULL ||
       tmnow->tm_mday !=  current_day->start.tm_mday ||
       tmnow->tm_mon  !=  current_day->start.tm_mon  ||
-      tmnow->tm_year !=  current_day->start.tm_year ||
-      tmnow->tm_hour !=  current_day->start.tm_hour ||
-      tmnow->tm_min  !=  current_day->start.tm_min)
+      tmnow->tm_year !=  current_day->start.tm_year
+      )
     {
-
       TRACE_MSG("New day");
       if (current_day != NULL)
         {
@@ -471,6 +493,12 @@ Statistics::load(ifstream &infile, bool history)
             }
         }
     }
+
+  if (history && stats != NULL)
+    {
+      add_history(stats);
+    }
+  
   TRACE_EXIT();
 }
 
@@ -655,7 +683,7 @@ Statistics::update_current_day(bool active)
 
       if (active)
         {
-          const time_t now = time(NULL);
+          const time_t now = core->get_time();
           struct tm *tmnow = localtime(&now);
           current_day->stop = *tmnow;
         }
@@ -672,45 +700,10 @@ Statistics::update_current_day(bool active)
         }
 
 
-      // Collect activity monitor stats.
-      IActivityMonitor *monitor = core->get_activity_monitor();
-      assert(monitor != NULL);
-
-      ActivityMonitorStatistics ams;
-      monitor->get_statistics(ams);
-
-      current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT] = ams.total_movement;
-      current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT] = ams.total_click_movement;
-      current_day->misc_stats[STATS_VALUE_TOTAL_MOVEMENT_TIME] = ams.total_movement_time;
-      current_day->misc_stats[STATS_VALUE_TOTAL_CLICKS] = ams.total_clicks;
-      current_day->misc_stats[STATS_VALUE_TOTAL_KEYSTROKES] = ams.total_keystrokes;
     }
 }
 
 
-void
-Statistics::update_enviromnent()
-{
-  if (core != NULL)
-    {
-      // Collect activity monitor stats.
-      IActivityMonitor *monitor = core->get_activity_monitor();
-      assert(monitor != NULL);
-
-      ActivityMonitorStatistics ams;
-
-      ams.total_movement = current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT];
-      ams.total_click_movement = current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT];
-      ams.total_movement_time = current_day->misc_stats[STATS_VALUE_TOTAL_MOVEMENT_TIME];
-      ams.total_clicks = current_day->misc_stats[STATS_VALUE_TOTAL_CLICKS];
-      ams.total_keystrokes = current_day->misc_stats[STATS_VALUE_TOTAL_KEYSTROKES];
-
-      monitor->set_statistics(ams);
-    }
-}
-
-
-
 #ifdef HAVE_DISTRIBUTION
 // Create the monitor based on the specified configuration.
 void
@@ -925,7 +918,6 @@ Statistics::client_message(DistributionClientMessageID id, bool master, const ch
       stats_to_history = false;
     }
 
-  update_enviromnent();
   dump();
 
   TRACE_EXIT();
@@ -952,3 +944,104 @@ Statistics::DailyStatsImpl::starts_before_date(int y, int m, int d)
                       && start.tm_mday < d))));
 }
 
+
+//! Activity is reported by the input monitor.
+void
+Statistics::action_notify()
+{
+}
+
+
+//! Mouse activity is reported by the input monitor.
+void
+Statistics::mouse_notify(int x, int y, int wheel_delta)
+{
+  static const int sensitivity = 3;
+
+  lock.lock();
+
+  if (current_day != NULL)
+    {
+      int delta_x = sensitivity;
+      int delta_y = sensitivity;
+
+      if (prev_x != -1 && prev_y != -1)
+        {
+          delta_x = x - prev_x;
+          delta_y = y - prev_y;
+        }
+      
+      prev_x = x;
+      prev_y = y;
+
+      if (abs(delta_x) >= sensitivity || abs(delta_y) >= sensitivity || wheel_delta != 0)
+        {
+          current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT]  +=
+            int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+
+          GTimeVal now, tv;
+
+          g_get_current_time(&now);
+          tvSUBTIME(tv, now, last_mouse_time);
+
+          if (!tvTIMEEQ0(last_mouse_time) && tv.tv_sec < 1 && tv.tv_sec >= 0 && tv.tv_usec >= 0)
+            {
+              tvADDTIME(current_day->total_mouse_time, current_day->total_mouse_time, tv);
+
+              current_day->misc_stats[STATS_VALUE_TOTAL_MOVEMENT_TIME] =
+                current_day->total_mouse_time.tv_sec;
+            }
+
+          last_mouse_time = now;
+        }
+    }
+  
+  lock.unlock();
+}
+
+
+//! Mouse button activity is reported by the input monitor.
+void
+Statistics::button_notify(int button_mask, bool is_press)
+{
+  (void)button_mask;
+
+  lock.lock();
+  if (current_day != NULL)
+    {
+      if (click_x != -1 && click_y != -1 &&
+          prev_x != -1  && prev_y != -1)
+        {
+          int delta_x = click_x - prev_x;
+          int delta_y = click_y - prev_y;
+
+          current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT] +=
+            int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+        }
+
+      click_x = prev_x;
+      click_y = prev_y;
+
+      if (is_press)
+        {
+          current_day->misc_stats[STATS_VALUE_TOTAL_CLICKS]++;
+        }
+    }
+  lock.unlock();
+}
+
+
+//! Keyboard activity is reported by the input monitor.
+void
+Statistics::keyboard_notify(int key_code, int modifier)
+{
+  (void)key_code;
+  (void)modifier;
+
+  lock.lock();
+  if (current_day != NULL)
+    {
+      current_day->misc_stats[STATS_VALUE_TOTAL_KEYSTROKES]++;
+    }
+  lock.unlock();
+}
