@@ -1,20 +1,23 @@
 // Timer.cc --- break timer
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007 Rob Caelers <robc@krandor.org>
+// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: Timer.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: Timer.cc 1539 2008-06-08 19:48:14Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,18 +29,23 @@ static const char rcsid[] = "$Id: Timer.cc 1298 2007-08-30 14:18:04Z rcaelers $"
 #include <stdio.h>
 #include <math.h>
 
+#include "CoreFactory.hh"
+#include "ICore.hh"
+
 #include "Timer.hh"
 #include "TimePredFactory.hh"
 #include "TimePred.hh"
 #include "TimeSource.hh"
 #include "timeutil.h"
 
+using namespace std;
+using namespace workrave;
 
 //! Constructs a new break timer.
 /*!
  *  \param time_source source of the current time that will be used by the timer.
  */
-Timer::Timer(TimeSource *time_source) :
+Timer::Timer() :
   timer_enabled(false),
   timer_frozen(false),
   activity_state(ACTIVITY_UNKNOWN),
@@ -63,11 +71,11 @@ Timer::Timer(TimeSource *time_source) :
   next_pred_reset_time(0),
   next_limit_time(0),
   total_overdue_time(0),
-  time_source(time_source),
   activity_monitor(NULL),
   activity_sensitive(true),
   insensitive_mode(MODE_IDLE_ON_LIMIT_REACHED)
 {
+  core = CoreFactory::get_core();
 }
 
 
@@ -101,7 +109,7 @@ Timer::enable()
       if (get_elapsed_time() >= limit_interval)
         {
           // Break is overdue, force a snooze.
-          last_limit_time = time_source->get_time();
+          last_limit_time = core->get_time();
           last_limit_elapsed = 0;
           compute_next_limit_time();
         }
@@ -156,7 +164,7 @@ Timer::set_limit_enabled(bool b)
  *
  */
 void
-Timer::set_limit(long limit_time)
+Timer::set_limit(int limit_time)
 {
   limit_interval = limit_time;
 
@@ -189,7 +197,7 @@ Timer::set_auto_reset_enabled(bool b)
  * \param resetTime after this amount of idle time the timer will reset itself.
  */
 void
-Timer::set_auto_reset(long reset_time)
+Timer::set_auto_reset(int reset_time)
 {
   if (reset_time > autoreset_interval)
     {
@@ -355,7 +363,7 @@ Timer::compute_next_predicate_reset_time()
         {
           // The timer did not reach a predicate reset before. Just take
           // the current time as the last reset time...
-          last_pred_reset_time = time_source->get_time();
+          last_pred_reset_time = core->get_time();
         }
 
       autoreset_interval_predicate->set_last(last_pred_reset_time);
@@ -389,14 +397,14 @@ Timer::reset_timer()
   elapsed_time = 0;
   last_limit_time = 0;
   last_limit_elapsed = 0;
-  last_reset_time = time_source->get_time();
+  last_reset_time = core->get_time();
   snooze_inhibited = false;
   snooze_on_active = true;
 
   if (timer_state == STATE_RUNNING)
     {
       // The timer is reset while running, Pretend the timer just started.
-      last_start_time = time_source->get_time();
+      last_start_time = core->get_time();
       last_stop_time = 0;
 
       compute_next_limit_time();
@@ -413,7 +421,7 @@ Timer::reset_timer()
       if (autoreset_enabled && autoreset_interval != 0)
         {
           elapsed_idle_time = autoreset_interval;
-          last_stop_time = time_source->get_time();
+          last_stop_time = core->get_time();
         }
     }
 
@@ -436,7 +444,7 @@ Timer::start_timer()
         {
           TRACE_MSG("!Frozen");
           // Timer is not frozen, so let's start.
-          last_start_time = time_source->get_time();
+          last_start_time = core->get_time();
           elapsed_idle_time = 0;
         }
       else
@@ -445,7 +453,7 @@ Timer::start_timer()
           // Instead, update the elapsed idle time.
           if (last_stop_time != 0)
             {
-              elapsed_idle_time += (time_source->get_time() - last_stop_time);
+              elapsed_idle_time += (core->get_time() - last_stop_time);
             }
           last_start_time = 0;
         }
@@ -476,7 +484,7 @@ Timer::stop_timer()
       TRACE_MSG("last_start_time = " <<  last_start_time);
 
       // Update last stop time.
-      last_stop_time = time_source->get_time();
+      last_stop_time = core->get_time();
 
       // Update elapsed time.
       if (last_start_time != 0)
@@ -518,7 +526,7 @@ Timer::snooze_timer()
       snooze_on_active = true;
 
       next_limit_time = 0;
-      last_limit_time = time_source->get_time();
+      last_limit_time = core->get_time();
       last_limit_elapsed = get_elapsed_time();
       compute_next_limit_time();
 
@@ -547,7 +555,7 @@ Timer::freeze_timer(bool freeze)
           // freeze timer.
           if (last_start_time != 0 && timer_state == STATE_RUNNING)
             {
-              elapsed_time += (time_source->get_time() - last_start_time);
+              elapsed_time += (core->get_time() - last_start_time);
               last_start_time = 0;
             }
         }
@@ -556,7 +564,7 @@ Timer::freeze_timer(bool freeze)
           // defrost timer.
           if (timer_state == STATE_RUNNING)
             {
-              last_start_time = time_source->get_time();
+              last_start_time = core->get_time();
               elapsed_idle_time = 0;
 
               compute_next_limit_time();
@@ -576,7 +584,7 @@ Timer::get_elapsed_idle_time() const
 
   if (timer_enabled && last_stop_time != 0)
     {
-      ret += (time_source->get_time() - last_stop_time);
+      ret += (core->get_time() - last_stop_time);
     }
 
   return ret;
@@ -587,11 +595,14 @@ Timer::get_elapsed_idle_time() const
 time_t
 Timer::get_elapsed_time() const
 {
+  TRACE_ENTER("Timer::get_elapsed_time");
   time_t ret = elapsed_time;
 
+  TRACE_MSG(ret << " " << core->get_time() << " " <<  last_start_time);
+    
   if (timer_enabled && last_start_time != 0)
     {
-      ret += (time_source->get_time() - last_start_time);
+      ret += (core->get_time() - last_start_time);
     }
 
   return ret;
@@ -648,7 +659,6 @@ Timer::shift_time(int delta)
   compute_next_predicate_reset_time();
 }
 
-
 //! Perform timer processing.
 /*! \param new_activity_state the current activity state as reported by the
  *         (global) activity monitor.
@@ -658,7 +668,7 @@ void
 Timer::process(ActivityState new_activity_state, TimerInfo &info)
 {
   TRACE_ENTER_MSG("Timer::Process", timer_id << timer_id << " " << new_activity_state);
-  time_t current_time= time_source->get_time();
+  time_t current_time= core->get_time();
 
   // Default event to return.
   info.event = TIMER_EVENT_NONE;
@@ -669,7 +679,27 @@ Timer::process(ActivityState new_activity_state, TimerInfo &info)
   TRACE_MSG("elap = " << info.elapsed_time);
   TRACE_MSG("enabled = " << timer_enabled);
   TRACE_MSG("last_start_time " << last_start_time);
+  TRACE_MSG("next_pred_reset_time " << next_pred_reset_time);
+  TRACE_MSG("next_reset_time " << next_reset_time);
+  TRACE_MSG("time " << current_time);
+  
+  const time_t now = core->get_time();
+  struct tm *tmnow = localtime(&now);
 
+  TRACE_MSG(tmnow->tm_mday << " "
+            << tmnow->tm_mon << " "
+            << tmnow->tm_year << " "
+            << tmnow->tm_hour << " "
+            << tmnow->tm_min << " "
+            << tmnow->tm_min << " "
+            << tmnow->tm_isdst << " "
+#ifdef PLATFORM_OS_UNIX            
+            << tmnow->tm_gmtoff << " "
+#endif            
+            << tzname[0] << " " 
+            << tzname[1]);
+
+  
   if (activity_sensitive)
     {
       TRACE_MSG("is activity sensitive");
@@ -765,18 +795,24 @@ Timer::process(ActivityState new_activity_state, TimerInfo &info)
       // So reset the timer and send a reset event.
       reset_timer();
 
-      last_pred_reset_time = time_source->get_time();
+      last_pred_reset_time = core->get_time();
       next_pred_reset_time = 0;
 
       compute_next_predicate_reset_time();
       info.event = TIMER_EVENT_RESET;
 
+      if (!activity_sensitive)
+        {
+          activity_state = ACTIVITY_IDLE;
+          TRACE_MSG("pred reset, setting state = IDLE");
+          stop_timer();
+        }
     }
   else if (next_limit_time != 0 && current_time >=  next_limit_time)
     {
       // A next limit time was set and the current time >= limit time.
       next_limit_time = 0;
-      last_limit_time = time_source->get_time();
+      last_limit_time = core->get_time();
       last_limit_elapsed = get_elapsed_time();
 
       snooze_on_active = true;
@@ -849,20 +885,21 @@ Timer::serialize_state() const
   stringstream ss;
 
   ss << timer_id << " "
-     << time_source->get_time() << " "
+     << core->get_time() << " "
      << get_elapsed_time() << " "
      << last_pred_reset_time << " "
      << total_overdue_time << " "
      << snooze_inhibited << " "
      << last_limit_time << " "
-     << last_limit_elapsed;
+     << last_limit_elapsed << " "
+     << timezone;
 
   return ss.str();
 }
 
 
 bool
-Timer::deserialize_state(std::string state)
+Timer::deserialize_state(const std::string &state, int version)
 {
   TRACE_ENTER("Timer::deserialize_state");
   istringstream ss(state);
@@ -871,9 +908,10 @@ Timer::deserialize_state(std::string state)
   time_t elapsed = 0;
   time_t lastReset = 0;
   time_t overdue = 0;
-  time_t now = time_source->get_time();
+  time_t now = core->get_time();
   time_t llt = 0;
   time_t lle = 0;
+  time_t tz = 0;
   bool si = false;
 
   ss >> saveTime
@@ -884,12 +922,37 @@ Timer::deserialize_state(std::string state)
      >> llt
      >> lle;
 
+  if (version == 3)
+    {
+      ss >> tz;
+      tz -= timezone;
+    }
+  
   // Sanity check...
   if (lastReset > saveTime)
     {
       lastReset = saveTime;
     }
 
+  // lastReset -= tz;
+
+  time_t xx = lastReset - tz;
+  struct tm *lt = localtime(&xx);
+
+  TRACE_MSG(lt->tm_mday << " "
+            << lt->tm_mon << " "
+            << lt->tm_year << " "
+            << lt->tm_hour << " "
+            << lt->tm_min << " "
+            << lt->tm_isdst << " "
+#ifdef PLATFORM_OS_UNIX            
+            << lt->tm_gmtoff << " "
+#endif
+            << tzname[0] << " " 
+            << tzname[1] << " " 
+            << timezone << " "
+            << daylight);
+  
   TRACE_MSG(si << " " << llt << " " << lle);
   TRACE_MSG(snooze_inhibited);
 
@@ -926,6 +989,40 @@ Timer::deserialize_state(std::string state)
   return true;
 }
 
+void
+Timer::set_state(int elapsed, int idle, int overdue)
+{
+  TRACE_ENTER("Timer::set_state");
+
+  elapsed_time = elapsed;
+  elapsed_idle_time = idle;
+
+  if (overdue >= 0)
+    {
+      total_overdue_time = overdue;
+    }
+  
+  if (last_start_time != 0)
+    {
+      last_start_time = core->get_time();
+    }
+
+  if (last_stop_time != 0)
+    {
+      last_stop_time = core->get_time();
+    }
+
+  if (elapsed_idle_time > autoreset_interval && autoreset_enabled)
+    {
+      elapsed_idle_time = autoreset_interval;
+    }
+
+  compute_next_reset_time();
+  compute_next_limit_time();
+  compute_next_predicate_reset_time();
+
+  TRACE_EXIT();
+}
 
 void
 Timer::set_values(int elapsed, int idle)
@@ -938,11 +1035,11 @@ Timer::set_values(int elapsed, int idle)
 
   if (timer_state == STATE_RUNNING)
     {
-      last_start_time = time_source->get_time();
+      last_start_time = core->get_time();
     }
   else if (timer_state == STATE_STOPPED)
     {
-      last_stop_time = time_source->get_time();
+      last_stop_time = core->get_time();
     }
 
   compute_next_limit_time();
@@ -953,7 +1050,7 @@ Timer::set_values(int elapsed, int idle)
 void
 Timer::set_state_data(const TimerStateData &data)
 {
-  time_t time_diff =  time_source->get_time() - data.current_time;
+  time_t time_diff =  core->get_time() - data.current_time;
 
   elapsed_time = data.elapsed_time;
   elapsed_idle_time = data.elapsed_idle_time;
@@ -979,11 +1076,11 @@ Timer::set_state_data(const TimerStateData &data)
 
   if (timer_state == STATE_RUNNING)
     {
-      last_start_time = time_source->get_time();
+      last_start_time = core->get_time();
     }
   else if (timer_state == STATE_STOPPED)
     {
-      last_stop_time = time_source->get_time();
+      last_stop_time = core->get_time();
     }
 
   compute_next_limit_time();
@@ -996,7 +1093,7 @@ void
 Timer::get_state_data(TimerStateData &data)
 {
   TRACE_ENTER("Timer::get_state_data");
-  data.current_time = time_source->get_time();
+  data.current_time = core->get_time();
 
   data.elapsed_time = get_elapsed_time();
   data.elapsed_idle_time = get_elapsed_idle_time();
