@@ -3,16 +3,19 @@
 // Copyright (C) 2007 Ray Satiro <raysatiro@yahoo.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 // $Id$
 //
 // This file contains code necessary to bypass mouse & keyboard
@@ -24,7 +27,11 @@
 // jay satiro, workrave project, may 2007
 //
 
-#if defined(WIN32)
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#if defined(PLATFORM_OS_WIN32)
 
 #include "debug.hh"
 
@@ -33,46 +40,35 @@
 
 #include "W32AlternateMonitor.hh"
 #include "Harpoon.hh"
+#include "CoreFactory.hh"
+#include "IConfigurator.hh"
 
-volatile IInputMonitorListener *W32AlternateMonitor::listener = NULL;
+using namespace workrave;
 
-W32AlternateMonitor::W32AlternateMonitor()
+W32AlternateMonitor::W32AlternateMonitor() : 
+  thread_id( 0 )
 {
   TRACE_ENTER( "W32AlternateMonitor::W32AlternateMonitor" );
   
-  listener = NULL;
+  CoreFactory::get_configurator()->get_value_with_default( "advanced/interval", interval, 500);
   
-  CoreFactory::get_configurator()->get_value_default( "advanced/interval", &interval, 500);
   TRACE_EXIT();
 }
 
 W32AlternateMonitor::~W32AlternateMonitor()
 {
   TRACE_ENTER( "W32AlternateMonitor::~W32AlternateMonitor" );
+  
+  terminate();
+  
   TRACE_EXIT();
 }
 
 
-bool W32AlternateMonitor::init( IInputMonitorListener *l )
+bool W32AlternateMonitor::init()
 {
   TRACE_ENTER( "W32AlternateMonitor::init" );
   
-  DWORD id;
-  HANDLE hThread;
-
-  if( listener != NULL )
-  /*
-  Calling terminate (listener = NULL), will cause the
-  monitor thread to exit. This shouldn't come up though.
-  */
-  {
-    TRACE_MSG( " listener != NULL " );
-    msg( " W32AlternateMonitor::init : listener != NULL " );
-    terminate();
-    Sleep( interval * 2 + 1000 ); // wait for thread to terminate
-  }
-  listener = l;
-  
   /* Try to get the address of GetLastInputInfo()... */
   GetLastInputInfo = ( BOOL ( WINAPI * ) ( LASTINPUTINFO * ) )
     GetProcAddress( GetModuleHandleA( "user32.dll" ), "GetLastInputInfo" );
@@ -84,38 +80,47 @@ bool W32AlternateMonitor::init( IInputMonitorListener *l )
     GetVersion(), so it shouldn't come to this.
   */
   {
-    //msg( "GetLastInputInfo() address not found in user32.dll" );
     TRACE_MSG( "GetLastInputInfo() address not found in user32.dll" );
     TRACE_EXIT();
     return false;
   }
   
-  hThread = CreateThread( NULL, 0, thread_Monitor, this, 0, &id );
+  thread_id = 0;
+  SetLastError( 0 );
+  thread_handle = CreateThread( NULL, 0, thread_Monitor, this, 0, (DWORD *)&thread_id );
   
-  if( hThread == NULL )
+  if( thread_handle == NULL || thread_id == 0 )
   {
-    //msg( "W32AlternateMonitor::init() : Thread could not be created.\n" );
     TRACE_MSG( "Thread could not be created. GetLastError : " << GetLastError() );
     TRACE_EXIT();
     return false;
   }
   
   /* Close the handle now, we don't need it anymore. */
-  CloseHandle( hThread );
+  CloseHandle( thread_handle );
+  thread_handle = NULL;
+  
+  Harpoon::init( NULL );
   
   TRACE_EXIT();
-
-  Harpoon::init(NULL);
   return true;
 }
 
 void W32AlternateMonitor::terminate()
 {
   TRACE_ENTER( "W32AlternateMonitor::terminate" );
-  listener = NULL;
+  
+  if( thread_id )
+    {
+	  TRACE_MSG( "Terminating thread id: " << thread_id );
+      PostThreadMessage( thread_id, WM_QUIT, 0, 0 );
+      thread_id = 0;
+    }
+  
   TRACE_EXIT();
 }
 
+
 DWORD WINAPI W32AlternateMonitor::thread_Monitor( LPVOID lpParam )
 {
   W32AlternateMonitor *pThis = (W32AlternateMonitor *) lpParam;
@@ -125,64 +130,38 @@ DWORD WINAPI W32AlternateMonitor::thread_Monitor( LPVOID lpParam )
 
 void W32AlternateMonitor::Monitor()
 {
-  TRACE_ENTER( "W32AlternateMonitor::Monitor" );
+  const DWORD current_thread_id = GetCurrentThreadId();
   
-  LASTINPUTINFO lii;
-  DWORD dwPreviousTime;
+  TRACE_ENTER_MSG( "W32AlternateMonitor::Monitor [ id: ", current_thread_id << " ]" );
+  
+  assert( GetLastInputInfo );
   
   SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL );
   
-  lii.cbSize = sizeof( lii ); 
   
-  Update( &lii );
+  LASTINPUTINFO lii;
+  DWORD dwPreviousTime = 0;
   
+  lii.cbSize = sizeof( lii );
+  lii.dwTime = GetTickCount();
   
-  while( listener != NULL )
+  while( thread_id == current_thread_id )
   /* Main loop */
   {
     dwPreviousTime = lii.dwTime;
-    Update( &lii );
     
-    if( lii.dwTime > dwPreviousTime )
+    if( ( *GetLastInputInfo )( &lii ) && lii.dwTime > dwPreviousTime )
     /* User session has received input */
     {
       /* Notify the activity monitor */
-       const_cast<IInputMonitorListener *>( listener )->action_notify();
+      fire_action();
     }
     
     Sleep( interval );
   }
   
-  TRACE_EXIT();
-}
-
-
-inline void W32AlternateMonitor::Update( LASTINPUTINFO *p )
-{
-  while( ( *GetLastInputInfo ) ( p ) == 0 && listener != NULL )
-  /*
-  If GetLastInputInfo errors out, sleep & try again.
-  When will this ever happen though?
-  */
-  {
-          // TRACE_MSG( "GetLastInputInfo() failed." );
-    Sleep( interval + rand() % 1000 );
-  }
-}
-
-void W32AlternateMonitor::msg( char *msg )
-{
-  MessageBoxA( NULL, msg, "Workrave: Debug Message", MB_OK );
-}
-
-void W32AlternateMonitor::exitmsg( char *msg )
-{
-  TRACE_ENTER( "W32AlternateMonitor::exitmsg" );
-  TRACE_MSG( msg );
-  MessageBoxA( NULL, msg, "Workrave: Unrecoverable Error. Exiting...", MB_OK );
   
   TRACE_EXIT();
-  exit( 0 );
 }
 
-#endif // defined(WIN32)
+#endif // defined(PLATFORM_OS_WIN32)
