@@ -1,20 +1,23 @@
 // W32Configurator.cc --- Configuration Access
 //
-// Copyright (C) 2002, 2005, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: W32Configurator.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: W32Configurator.cc 1351 2007-10-14 20:56:54Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,6 +29,8 @@ static const char rcsid[] = "$Id: W32Configurator.cc 1298 2007-08-30 14:18:04Z r
 #include "debug.hh"
 #include "W32Configurator.hh"
 
+using namespace std;
+
 W32Configurator::W32Configurator()
 {
   key_root = "Software/Workrave";
@@ -61,7 +66,39 @@ W32Configurator::save()
 
 
 bool
-W32Configurator::get_value(string key, string *out) const
+W32Configurator::remove_key(const std::string &key)
+{
+  TRACE_ENTER_MSG("W32Configurator::remove_key", key);
+
+  HKEY handle;
+  bool rc = false;
+  string k, p, p32, c;
+  LONG err;
+
+  k = key_add_part(key_root, key);
+  key_split(k, p, c);
+  p32 = key_win32ify(p);
+  err = RegOpenKeyEx(HKEY_CURRENT_USER, p32.c_str(), 0, KEY_ALL_ACCESS, &handle);
+  if (err == ERROR_SUCCESS)
+    {
+      DWORD size;
+      char buf[256]; // FIXME: yuck, should be dynamic.
+      size = sizeof(buf);
+      err = RegDeleteKey(handle, c.c_str());
+      if (err == ERROR_SUCCESS)
+        {
+          rc = true;
+        }
+      RegCloseKey(handle);
+    }
+
+  TRACE_EXIT();
+  return rc;
+}
+
+
+bool
+W32Configurator::get_value(const string &key, string &out) const
 {
   TRACE_ENTER_MSG("W32Configurator::get_value", key << "," << out);
 
@@ -102,7 +139,7 @@ W32Configurator::get_value(string key, string *out) const
 
       if ( err == ERROR_SUCCESS && type == REG_SZ )
         {
-          *out = buffer;
+          out = buffer;
           rc = true;
         }
 
@@ -116,13 +153,13 @@ W32Configurator::get_value(string key, string *out) const
 
 
 bool
-W32Configurator::get_value(string key, bool *out) const
+W32Configurator::get_value(const string &key, bool &out) const
 {
   long l;
-  bool rc = get_value(key, &l);
+  bool rc = get_value(key, l);
   if (rc)
     {
-      *out = (bool) l;
+      out = (bool) l;
     }
   return rc;
 }
@@ -134,13 +171,13 @@ W32Configurator::get_value(string key, bool *out) const
  *  \retval false attribute not found.
  */
 bool
-W32Configurator::get_value(string key, int *out) const
+W32Configurator::get_value(const string &key, int &out) const
 {
   long l;
-  bool rc = get_value(key, &l);
+  bool rc = get_value(key, l);
   if (rc)
     {
-      *out = (int) l;
+      out = (int) l;
     }
   return rc;
 }
@@ -152,13 +189,13 @@ W32Configurator::get_value(string key, int *out) const
  *  \retval false attribute not found.
  */
 bool
-W32Configurator::get_value(string key, long *out) const
+W32Configurator::get_value(const string &key, long &out) const
 {
   string s;
-  bool rc = get_value(key, &s);
+  bool rc = get_value(key, s);
   if (rc)
     {
-      int f = sscanf(s.c_str(), "%ld", out);
+      int f = sscanf(s.c_str(), "%ld", &out);
       rc = (f == 1);
     }
   return rc;
@@ -171,13 +208,13 @@ W32Configurator::get_value(string key, long *out) const
  *  \retval false attribute not found.
  */
 bool
-W32Configurator::get_value(string key, double *out) const
+W32Configurator::get_value(const string &key, double &out) const
 {
   string s;
-  bool rc = get_value(key, &s);
+  bool rc = get_value(key, s);
   if (rc)
     {
-      int f = sscanf(s.c_str(), "%lf", out);
+      int f = sscanf(s.c_str(), "%lf", &out);
       rc = (f == 1);
     }
   return rc;
@@ -185,7 +222,7 @@ W32Configurator::get_value(string key, double *out) const
 
 
 bool
-W32Configurator::set_value(string key, string v)
+W32Configurator::set_value(const string &key, string v)
 {
   TRACE_ENTER_MSG("W32Configurator::set_value", key << "," << v);
 
@@ -206,10 +243,6 @@ W32Configurator::set_value(string key, string v)
       err = RegSetValueEx(handle, c.c_str(), 0, REG_SZ, (BYTE *) v.c_str(), v.length()+1);
       RegCloseKey(handle);
       rc = (err == ERROR_SUCCESS);
-      if (rc)
-  {
-    fire_configurator_event(key);
-  }
     }
   TRACE_EXIT();
   return rc;
@@ -219,7 +252,7 @@ W32Configurator::set_value(string key, string v)
 
 
 bool
-W32Configurator::set_value(string key, int v)
+W32Configurator::set_value(const string &key, int v)
 {
   char buf[32];
   sprintf(buf, "%d", v);
@@ -227,7 +260,7 @@ W32Configurator::set_value(string key, int v)
 }
 
 bool
-W32Configurator::set_value(string key, long v)
+W32Configurator::set_value(const string &key, long v)
 {
   char buf[32];
   sprintf(buf, "%ld", v);
@@ -235,7 +268,7 @@ W32Configurator::set_value(string key, long v)
 }
 
 bool
-W32Configurator::set_value(string key, bool v)
+W32Configurator::set_value(const string &key, bool v)
 {
   char buf[32];
   sprintf(buf, "%d", v ? 1 : 0);
@@ -244,7 +277,7 @@ W32Configurator::set_value(string key, bool v)
 
 
 bool
-W32Configurator::set_value(string key, double v)
+W32Configurator::set_value(const string &key, double v)
 {
   char buf[32];
   sprintf(buf, "%f", v);
@@ -262,7 +295,7 @@ W32Configurator::key_add_part(string s, string t) const
 }
 
 void
-W32Configurator::key_split(string key, string &parent, string &child) const
+W32Configurator::key_split(const string &key, string &parent, string &child) const
 {
   const char *s = key.c_str();
   char *slash = strrchr(s, '/');
@@ -279,7 +312,7 @@ W32Configurator::key_split(string key, string &parent, string &child) const
 }
 
 string
-W32Configurator::key_win32ify(string key) const
+W32Configurator::key_win32ify(const string &key) const
 {
   string rc = key;
   strip_trailing_slash(rc);
@@ -294,3 +327,31 @@ W32Configurator::key_win32ify(string key) const
 }
 
 
+//! Removes the trailing '/'.
+void
+W32Configurator::strip_trailing_slash(string &key) const
+{
+  int len = key.length();
+  if (len > 0)
+    {
+      if (key[len - 1] == '/')
+        {
+          key = key.substr(0, len - 1);
+        }
+    }
+}
+
+
+//! Adds add trailing '/' if it isn't there yet.
+void
+W32Configurator::add_trailing_slash(string &key) const
+{
+  int len = key.length();
+  if (len > 0)
+    {
+      if (key[len - 1] != '/')
+        {
+          key += '/';
+        }
+    }
+}
