@@ -3,18 +3,21 @@
 // Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: W32InputMonitor.cc 1317 2007-09-08 18:18:06Z rcaelers $";
+static const char rcsid[] = "$Id: W32InputMonitor.cc 1356 2007-10-22 18:22:13Z rcaelers $";
 
 #include <assert.h>
 #ifdef HAVE_CONFIG_H
@@ -27,7 +30,6 @@ static const char rcsid[] = "$Id: W32InputMonitor.cc 1317 2007-09-08 18:18:06Z r
 #include <winuser.h>
 #include "debug.hh"
 #include "W32InputMonitor.hh"
-#include "IInputMonitorListener.hh"
 
 #include "ICore.hh"
 #include "CoreFactory.hh"
@@ -50,7 +52,7 @@ typedef struct {
     DWORD mouseData;
 } MOUSEHOOKSTRUCTEX, *PMOUSEHOOKSTRUCTEX;
 
-IInputMonitorListener *W32InputMonitor::listener = NULL;
+W32InputMonitor *W32InputMonitor::singleton = NULL;
 
 W32InputMonitor::W32InputMonitor()
 {
@@ -63,12 +65,17 @@ W32InputMonitor::~W32InputMonitor()
 }
 
 bool
-W32InputMonitor::init( IInputMonitorListener *l )
+W32InputMonitor::init()
 {
-  assert(! listener);
-  listener = l;
-
-  return Harpoon::init(on_harpoon_event);
+  if (singleton == NULL)
+    {
+      singleton = this;
+      return Harpoon::init(on_harpoon_event);
+    }
+  else
+    {
+      return false;
+    }
 }
 
 
@@ -77,8 +84,6 @@ void
 W32InputMonitor::terminate()
 {
   Harpoon::terminate();
-
-  listener = NULL;
 }
 
 
@@ -88,29 +93,29 @@ W32InputMonitor::on_harpoon_event(HarpoonEvent *event)
   switch (event->type)
     {
     case HARPOON_BUTTON_PRESS:
-      listener->button_notify(0, true); // FIXME: proper parameter
+      singleton->fire_button(0, true); // FIXME: proper parameter
       break;
 
     case HARPOON_BUTTON_RELEASE:
-      listener->button_notify(0, false); // FIXME: proper parameter
+      singleton->fire_button(0, false); // FIXME: proper parameter
       break;
 
     case HARPOON_2BUTTON_PRESS:
     case HARPOON_KEY_PRESS:
-      listener->action_notify();
+      singleton->fire_action();
       break;
 
     case HARPOON_MOUSE_WHEEL:
-      listener->mouse_notify(event->mouse.x, event->mouse.y,
-                             event->mouse.wheel);
+      singleton->fire_mouse(event->mouse.x, event->mouse.y,
+                            event->mouse.wheel);
       break;
 
     case HARPOON_KEY_RELEASE:
-      listener->keyboard_notify(0, 0);
+      singleton->fire_keyboard(0, 0);
       break;
 
     case HARPOON_MOUSE_MOVE:
-      listener->mouse_notify(event->mouse.x, event->mouse.y, 0);
+      singleton->fire_mouse(event->mouse.x, event->mouse.y, 0);
       break;
 
     default:
