@@ -1,20 +1,23 @@
 // Util.cc --- General purpose utility functions
 //
-// Copyright (C) 2001, 2002, 2003, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: Util.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: Util.cc 1553 2008-06-29 11:20:41Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -35,7 +38,7 @@ static const char rcsid[] = "$Id: Util.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 #include <sys/types.h>
 #endif
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 #include <windows.h>
 // HACK: #include <shlobj.h>, need -fvtable-thunks.
 // Perhaps we should enable this, but let's hack it for now...
@@ -52,6 +55,11 @@ BOOL WINAPI PathCanonicalize(LPSTR,LPCSTR);
 // (end of hack)
 #endif
 
+#ifdef PLATFORM_OS_OSX
+#include <mach-o/dyld.h>
+#include <sys/param.h>
+#endif
+
 #include "Util.hh"
 
 #include <glib.h>
@@ -75,7 +83,7 @@ Util::get_home_directory()
       // Default to current directory
       ret = "./";
 
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX) || defined(PLATFORM_OS_OSX)
       const char *home = getenv("WORKRAVE_HOME");
 
       if (home == NULL)
@@ -90,7 +98,7 @@ Util::get_home_directory()
 
           mkdir(ret.c_str(), 0700);
         }
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
       void *pidl;
       char buf[MAX_PATH];
       HRESULT hr = SHGetSpecialFolderLocation(HWND_DESKTOP,
@@ -127,7 +135,7 @@ Util::get_home_directory()
 void
 Util::set_home_directory(const string &home)
 {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   if (home.substr(0, 2) == ".\\" ||
       home.substr(0, 3) == "..\\")
     {
@@ -147,7 +155,7 @@ Util::set_home_directory(const string &home)
       home_directory = home + "/";
     }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   CreateDirectory(home_directory.c_str(), NULL);
 #else
   mkdir(home_directory.c_str(), 0777);
@@ -173,7 +181,7 @@ Util::file_exists(string path)
 }
 
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 //! Returns the directory in which workrave is installed.
 string
 Util::get_application_directory()
@@ -191,6 +199,59 @@ Util::get_application_directory()
   // app_dir_name == c:\program files\workrave
   return string(app_dir_name);
 }
+
+bool
+Util::registry_get_value(const char *path, const char *name,
+                         char *out)
+{
+  HKEY handle;
+  bool rc = false;
+  LONG err;
+
+  err = RegOpenKeyEx(HKEY_CURRENT_USER, path, 0, KEY_ALL_ACCESS, &handle);
+  if (err == ERROR_SUCCESS)
+    {
+      DWORD type, size;
+      size = MAX_PATH;
+      err = RegQueryValueEx(handle, name, 0, &type, (LPBYTE) out, &size);
+      if (err == ERROR_SUCCESS)
+        {
+          rc = true;
+        }
+      RegCloseKey(handle);
+    }
+  return rc;
+}
+
+bool
+Util::registry_set_value(const char *path, const char *name,
+                         const char *value)
+{
+  HKEY handle;
+  bool rc = false;
+  DWORD disp;
+  LONG err;
+
+  err = RegCreateKeyEx(HKEY_CURRENT_USER, path, 0,
+                       "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
+                       NULL, &handle, &disp);
+  if (err == ERROR_SUCCESS)
+    {
+      if (value != NULL)
+        {
+          err = RegSetValueEx(handle, name, 0, REG_SZ, (BYTE *) value,
+                              strlen(value)+1);
+        }
+      else
+        {
+          err = RegDeleteValue(handle, name);
+        }
+      RegCloseKey(handle);
+      rc = (err == ERROR_SUCCESS);
+    }
+  return rc;
+}
+
 #endif
 
 
@@ -204,8 +265,17 @@ Util::get_search_path(SearchPathId type)
   list<string> &searchPath = search_paths[type];
 
   string home_dir = get_home_directory();
-#if defined(WIN32)
+#if defined(PLATFORM_OS_WIN32)
   string app_dir = get_application_directory();
+#elif defined(PLATFORM_OS_OSX)
+  char execpath[MAXPATHLEN+1];
+  uint32_t pathsz = sizeof (execpath);
+      
+  _NSGetExecutablePath (execpath, &pathsz);
+
+  gchar *dir_path = g_path_get_dirname(execpath);
+  string app_dir = dir_path;
+  g_free(dir_path);
 #endif
 
   char *cwd = g_get_current_dir();
@@ -217,41 +287,73 @@ Util::get_search_path(SearchPathId type)
 
   if (type == SEARCH_PATH_IMAGES)
     {
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
       if (home_dir != "./")
         {
           searchPath.push_back(home_dir + "/");
           searchPath.push_back(home_dir + "/images");
         }
-      searchPath.push_back(string(WORKRAVE_DATADIR) + "/images");
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/images");
       searchPath.push_back("/usr/local/share/workrave/images");
       searchPath.push_back("/usr/share/workrave/images");
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
       searchPath.push_back(app_dir + "\\share\\images");
+#elif defined(PLATFORM_OS_OSX)
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/images");
+      searchPath.push_back(app_dir + "/share/workrave/images");
+      searchPath.push_back(app_dir +  "/../Resources/images");
+#endif
+    }
+  if (type == SEARCH_PATH_SOUNDS)
+    {
+#if defined(PLATFORM_OS_UNIX)
+      if (home_dir != "./")
+        {
+          searchPath.push_back(home_dir + "/");
+          searchPath.push_back(home_dir + "/sounds");
+        }
+      searchPath.push_back(string(WORKRAVE_DATADIR) + "/sounds/workrave");
+      searchPath.push_back("/usr/local/share/sounds/workrave");
+      searchPath.push_back("/usr/share/sounds/workrave");
+#elif defined(PLATFORM_OS_WIN32)
+      searchPath.push_back(app_dir + "\\share\\sounds");
+#elif defined(PLATFORM_OS_OSX)
+      searchPath.push_back(string(WORKRAVE_DATADIR) + "/sounds/workrave");
+      searchPath.push_back(app_dir + "/share/sounds/workrave");
+      searchPath.push_back(app_dir +  "/../Resources/sounds");
 #endif
     }
   else if (type == SEARCH_PATH_CONFIG)
     {
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
       if (home_dir != "./")
         {
           searchPath.push_back(home_dir + "/");
           searchPath.push_back(home_dir + "/etc");
         }
-      searchPath.push_back(string(WORKRAVE_DATADIR) + "/etc");
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/etc");
       searchPath.push_back("/usr/local/share/workrave/etc");
       searchPath.push_back("/usr/share/workrave/etc");
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
       searchPath.push_back(home_dir + "\\");
       searchPath.push_back(app_dir + "\\etc");
+#elif defined(PLATFORM_OS_OSX)
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/etc");
+      searchPath.push_back(app_dir + "/etc");
+      searchPath.push_back(home_dir + "/");
+      searchPath.push_back(app_dir +  "/../Resources/config");
 #endif
     }
   else if (type == SEARCH_PATH_EXERCISES)
     {
-#if defined(HAVE_X)
-      searchPath.push_back(string(WORKRAVE_DATADIR) + "/exercises");
-#elif defined(WIN32)
+#if defined(PLATFORM_OS_UNIX)
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/exercises");
+#elif defined(PLATFORM_OS_WIN32)
       searchPath.push_back(app_dir + "\\share\\exercises");
+#elif defined(PLATFORM_OS_OSX)
+      searchPath.push_back(string(WORKRAVE_PKGDATADIR) + "/exercises");
+      searchPath.push_back(app_dir + "/share/exercises");
+      searchPath.push_back(app_dir +  "/../Resources/exercises");
 #else
 #error Not properly ported.
 #endif
@@ -272,7 +374,7 @@ Util::complete_directory(string path, Util::SearchPathId type)
 
   for (list<string>::const_iterator i = searchPath.begin(); !found && i != searchPath.end(); i++)
     {
-      fullPath = (*i) + "/" + path;
+      fullPath = (*i) + G_DIR_SEPARATOR_S + path;
       found = file_exists(fullPath);
     }
 
@@ -283,3 +385,23 @@ Util::complete_directory(string path, Util::SearchPathId type)
 
   return fullPath;
 }
+
+bool
+Util::running_gnome()
+{
+  bool ret = false;
+
+#ifdef PLATFORM_OS_UNIX
+	gchar *tmp = g_find_program_in_path("gnome-open");
+
+	if (tmp != NULL)
+    {
+      g_free(tmp);
+      tmp = (gchar *)g_getenv("GNOME_DESKTOP_SESSION_ID");
+
+      ret = ((tmp != NULL) && (*tmp != '\0'));
+    }
+#endif
+	return ret;
+}
+
