@@ -1,21 +1,24 @@
 /*
  * harpoon.c
  *
- * Copyright (C) 2002-2007 Raymond Penners <raymond@dotsphinx.com>
+ * Copyright (C) 2002-2008 Raymond Penners <raymond@dotsphinx.com>
  * Copyright (C) 2007 Ray Satiro <raysatiro@yahoo.com>
  * All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * $Id: harpoon.c 1322 2007-09-08 23:19:30Z rcaelers $
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * $Id: harpoon.c 1453 2008-02-11 20:39:45Z rcaelers $
  */
 
 #include <windows.h>
@@ -260,8 +263,15 @@ harpoon_window_proc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
             case HARPOON_2BUTTON_PRESS:
             case HARPOON_MOUSE_WHEEL:
             case HARPOON_MOUSE_MOVE:
-              evt.mouse.x = LOWORD(lParam);
-              evt.mouse.y = HIWORD(lParam);
+       /*
+       The x and y mouse coordinates are packed into lParam.
+       Here we separate x and y. It's important to cast as 
+       signed, because the coordinate(s) could be negative.
+       Casting to the signed type allows the compiler to 
+       properly promote the signed short to a signed int.
+       */
+              evt.mouse.x = (short)LOWORD( lParam );
+              evt.mouse.y = (short)HIWORD( lParam );
               if (evt.type == HARPOON_MOUSE_WHEEL)
                 {
                   evt.mouse.button = -1;
@@ -306,78 +316,145 @@ harpoon_mouse_hook (int code, WPARAM wpar, LPARAM lpar)
       int button = -1;
       int x = pmhs->pt.x;
       int y = pmhs->pt.y;
-
+      
+      // If WH_MOUSE_LL is hooked, 
+      // WH_MOUSE messages are not appended to the debug window.
+      // This is mainly to avoid overflow.
       switch (wpar)
         {
-        case WM_LBUTTONDOWN:
-          button = 0;
-          evt = HARPOON_BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_LBUTTONDOWN" );
-          break;
-        case WM_MBUTTONDOWN:
-          button = 1;
-          evt = HARPOON_BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_MBUTTONDOWN" );
-          break;
-        case WM_RBUTTONDOWN:
-          button = 2;
-          evt = HARPOON_BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_RBUTTONDOWN" );
-          break;
-
-        case WM_LBUTTONUP:
-          button = 0;
-          evt = HARPOON_BUTTON_RELEASE;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_LBUTTONUP" );
-          break;
-        case WM_MBUTTONUP:
-          button = 1;
-          evt = HARPOON_BUTTON_RELEASE;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_MBUTTONUP" );
-          break;
-        case WM_RBUTTONUP:
-          button = 2;
-          evt = HARPOON_BUTTON_RELEASE;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_RBUTTONUP" );
-          break;
-
-        case WM_LBUTTONDBLCLK:
-          button = 0;
-          evt = HARPOON_2BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_LBUTTONDBLCLK" );
-          break;
-        case WM_MBUTTONDBLCLK:
-          button = 1;
-          evt = HARPOON_2BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_MBUTTONDBLCLK" );
-          break;
-        case WM_RBUTTONDBLCLK:
-          button = 2;
-          evt = HARPOON_2BUTTON_PRESS;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_RBUTTONDBLCLK" );
-          break;
-
-        case WM_MOUSEWHEEL:
-          evt = HARPOON_MOUSE_WHEEL;
-          button = 1;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: WM_MOUSEWHEEL" );
-          break;
-
-        default:
-          evt = HARPOON_MOUSE_MOVE;
-          if( debug && !mouse_ll_hook )
-              debug_send_message( "WH_MOUSE: HARPOON_MOUSE_MOVE" );
-        }
+          case WM_NCLBUTTONDOWN:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCLBUTTONDOWN" );
+          case WM_LBUTTONDOWN:
+              button = 0;
+              evt = HARPOON_BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_LBUTTONDOWN )
+                  debug_send_message( "WH_MOUSE: WM_LBUTTONDOWN" );
+              break;
+          case WM_NCMBUTTONDOWN:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCMBUTTONDOWN" );
+          case WM_MBUTTONDOWN:
+              button = 1;
+              evt = HARPOON_BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_MBUTTONDOWN )
+                  debug_send_message( "WH_MOUSE: WM_MBUTTONDOWN" );
+              break;
+          case WM_NCRBUTTONDOWN:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCRBUTTONDOWN" );
+          case WM_RBUTTONDOWN:
+              button = 2;
+              evt = HARPOON_BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_RBUTTONDOWN )
+                  debug_send_message( "WH_MOUSE: WM_RBUTTONDOWN" );
+              break;
+          case WM_NCXBUTTONDOWN:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCXBUTTONDOWN" );
+          case WM_XBUTTONDOWN:
+              button = ( HIWORD( wpar ) == XBUTTON1 ) ? 3 : 4;
+              evt = HARPOON_BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_XBUTTONDOWN )
+                  debug_send_message( "WH_MOUSE: WM_XBUTTONDOWN" );
+              break;
+          
+          case WM_NCLBUTTONUP:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCLBUTTONUP" );
+          case WM_LBUTTONUP:
+              button = 0;
+              evt = HARPOON_BUTTON_RELEASE;
+              if( debug && !mouse_ll_hook && wpar == WM_LBUTTONUP )
+                  debug_send_message( "WH_MOUSE: WM_LBUTTONUP" );
+              break;
+          case WM_NCMBUTTONUP:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCMBUTTONUP" );
+          case WM_MBUTTONUP:
+              button = 1;
+              evt = HARPOON_BUTTON_RELEASE;
+              if( debug && !mouse_ll_hook && wpar == WM_MBUTTONUP )
+                  debug_send_message( "WH_MOUSE: WM_MBUTTONUP" );
+              break;
+          case WM_NCRBUTTONUP:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCRBUTTONUP" );
+          case WM_RBUTTONUP:
+              button = 2;
+              evt = HARPOON_BUTTON_RELEASE;
+              if( debug && !mouse_ll_hook && wpar == WM_RBUTTONUP )
+                  debug_send_message( "WH_MOUSE: WM_RBUTTONUP" );
+              break;
+          case WM_NCXBUTTONUP:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCXBUTTONUP" );
+          case WM_XBUTTONUP:
+              button = ( HIWORD( wpar ) == XBUTTON1 ) ? 3 : 4;
+              evt = HARPOON_BUTTON_RELEASE;
+              if( debug && !mouse_ll_hook && wpar == WM_XBUTTONUP )
+                  debug_send_message( "WH_MOUSE: WM_XBUTTONUP" );
+              break;
+          
+          case WM_NCLBUTTONDBLCLK:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCLBUTTONDBLCLK" );
+          case WM_LBUTTONDBLCLK:
+              button = 0;
+              evt = HARPOON_2BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_LBUTTONDBLCLK )
+                  debug_send_message( "WH_MOUSE: WM_LBUTTONDBLCLK" );
+              break;
+          case WM_NCMBUTTONDBLCLK:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCMBUTTONDBLCLK" );
+          case WM_MBUTTONDBLCLK:
+              button = 1;
+              evt = HARPOON_2BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_MBUTTONDBLCLK )
+                  debug_send_message( "WH_MOUSE: WM_MBUTTONDBLCLK" );
+              break;
+          case WM_NCRBUTTONDBLCLK:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCRBUTTONDBLCLK" );
+          case WM_RBUTTONDBLCLK:
+              button = 2;
+              evt = HARPOON_2BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_RBUTTONDBLCLK )
+                  debug_send_message( "WH_MOUSE: WM_RBUTTONDBLCLK" );
+              break;
+          case WM_NCXBUTTONDBLCLK:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCXBUTTONDBLCLK" );
+          case WM_XBUTTONDBLCLK:
+              button = ( HIWORD( wpar ) == XBUTTON1 ) ? 3 : 4;
+              evt = HARPOON_2BUTTON_PRESS;
+              if( debug && !mouse_ll_hook && wpar == WM_XBUTTONDBLCLK )
+                  debug_send_message( "WH_MOUSE: WM_XBUTTONDBLCLK" );
+              break;
+          
+          case WM_MOUSEWHEEL:
+              evt = HARPOON_MOUSE_WHEEL;
+              button = 1;
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_MOUSEWHEEL" );
+              break;
+          case WM_MOUSEHWHEEL:
+              evt = HARPOON_MOUSE_WHEEL;
+              button = 2;
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_MOUSEHWHEEL" );
+              break;
+          
+          case WM_NCMOUSEMOVE:
+              if( debug && !mouse_ll_hook )
+                  debug_send_message( "WH_MOUSE: WM_NCMOUSEMOVE" );
+          case WM_MOUSEMOVE:
+              evt = HARPOON_MOUSE_MOVE;
+              if( debug && !mouse_ll_hook && wpar == WM_MOUSEMOVE )
+                  debug_send_message( "WH_MOUSE: WM_MOUSEMOVE" );
+            }
+      
       /*
    The low-level mouse hook is always preferred over the
    regular mouse hook. The low-level hook posts its own
@@ -385,9 +462,9 @@ harpoon_mouse_hook (int code, WPARAM wpar, LPARAM lpar)
    see if there is a low-level hook. If not, we post our
    own message.
    */
-      if( !mouse_ll_hook )
+      if( !mouse_ll_hook && evt != HARPOON_NOTHING )
           harpoon_post_message (evt, button, MAKELONG(x, y));
-  
+      
       if (evt == HARPOON_BUTTON_RELEASE)
         {
           forcecallnext = TRUE;
@@ -407,68 +484,69 @@ harpoon_mouse_ll_hook (int code, WPARAM wpar, LPARAM lpar)
       int button = -1;
       int x = pmhs->pt.x;
       int y = pmhs->pt.y;
-
+      
       switch (wpar)
         {
-        case WM_LBUTTONDOWN:
-          button = 0;
-          evt = HARPOON_BUTTON_PRESS;
-          if_debug_send_message( "WH_MOUSE_LL: WM_LBUTTONDOWN" );
-          break;
-        case WM_MBUTTONDOWN:
-          button = 1;
-          evt = HARPOON_BUTTON_PRESS;
-          if_debug_send_message( "WH_MOUSE_LL: WM_MBUTTONDOWN" );
-          break;
-        case WM_RBUTTONDOWN:
-          button = 2;
-          evt = HARPOON_BUTTON_PRESS;
-          if_debug_send_message( "WH_MOUSE_LL: WM_RBUTTONDOWN" );
-          break;
-
-        case WM_LBUTTONUP:
-          button = 0;
-          evt = HARPOON_BUTTON_RELEASE;
-          if_debug_send_message( "WH_MOUSE_LL: WM_LBUTTONUP" );
-          break;
-        case WM_MBUTTONUP:
-          button = 1;
-          evt = HARPOON_BUTTON_RELEASE;
-          if_debug_send_message( "WH_MOUSE_LL: WM_MBUTTONUP" );
-          break;
-        case WM_RBUTTONUP:
-          button = 2;
-          evt = HARPOON_BUTTON_RELEASE;
-          if_debug_send_message( "WH_MOUSE_LL: WM_RBUTTONUP" );
-          break;
-
-        case WM_LBUTTONDBLCLK:
-          button = 0;
-          evt = HARPOON_2BUTTON_PRESS;
-          if_debug_send_message( "WH_MOUSE_LL: WM_LBUTTONDBLCLK" );
-          break;
-        case WM_MBUTTONDBLCLK:
-          button = 1;
-          evt = HARPOON_2BUTTON_PRESS;
-          if_debug_send_message( "WH_MOUSE_LL: WM_MBUTTONDBLCLK" );
-          break;
-        case WM_RBUTTONDBLCLK:
-          button = 2;
-          evt = HARPOON_2BUTTON_PRESS;
-           if_debug_send_message( "WH_MOUSE_LL: WM_RBUTTONDBLCLK" );
-         break;
-
-        case WM_MOUSEWHEEL:
-          evt = HARPOON_MOUSE_WHEEL;
-          button = 1;
-          if_debug_send_message( "WH_MOUSE_LL: WM_MOUSEWHEEL" );
-          break;
-
-        default:
-          evt = HARPOON_MOUSE_MOVE;
-          if_debug_send_message( "WH_MOUSE_LL: HARPOON_MOUSE_MOVE" );
+          case WM_LBUTTONDOWN:
+              button = 0;
+              evt = HARPOON_BUTTON_PRESS;
+              if_debug_send_message( "WH_MOUSE_LL: WM_LBUTTONDOWN" );
+              break;
+          case WM_MBUTTONDOWN:
+              button = 1;
+              evt = HARPOON_BUTTON_PRESS;
+              if_debug_send_message( "WH_MOUSE_LL: WM_MBUTTONDOWN" );
+              break;
+          case WM_RBUTTONDOWN:
+              button = 2;
+              evt = HARPOON_BUTTON_PRESS;
+              if_debug_send_message( "WH_MOUSE_LL: WM_RBUTTONDOWN" );
+              break;
+          case WM_XBUTTONDOWN:
+              button = ( HIWORD( wpar ) == XBUTTON1 ) ? 3 : 4;
+              evt = HARPOON_BUTTON_PRESS;
+              if_debug_send_message( "WH_MOUSE_LL: WM_XBUTTONDOWN" );
+              break;
+          
+          case WM_LBUTTONUP:
+              button = 0;
+              evt = HARPOON_BUTTON_RELEASE;
+              if_debug_send_message( "WH_MOUSE_LL: WM_LBUTTONUP" );
+              break;
+          case WM_MBUTTONUP:
+              button = 1;
+              evt = HARPOON_BUTTON_RELEASE;
+              if_debug_send_message( "WH_MOUSE_LL: WM_MBUTTONUP" );
+              break;
+          case WM_RBUTTONUP:
+              button = 2;
+              evt = HARPOON_BUTTON_RELEASE;
+              if_debug_send_message( "WH_MOUSE_LL: WM_RBUTTONUP" );
+              break;
+          case WM_XBUTTONUP:
+              button = ( HIWORD( wpar ) == XBUTTON1 ) ? 3 : 4;
+              evt = HARPOON_BUTTON_RELEASE;
+              if_debug_send_message( "WH_MOUSE_LL: WM_XBUTTONUP" );
+              break;
+          
+          case WM_MOUSEWHEEL:
+              evt = HARPOON_MOUSE_WHEEL;
+              button = 1;
+              if_debug_send_message( "WH_MOUSE_LL: WM_MOUSEWHEEL" );
+              break;
+          case WM_MOUSEHWHEEL:
+              evt = HARPOON_MOUSE_WHEEL;
+              button = 2;
+              if_debug_send_message( "WH_MOUSE_LL: WM_MOUSEHWHEEL" );
+              break;
+          
+          case WM_MOUSEMOVE:
+              evt = HARPOON_MOUSE_MOVE;
+              if_debug_send_message( "WH_MOUSE_LL: WM_MOUSEMOVE" );
         }
-      harpoon_post_message (evt, button, MAKELONG(x, y));
+      
+      if( evt != HARPOON_NOTHING )
+          harpoon_post_message (evt, button, MAKELONG(x, y));
     }
 
   return harpoon_generic_hook_return (code, wpar, lpar, mouse_ll_hook, TRUE);
@@ -487,6 +565,7 @@ harpoon_mouse_block_hook (int code, WPARAM wpar, LPARAM lpar)
         case WM_LBUTTONUP:
         case WM_MBUTTONUP:
         case WM_RBUTTONUP:
+        case WM_XBUTTONUP:
           forcecallnext = TRUE;
         }
     }
@@ -541,7 +620,25 @@ harpoon_keyboard_hook (int code, WPARAM wpar, LPARAM lpar)
 
       evt = pressed ? HARPOON_KEY_PRESS : HARPOON_KEY_RELEASE;
       forcecallnext = !pressed;
-      harpoon_post_message (evt, 0, flags);
+      
+      /*
+        The low-level keyboard hook is always preferred over the
+        regular keyboard hook. The low-level hook posts its own
+        message to the notification window. Here, we check to
+        see if there is a low-level hook. If not, we post our
+        own message.
+      */
+      if( !keyboard_ll_hook )
+        {
+          // The low level hook also intercepts keys injected using keybd_event.
+          // Some application use this function to toggle the keyboard lights...
+          if (wpar != VK_NUMLOCK &&
+              wpar != VK_CAPITAL &&
+              wpar != VK_SCROLL)
+            {
+              harpoon_post_message (evt, 0, flags);
+            }
+        }
       if_debug_send_message( "WH_KEYBOARD" );
     }
   return harpoon_generic_hook_return (code, wpar, lpar, keyboard_hook,
@@ -711,6 +808,12 @@ harpoon_init ( char imported_critical_file_list[][511], BOOL debug_harpoon )
           SendMessage( debug_hwnd, EM_SETLIMITTEXT, 0x7FFFFFFE, 0 );
           // Assign debug flag global.
           debug = debug_harpoon;
+          debug_send_message( 
+              "Note: If both WH_MOUSE and WH_MOUSE_LL are hooked, "
+              "WH_MOUSE messages are not appended to the debug window. "
+              "This is done mainly to avoid overflow."
+            );
+          debug_send_message( "Initializing..." );
         }
       else
         {
@@ -1020,13 +1123,13 @@ _mbstrncpy_lowercase( const char *out, const char *in, int bytes )
 // keep this code out of dllmain. don't call from dllmain.
 {
   int mb = 0;
-  char *src = (char *)in;
-  char *dest = (char *)out;
+  unsigned char *src = (unsigned char *)in;
+  unsigned char *dest = (unsigned char *)out;
 
   if( bytes <= 0 || !in || !out )
       return NULL;
 
-  while( mb = _mbsnextc( src ) )
+  while( ( mb = _mbsnextc( src ) ) > 0 )
   // This makes a lowercase copy of the filename.
     {
       // Point to next (mb) character:
