@@ -1,18 +1,21 @@
 /*
  * credits.h --- Workrave credits
  *
- * Copyright (C) 2007 Rob Caelers <robc@krandor.org>
+ * Copyright (C) 2007, 2008 Rob Caelers <robc@krandor.nl>
  * All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef CREDITS_H
@@ -21,7 +24,7 @@
 #include <stdlib.h>
 
 const char *workrave_copyright =
-  "Copyright 2001-2007 Rob Caelers & Raymond Penners";
+  "Copyright 2001-2008 Rob Caelers & Raymond Penners";
 
 const char *workrave_authors[] = {
   "Core developers:",
@@ -30,6 +33,7 @@ const char *workrave_authors[] = {
   "",
   "Contributors:",
   "Ray Satiro <raysatiro@yahoo.com>",
+  "Ruairi Fullam",
   NULL
 };
 
