@@ -1,19 +1,22 @@
 // System.cc
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: System.cc 1331 2007-09-14 17:22:10Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: System.cc 1471 2008-03-09 20:17:13Z rcaelers $
 //
 
 #ifdef HAVE_CONFIG_H
@@ -32,20 +35,26 @@
 #include "System.hh"
 #include "debug.hh"
 
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
 #include <X11/X.h>
 #include <X11/Xproto.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #endif
 
-#ifdef WIN32
+#if defined(HAVE_UNIX)
+#include <sys/wait.h>
+#endif
+
+#ifdef PLATFORM_OS_WIN32
 #include <shlobj.h>
 #include "harpoon.h"
 
 #include "CoreFactory.hh"
 #include "IConfigurator.hh"
 
+using namespace workrave;
+
 /* MinGW does not have this one yet */
 #undef INTERFACE
 #define INTERFACE IShellDispatch
@@ -98,15 +107,15 @@ const GUID CLSID_Shell =
   { 0xa4, 0x9e, 0x44, 0x45, 0x53, 0x54 }
 };
 
-#endif /* WIN32 */
+#endif /* PLATFORM_OS_WIN32 */
 
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
 
 bool System::kde = false;
 bool System::lockable = false;
 std::string System::lock_display;
 
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
 
 HINSTANCE System::user32_dll = NULL;
 System::LockWorkStationFunc System::lock_func = NULL;
@@ -118,9 +127,9 @@ bool
 System::is_lockable()
 {
   bool ret;
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
   ret = lockable;
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
   ret = lock_func != NULL;
 #else
   ret = false;
@@ -128,7 +137,7 @@ System::is_lockable()
   return ret;
 }
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
 static bool
 invoke(const gchar* command, bool async = false)
 {
@@ -164,7 +173,7 @@ System::lock()
   TRACE_ENTER("System::lock");
   if (is_lockable())
     {
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
       gchar *program = NULL, *cmd = NULL;
       if (is_kde() && (program = g_find_program_in_path("kdesktop_lock")))
         {
@@ -195,7 +204,7 @@ System::lock()
 end:  // cleanup of created strings, jump to avoid duplication
       g_free(program);
       g_free(cmd);
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
       (*lock_func)();
 #endif
     }
@@ -206,9 +215,13 @@ bool
 System::is_shutdown_supported()
 {
   bool ret;
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
+# if defined(HAVE_GNOME)
+  ret = true;
+# else  
   ret = false;
-#elif defined(WIN32)
+# endif  
+#elif defined(PLATFORM_OS_WIN32)
   ret = shutdown_supported;
 #else
   ret = false;
@@ -219,13 +232,22 @@ System::is_shutdown_supported()
 void
 System::shutdown()
 {
-#if defined(HAVE_X)
-#elif defined(WIN32)
+#if defined(PLATFORM_OS_UNIX)
+# if defined(HAVE_GNOME)
+  gchar *program = NULL, *cmd = NULL;
+
+  if ((program = g_find_program_in_path("gnome-session-save")))
+    {
+      cmd = g_strdup_printf("%s --kill", program);
+      invoke(cmd, false);
+    }
+# endif
+#elif defined(PLATFORM_OS_WIN32)
   shutdown_helper(true);
 #endif
 }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 bool
 System::shutdown_helper(bool for_real)
 {
@@ -249,13 +271,13 @@ System::shutdown_helper(bool for_real)
 
 void
 System::init(
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
              const char *display
 #endif
              )
 {
   TRACE_ENTER("System::init");
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
   init_kde(display);
  
   gchar *program;
@@ -268,7 +290,7 @@ System::init(
   else if ((program = g_find_program_in_path("xlock")) != NULL)
     lockable = true;
  
-  if (lockable)
+  if (lockable && display != NULL)
     {
       g_free(program);
       lock_display = display;
@@ -278,7 +300,7 @@ System::init(
     {
       TRACE_MSG("Locking disabled");
     }
-#elif defined(WIN32)
+#elif defined(PLATFORM_OS_WIN32)
   // Note: this memory is never freed
   user32_dll = LoadLibrary("user32.dll");
   if (user32_dll != NULL)
@@ -292,7 +314,7 @@ System::init(
 }
 
 
-#if defined(HAVE_X)
+#if defined(PLATFORM_OS_UNIX)
 static bool
 get_self_typed_prop (Display *display,
                      Window      xwindow,
@@ -396,11 +418,14 @@ void
 System::init_kde(const char *display)
 {
   TRACE_ENTER("System::init_kde");
-  Display * dis = XOpenDisplay(display);
-  if (dis != None)
+  if (display != NULL)
     {
-      kde = look_for_kdesktop_recursive (dis, XRootWindow(dis, 0));
-      XCloseDisplay(dis);
+      Display * dis = XOpenDisplay(display);
+      if (dis != None)
+        {
+          kde = look_for_kdesktop_recursive (dis, XRootWindow(dis, 0));
+          XCloseDisplay(dis);
+        }
     }
   TRACE_RETURN(kde);
 }
