@@ -1,20 +1,23 @@
 // TimerBoxControl.cc --- Timers Widgets
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: TimerBoxControl.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: TimerBoxControl.cc 1434 2008-02-01 22:20:17Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,7 +35,7 @@ static const char rcsid[] = "$Id: TimerBoxControl.cc 1298 2007-08-30 14:18:04Z r
 #include "Text.hh"
 
 #include "CoreFactory.hh"
-#include "ITimer.hh"
+#include "CoreConfig.hh"
 #include "IBreak.hh"
 #include "IConfigurator.hh"
 
@@ -40,6 +43,7 @@ static const char rcsid[] = "$Id: TimerBoxControl.cc 1298 2007-08-30 14:18:04Z r
 #include "IDistributionManager.hh"
 #endif
 
+using namespace workrave;
 
 const std::string TimerBoxControl::CFG_KEY_TIMERBOX = "gui/";
 const std::string TimerBoxControl::CFG_KEY_TIMERBOX_CYCLE_TIME = "/cycle_time";
@@ -138,18 +142,13 @@ TimerBoxControl::init()
 {
   TRACE_ENTER("TimerBoxControl::init");
 
-  ICore *core = CoreFactory::get_core();
-
   // Listen for configugration changes.
   IConfigurator *config = CoreFactory::get_configurator();
   config->add_listener(TimerBoxControl::CFG_KEY_TIMERBOX + name, this);
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
-      IBreak *break_data = core->get_break(BreakId(i));
-      config->add_listener("gui/breaks/"
-                           + break_data->get_name()
-                           + "/enabled", this);
+      config->add_listener(CoreConfig::CFG_KEY_BREAK_ENABLED % BreakId(i), this);
 
       break_position[i] = i;
       break_flags[i] = 0;
@@ -194,8 +193,7 @@ TimerBoxControl::update_widgets()
   for (int count = 0; count < BREAK_ID_SIZEOF; count++)
     {
       ICore *core = CoreFactory::get_core();
-      IBreak *break_data = core->get_break(BreakId(count));
-      ITimer *timer = break_data->get_timer();
+      IBreak *b = core->get_break((BreakId)count);
 
       std::string text;
       ITimeBar::ColorId primary_color;
@@ -203,73 +201,46 @@ TimerBoxControl::update_widgets()
       ITimeBar::ColorId secondary_color;
       int secondary_val, secondary_max;
 
-#ifdef LETS_SEE_HOW_WORKRAVE_BEHAVES_WITHOUT_THIS
-      if (!node_master && num_peers > 0)
+      if (b == NULL)
         {
-          text = _("Inactive");
-          primary_color = ITimeBar::COLOR_ID_INACTIVE;
-          secondary_color = ITimeBar::COLOR_ID_INACTIVE;
-          primary_val = 0;
-          primary_max = 60;
-          secondary_val = 0;
-          secondary_max = 60;
+          continue;
+        }
+
+      // Collect some data.
+      time_t maxActiveTime = b->get_limit();
+      time_t activeTime = b->get_elapsed_time();
+      time_t breakDuration = b->get_auto_reset();
+      time_t idleTime = b->get_elapsed_idle_time();
+      bool overdue = (maxActiveTime < activeTime);
+
+      // Set the text
+      if (b->is_limit_enabled() && maxActiveTime != 0)
+        {
+          text = Text::time_to_string(maxActiveTime - activeTime);
         }
       else
-#endif
         {
-          if (timer == NULL)
-            {
-              continue;
-            }
-
-          ITimer::TimerState timerState = timer->get_state();
-
-          // Collect some data.
-          time_t maxActiveTime = timer->get_limit();
-          time_t activeTime = timer->get_elapsed_time();
-          time_t breakDuration = timer->get_auto_reset();
-          time_t idleTime = timer->get_elapsed_idle_time();
-          bool overdue = (maxActiveTime < activeTime);
-
-          // Set the text
-          if (timer->is_limit_enabled() && maxActiveTime != 0)
-            {
-              text = Text::time_to_string(maxActiveTime - activeTime);
-            }
-          else
-            {
-              text = Text::time_to_string(activeTime);
-            }
-          // And set the bar.
-          secondary_val = secondary_max = 0;
-          secondary_color = ITimeBar::COLOR_ID_INACTIVE;
-
-          if (timerState == ITimer::STATE_INVALID)
-            {
-              primary_color = ITimeBar::COLOR_ID_INACTIVE;
-              primary_val = 0;
-              primary_max = 60;
-              text = _("Wait");
-            }
-          else
-            {
-              // Timer is running, show elapsed time.
-              primary_val = activeTime;
-              primary_max = maxActiveTime;
-
-              primary_color = overdue
-                ? ITimeBar::COLOR_ID_OVERDUE : ITimeBar::COLOR_ID_ACTIVE;
-
-              if (//timerState == ITimer::STATE_STOPPED &&
-                  timer->is_auto_reset_enabled() && breakDuration != 0)
-                {
-                  // resting.
-                  secondary_color = ITimeBar::COLOR_ID_INACTIVE;
-                  secondary_val = idleTime;
-                  secondary_max = breakDuration;
-                }
-            }
+          text = Text::time_to_string(activeTime);
         }
+      // And set the bar.
+      secondary_val = secondary_max = 0;
+      secondary_color = ITimeBar::COLOR_ID_INACTIVE;
+
+      // Timer is running, show elapsed time.
+      primary_val = activeTime;
+      primary_max = maxActiveTime;
+
+      primary_color = overdue
+        ? ITimeBar::COLOR_ID_OVERDUE : ITimeBar::COLOR_ID_ACTIVE;
+
+      if (b->is_auto_reset_enabled() && breakDuration != 0)
+        {
+          // resting.
+          secondary_color = ITimeBar::COLOR_ID_INACTIVE;
+          secondary_val = idleTime;
+          secondary_max = breakDuration;
+        }
+
       view->set_time_bar(BreakId(count), text,
                          primary_color, primary_val, primary_max,
                          secondary_color, secondary_val, secondary_max);
@@ -354,9 +325,9 @@ TimerBoxControl::init_slot(int slot)
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
       ICore *core = CoreFactory::get_core();
-      IBreak *break_data = core->get_break(BreakId(i));
+      IBreak *b = core->get_break(BreakId(i));
 
-      bool on = break_data->get_break_enabled();
+      bool on = b->is_enabled();
 
       if (on && break_position[i] == slot && !(break_flags[i] & BREAK_HIDE))
         {
@@ -376,10 +347,9 @@ TimerBoxControl::init_slot(int slot)
       int flags = break_flags[id];
 
       ICore *core = CoreFactory::get_core();
-      IBreak *break_data = core->get_break(BreakId(id));
-      ITimer *timer = break_data->get_timer();
+      IBreak *b = core->get_break((BreakId)i);
 
-      time_t time_left = timer->get_limit() - timer->get_elapsed_time();
+      time_t time_left = b->get_limit() - b->get_elapsed_time();
 
       // Exclude break if not imminent.
       if (flags & BREAK_WHEN_IMMINENT && time_left > break_imminent_time[id] &&
@@ -510,7 +480,7 @@ TimerBoxControl::read_configuration()
 
 //! Callback that the configuration has changed.
 void
-TimerBoxControl::config_changed_notify(string key)
+TimerBoxControl::config_changed_notify(const string &key)
 {
   (void) key;
 
@@ -529,7 +499,7 @@ TimerBoxControl::get_cycle_time(string name)
 {
   int ret;
   if (! CoreFactory::get_configurator()
-      ->get_value(TimerBoxControl::CFG_KEY_TIMERBOX + name + TimerBoxControl::CFG_KEY_TIMERBOX_CYCLE_TIME, &ret))
+      ->get_value(TimerBoxControl::CFG_KEY_TIMERBOX + name + TimerBoxControl::CFG_KEY_TIMERBOX_CYCLE_TIME, ret))
     {
       ret = 10;
     }
@@ -561,7 +531,7 @@ TimerBoxControl::get_timer_imminent_time(string name, BreakId timer)
   const string key = get_timer_config_key(name, timer, CFG_KEY_TIMERBOX_IMMINENT);
   int ret;
   if (! CoreFactory::get_configurator()
-      ->get_value(key, &ret))
+      ->get_value(key, ret))
     {
       ret = 30;
     }
@@ -583,7 +553,7 @@ TimerBoxControl::get_timer_slot(string name, BreakId timer)
   const string key = get_timer_config_key(name, timer, CFG_KEY_TIMERBOX_POSITION);
   int ret;
   if (! CoreFactory::get_configurator()
-      ->get_value(key, &ret))
+      ->get_value(key, ret))
     {
       if (name == "applet")
         {
@@ -614,7 +584,7 @@ TimerBoxControl::get_timer_flags(string name, BreakId timer)
   const string key = get_timer_config_key(name, timer, CFG_KEY_TIMERBOX_FLAGS);
   int ret;
   if (! CoreFactory::get_configurator()
-      ->get_value(key, &ret))
+      ->get_value(key, ret))
     {
       ret = 0;
     }
@@ -635,7 +605,7 @@ TimerBoxControl::is_enabled(string name)
 {
   bool ret = true;
   if (! CoreFactory::get_configurator()
-      ->get_value(CFG_KEY_TIMERBOX + name + CFG_KEY_TIMERBOX_ENABLED, &ret))
+      ->get_value(CFG_KEY_TIMERBOX + name + CFG_KEY_TIMERBOX_ENABLED, ret))
     {
       ret = true;
     }
