@@ -1,20 +1,23 @@
 // AppletControl.cc --- Applet info Control
 //
-// Copyright (C) 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: AppletControl.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: AppletControl.cc 1473 2008-03-10 17:47:35Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -31,15 +34,15 @@ static const char rcsid[] = "$Id: AppletControl.cc 1298 2007-08-30 14:18:04Z rca
 #include "KdeAppletWindow.hh"
 #endif
 
-#ifdef HAVE_GNOME
+#ifdef HAVE_GNOMEAPPLET
 #include "GnomeAppletWindow.hh"
 #endif
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
 #include "X11SystrayAppletWindow.hh"
 #endif
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 #include "W32AppletWindow.hh"
 #endif
 
@@ -50,6 +53,8 @@ static const char rcsid[] = "$Id: AppletControl.cc 1298 2007-08-30 14:18:04Z rca
 #include "CoreFactory.hh"
 #include "IConfigurator.hh"
 
+using namespace workrave;
+using namespace std;
 
 //! Constructor.
 AppletControl::AppletControl()
@@ -87,15 +92,15 @@ AppletControl::init()
   applets[APPLET_KDE] = new KdeAppletWindow(this);
 #endif
 
-#ifdef HAVE_GNOME
+#ifdef HAVE_GNOMEAPPLET
   applets[APPLET_GNOME] = new GnomeAppletWindow(this);
 #endif
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   applets[APPLET_TRAY] = new X11SystrayAppletWindow(this);
 #endif
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   applets[APPLET_W32] = new W32AppletWindow();
 #endif
 
@@ -111,10 +116,13 @@ AppletControl::init()
 void
 AppletControl::show()
 {
+  TRACE_ENTER("AppletControl::show");
+  
   bool specific = false;
   AppletState rc;
 
   rc = activate_applet(APPLET_GNOME);
+  TRACE_MSG("Gnome " << rc);
   if (rc != AppletWindow::APPLET_STATE_DISABLED)
     {
       // Applet now visible or pending.
@@ -123,28 +131,34 @@ AppletControl::show()
     }
 
   rc = activate_applet(APPLET_KDE);
+  TRACE_MSG("kde " << rc);
   if (rc != AppletWindow::APPLET_STATE_DISABLED)
     {
       specific = true;
     }
 
   rc = activate_applet(APPLET_W32);
+  TRACE_MSG("Win32 " << rc);
   if (rc != AppletWindow::APPLET_STATE_DISABLED)
     {
       specific = true;
     }
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   if (specific)
     {
       deactivate_applet(APPLET_TRAY);
+      TRACE_MSG("X11 deact ");
     }
   else
     {
-      activate_applet(APPLET_TRAY);
+      rc = activate_applet(APPLET_TRAY);
+      TRACE_MSG("X11 act " << rc);
     }
 #endif
   check_visible();
+  
+  TRACE_EXIT();
 }
 
 
@@ -162,7 +176,7 @@ AppletControl::show(AppletType type)
       specific = true;
     }
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   if (applets[APPLET_TRAY] != NULL)
     {
       if ((type == APPLET_KDE || type == APPLET_GNOME)
@@ -205,6 +219,7 @@ AppletControl::hide(AppletType type)
 void
 AppletControl::set_applet_state(AppletType type, AppletWindow::AppletState state)
 {
+  TRACE_ENTER_MSG("AppletControl::set_applet_state", type << " " << state);
   switch (state)
     {
     case AppletWindow::APPLET_STATE_DISABLED:
@@ -220,7 +235,7 @@ AppletControl::set_applet_state(AppletType type, AppletWindow::AppletState state
       break;
     }
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   if (visible[type] &&
       (type == APPLET_KDE || type == APPLET_GNOME))
     {
@@ -234,6 +249,7 @@ AppletControl::set_applet_state(AppletType type, AppletWindow::AppletState state
     }
 
   check_visible();
+  TRACE_EXIT();
 }
 
 
@@ -267,7 +283,11 @@ void
 AppletControl::heartbeat()
 {
   TRACE_ENTER("AppletControl::heartbeat");
-  if (delayed_show > 0)
+  if (delayed_show < 0 && !is_visible())
+    {
+      delayed_show = 60;
+    }
+  else if (delayed_show > 0)
     {
       delayed_show--;
     }
@@ -322,7 +342,7 @@ AppletControl::read_configuration()
 
 //! Callback that the configuration has changed.
 void
-AppletControl::config_changed_notify(string key)
+AppletControl::config_changed_notify(const string &key)
 {
   TRACE_ENTER_MSG("AppletControl::config_changed_notify", key);
   (void) key;
