@@ -1,25 +1,35 @@
 // BreakWindow.cc --- base class for the break windows
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: BreakWindow.cc 1330 2007-09-14 16:20:05Z rcaelers $";
+static const char rcsid[] = "$Id: BreakWindow.cc 1524 2008-04-02 20:13:31Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <ctime>
+
+// FIXME: Debug code, remove later
+#ifdef PLATFORM_OS_WIN32
+#include "w32debug.hh"
+#endif
+
 #include "preinclude.h"
 #include "debug.hh"
 #include "nls.h"
@@ -31,6 +41,7 @@ static const char rcsid[] = "$Id: BreakWindow.cc 1330 2007-09-14 16:20:05Z rcael
 
 #include <math.h>
 
+#include "GUI.hh"
 #include "BreakWindow.hh"
 #include "IBreakResponse.hh"
 #include "GtkUtil.hh"
@@ -42,20 +53,22 @@ static const char rcsid[] = "$Id: BreakWindow.cc 1330 2007-09-14 16:20:05Z rcael
 #include "IConfigurator.hh"
 #include "CoreFactory.hh"
 
-#if defined(WIN32)
+#if defined(PLATFORM_OS_WIN32)
 #include "DesktopWindow.hh"
-#elif defined(HAVE_X)
+#elif defined(PLATFORM_OS_UNIX)
 #include "desktop-window.h"
 #endif
 
+using namespace workrave;
 
 //! Constructor
 /*!
  *  \param control The controller.
  */
 BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
-                         bool ignorable, GUI::BlockMode mode) :
-#ifdef WIN32
+                         BreakFlags break_flags,
+                         GUIConfig::BlockMode mode) :
+#ifdef PLATFORM_OS_WIN32
 /*
  Windows will have a gtk toplevel window regardless of mode.
  Hopefully this takes care of the phantom parent problem.
@@ -64,22 +77,23 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
 */
          Gtk::Window( Gtk::WINDOW_TOPLEVEL ),
 #else
-         Gtk::Window(mode==GUI::BLOCK_MODE_NONE
+         Gtk::Window(mode==GUIConfig::BLOCK_MODE_NONE
                      ? Gtk::WINDOW_TOPLEVEL
                      : Gtk::WINDOW_POPUP),
 #endif
          block_mode(mode),
-         ignorable_break(ignorable),
+         break_flags(break_flags),
          frame(NULL),
          break_response(NULL),
-         gui(NULL)
+         gui(NULL),
+         visible(false)
 {
   this->break_id = break_id;
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   desktop_window = NULL;
 
-  if( mode != GUI::BLOCK_MODE_NONE )
+  if (mode != GUIConfig::BLOCK_MODE_NONE)
   {
     // Disable titlebar to appear like a popup
     set_decorated(false);
@@ -87,35 +101,75 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
   }
 #endif
 
-#ifdef HAVE_X
-  GtkUtil::set_wmclass(*this, "Break");
-#endif
-
   // On W32, must be *before* realize, otherwise a border is drawn.
   set_resizable(false);
 
   // Need to realize window before it is shown
   // Otherwise, there is not gobj()...
   realize();
-  if (mode == GUI::BLOCK_MODE_NONE)
+  if (mode == GUIConfig::BLOCK_MODE_NONE)
     {
       Glib::RefPtr<Gdk::Window> window = get_window();
       window->set_functions(Gdk::FUNC_MOVE);
     }
 
+  // trace window handles:
+  // FIXME: debug, remove later
+#ifdef PLATFORM_OS_WIN32
+  HWND _hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
+  HWND _scope = (HWND) GDK_WINDOW_HWND( GTK_WIDGET( this->gobj() )->window );
+  HWND _hRoot = GetAncestor( _hwnd, GA_ROOT );
+  HWND _hParent = GetAncestor( _hwnd, GA_PARENT );
+  HWND _hDesktop = GetDesktopWindow();
+  
+  APPEND_TIME( "BreakWindow created", hex << _hwnd );
+  
+  if( _hwnd != _scope )
+    {
+      APPEND( "!!!!!!!!!!!!!!!", "Scope issue: " << hex << _scope );
+      APPEND_ENDL();
+	}
+  
+  if( _hwnd != _hRoot )
+    {
+	  APPEND( "GetDesktopWindow()", hex << _hDesktop );
+      APPEND( "!!!!!!!!!!!!!!!", "BreakWindow GA_ROOT: " << hex << _hRoot );
+      APPEND_ENDL();
+	}
+  
+  if( _hParent != _hDesktop )
+    {
+	  APPEND( "GetDesktopWindow()", hex << _hDesktop );
+      APPEND( "!!!!!!!!!!!!!!!", "PreludeWindow GA_PARENT: " << hex << _hParent );
+      
+      HWND _hTemp;
+      while( IsWindow( _hParent ) && _hParent != _hDesktop )
+        {
+		  _hTemp = _hParent;
+          _hParent = GetAncestor( _hTemp, GA_PARENT );
+		  HWND _hParent2 = (HWND)GetWindowLong( _hTemp, GWL_HWNDPARENT );
+          if( _hParent == _hTemp )
+              break;
+          APPEND( "!!!!!!!!!!!!!!!", hex << _hTemp << " GA_PARENT: " << hex << _hParent );
+          APPEND( "!!!!!!!!!!!!!!!", hex << _hTemp << " GWL_HWNDPARENT: " << hex << _hParent2 );
+        }
+      APPEND_ENDL();
+    }
+    
+#endif
+
   this->head = head;
-#ifdef HAVE_GTK_MULTIHEAD
   if (head.valid)
     {
       Gtk::Window::set_screen(head.screen);
     }
-#endif
 
   bool initial_ignore_activity = false;
-#ifdef WIN32
+
+#ifdef PLATFORM_OS_WIN32
   bool force_focus = false;
-  CoreFactory::get_configurator()->get_value_default( "advanced/force_focus",
-                                                      &force_focus,
+  CoreFactory::get_configurator()->get_value_with_default( "advanced/force_focus",
+                                                      force_focus,
                                                       false);
   if (force_focus)
     {
@@ -127,10 +181,8 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
   ICore *core = CoreFactory::get_core();
   assert(core != NULL);
   core->set_insist_policy(initial_ignore_activity ?
-                        ICore::INSIST_POLICY_IGNORE :
-                        (block_mode != GUI::BLOCK_MODE_NONE
-                         ? ICore::INSIST_POLICY_HALT
-                         : ICore::INSIST_POLICY_RESET));
+                          ICore::INSIST_POLICY_IGNORE :
+                          ICore::INSIST_POLICY_HALT);
 }
 
 
@@ -142,7 +194,7 @@ BreakWindow::init_gui()
     {
       gui = manage(create_gui());
 
-      if (block_mode == GUI::BLOCK_MODE_NONE)
+      if (block_mode == GUIConfig::BLOCK_MODE_NONE)
         {
           set_border_width(12);
           add(*gui);
@@ -164,12 +216,12 @@ BreakWindow::init_gui()
           window_frame->add(*frame);
           frame->add(*gui);
 
-          if (block_mode == GUI::BLOCK_MODE_ALL)
+          if (block_mode == GUIConfig::BLOCK_MODE_ALL)
             {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
               desktop_window = new DesktopWindow(head);
               add(*window_frame);
-#else
+#elif defined(PLATFORM_OS_UNIX)
               set_size_request(head.get_width(),
                                head.get_height());
               set_app_paintable(true);
@@ -185,19 +237,15 @@ BreakWindow::init_gui()
               add(*window_frame);
             }
         }
-      show_all_children();
-      stick();
-
-      // Set window hints.
-      WindowHints::set_skip_winlist(Gtk::Widget::gobj(), true);
-      // causes windows to display on win32
-      // WindowHints::set_always_on_top(Gtk::Widget::gobj(), true);
 
       // FIXME: check if it was intentionally not unset for RB
       if (break_id != BREAK_ID_REST_BREAK)
         {
           unset_flags(Gtk::CAN_FOCUS);
         }
+      
+      show_all_children();
+      stick();
     }
 }
 
@@ -213,7 +261,7 @@ BreakWindow::~BreakWindow()
       frame->set_frame_flashing(0);
     }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   delete desktop_window;
 #endif
   TRACE_EXIT();
@@ -238,7 +286,7 @@ BreakWindow::create_lock_button()
     {
       ret = manage(GtkUtil::create_image_button(_("Lock"), "lock.png"));
       ret->signal_clicked()
-        .connect(MEMBER_SLOT(*this, &BreakWindow::on_lock_button_clicked));
+        .connect(sigc::mem_fun(*this, &BreakWindow::on_lock_button_clicked));
       GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
     }
   else
@@ -257,7 +305,7 @@ BreakWindow::create_shutdown_button()
     {
       ret = manage(GtkUtil::create_image_button(_("Shut down"), "shutdown.png"));
       ret->signal_clicked()
-        .connect(MEMBER_SLOT(*this, &BreakWindow::on_shutdown_button_clicked));
+        .connect(sigc::mem_fun(*this, &BreakWindow::on_shutdown_button_clicked));
       GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
     }
   else
@@ -274,7 +322,7 @@ BreakWindow::create_skip_button()
   Gtk::Button *ret;
   ret = manage(GtkUtil::create_custom_stock_button(_("Skip"), Gtk::Stock::CLOSE));
   ret->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &BreakWindow::on_skip_button_clicked));
+    .connect(sigc::mem_fun(*this, &BreakWindow::on_skip_button_clicked));
   GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
   return ret;
 }
@@ -287,7 +335,7 @@ BreakWindow::create_postpone_button()
   Gtk::Button *ret;
   ret = manage(GtkUtil::create_custom_stock_button(_("Postpone"), Gtk::Stock::REDO));
   ret->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &BreakWindow::on_postpone_button_clicked));
+    .connect(sigc::mem_fun(*this, &BreakWindow::on_postpone_button_clicked));
   GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
   return ret;
 }
@@ -312,6 +360,11 @@ BreakWindow::on_lock_button_clicked()
 void
 BreakWindow::on_shutdown_button_clicked()
 {
+#ifdef HAVE_GNOME
+  GUI *gui = GUI::get_instance();
+  assert(gui != NULL);
+  gui->interrupt_grab();
+#endif
   System::shutdown();
 }
 
@@ -366,7 +419,7 @@ BreakWindow::create_break_buttons(bool lockable,
 {
   Gtk::HButtonBox *box = NULL;
 
-  if (ignorable_break || lockable || shutdownable)
+  if ((break_flags != BREAK_FLAGS_NONE) || lockable || shutdownable)
     {
       box = new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6);
 
@@ -389,11 +442,14 @@ BreakWindow::create_break_buttons(bool lockable,
             }
         }
 
-      if (ignorable_break)
+      if ((break_flags & BREAK_FLAGS_SKIPPABLE) != 0)
         {
           Gtk::Button *skip_button = create_skip_button();
           box->pack_end(*skip_button, Gtk::SHRINK, 0);
-
+        }
+      
+        if ((break_flags & BREAK_FLAGS_POSTPONABLE) != 0)
+        {
           Gtk::Button *postpone_button = create_postpone_button();
           box->pack_end(*postpone_button, Gtk::SHRINK, 0);
         }
@@ -403,28 +459,47 @@ BreakWindow::create_break_buttons(bool lockable,
 }
 
 
+//! Starts the daily limit.
+void
+BreakWindow::init()
+{
+  TRACE_ENTER("BreakWindow::init");
+  init_gui();
+  TRACE_EXIT();
+}
+
 //! Starts the daily limit.
 void
 BreakWindow::start()
 {
   TRACE_ENTER("BreakWindow::start");
 
-  init_gui();
+  update_break_window();
   center();
-#ifdef WIN32
+
+#ifdef PLATFORM_OS_WIN32
   if (desktop_window)
     desktop_window->set_visible(true);
 #endif
   show_all();
 
   // Set window hints.
-  WindowHints::set_skip_winlist(Gtk::Widget::gobj(), true);
-  WindowHints::set_always_on_top(Gtk::Widget::gobj(), true);
-  raise();
+  set_skip_pager_hint(true);
+  set_skip_taskbar_hint(true);
 
-#ifdef CAUSES_FVWM_FOCUS_PROBLEMS
-  present(); // After grab() please (Windows)
+#ifdef PLATFORM_OS_WIN32
+  // FIXME: hack until gtk+ is fixed.
+  GtkWidget *gtkwin = Gtk::Widget::gobj();
+  GdkWindow *gdkwin = gtkwin->window;
+
+  parent = GetWindowLong((HWND)GDK_WINDOW_HWND(gdkwin), GWL_HWNDPARENT);
+    
+  SetWindowLong((HWND)GDK_WINDOW_HWND(gdkwin), GWL_HWNDPARENT,
+                (long) GetDesktopWindow());
 #endif
+  
+  WindowHints::set_always_on_top(this, true);
+  raise();
 
   // In case the show_all resized the window...
   center();
@@ -437,13 +512,22 @@ BreakWindow::stop()
 {
   TRACE_ENTER("BreakWindow::stop");
 
+#ifdef PLATFORM_OS_WIN32
+  GtkWidget *gtkwin = Gtk::Widget::gobj();
+  GdkWindow *gdkwin = gtkwin->window;
+  SetWindowLong((HWND)GDK_WINDOW_HWND(gdkwin), GWL_HWNDPARENT,
+                parent);
+#endif
+  
   if (frame != NULL)
     {
       frame->set_frame_flashing(0);
     }
 
   hide_all();
-#ifdef WIN32
+  visible = false;
+  
+#ifdef PLATFORM_OS_WIN32
   if (desktop_window)
     desktop_window->set_visible(false);
 #endif
@@ -468,12 +552,14 @@ BreakWindow::destroy()
 void
 BreakWindow::refresh()
 {
-#ifdef WIN32
-  if (block_mode != GUI::BLOCK_MODE_NONE)
-    {
-      WindowHints::set_always_on_top(Gtk::Widget::gobj(), true);
-    }
-#endif
+  update_break_window();
+  
+// #ifdef PLATFORM_OS_WIN32
+//   if (block_mode != GUIConfig::BLOCK_MODE_NONE)
+//     {
+//       WindowHints::set_always_on_top(this, true);
+//     }
+// #endif
 }
 
 Glib::RefPtr<Gdk::Window>
@@ -484,3 +570,7 @@ BreakWindow::get_gdk_window()
 
 
 
+void
+BreakWindow::update_break_window()
+{
+}
