@@ -1,20 +1,23 @@
 // DailyLimitWindow.cc --- window for the daily limit
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: DailyLimitWindow.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: DailyLimitWindow.cc 1485 2008-03-13 20:52:02Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,16 +34,18 @@ static const char rcsid[] = "$Id: DailyLimitWindow.cc 1298 2007-08-30 14:18:04Z
 #include <gtkmm/box.h>
 #include <gtkmm/button.h>
 
+#include "GUI.hh"
 #include "IBreakResponse.hh"
 #include "WindowHints.hh"
 #include "Util.hh"
 #include "Hig.hh"
 
+using namespace workrave;
 
 //! Construct a new Daily limit window.
-DailyLimitWindow::DailyLimitWindow(HeadInfo &head, bool ignorable,
-                                   GUI::BlockMode mode) :
-  BreakWindow(BREAK_ID_DAILY_LIMIT, head, ignorable, mode)
+DailyLimitWindow::DailyLimitWindow(HeadInfo &head, BreakFlags break_flags,
+                                   GUIConfig::BlockMode mode) :
+  BreakWindow(BREAK_ID_DAILY_LIMIT, head, break_flags, mode)
 {
   set_title(_("Daily limit"));
 }
@@ -94,5 +99,3 @@ DailyLimitWindow::set_progress(int value, int max_value)
   (void) max_value;
   // FIXME: show progress until timer restart time?
 }
-
-
