@@ -1,20 +1,23 @@
 // EventImage.cc ---
 //
-// Copyright (C) 2003, 2004 Rob Caelers <robc@krandor.org>
+// Copyright (C) 2003, 2004, 2007 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: EventImage.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: EventImage.cc 1351 2007-10-14 20:56:54Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -98,7 +101,6 @@ EventImage::on_unmap_event(GdkEventAny *event)
   return Gtk::Image::on_unmap_event(event);
 }
 
-#ifdef HAVE_GTKMM24
 void
 EventImage::on_size_allocate(Gtk::Allocation &allocation)
 {
@@ -115,22 +117,3 @@ EventImage::on_size_allocate(Gtk::Allocation &allocation)
                              allocation.get_height());
     }
 }
-
-#else
-void
-EventImage::on_size_allocate(GtkAllocation *allocation)
-{
-  Gtk::Image::on_size_allocate(allocation);
-
-  GtkWidget *widget = GTK_WIDGET(gobj());
-
-  if (GTK_WIDGET_REALIZED(widget))
-    {
-      gdk_window_move_resize(event_window,
-                             allocation->x ,
-                             allocation->y ,
-                             allocation->width,
-                             allocation->height);
-    }
-}
-#endif
