@@ -1,20 +1,23 @@
 // GUI.cc --- The WorkRave GUI
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: GUI.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: GUI.cc 1549 2008-06-28 21:51:43Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +27,14 @@ static const char rcsid[] = "$Id: GUI.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 #include "nls.h"
 #include "debug.hh"
 
-#include <sstream>
+#if defined(PLATFORM_OS_WIN32)
+#include "w32debug.hh"
+#endif
+
+#include <gtkmm/main.h>
+#include <gtkmm/messagedialog.h>
+#include <glibmm/refptr.h>
+
 #include <unistd.h>
 #include <stdio.h>
 #include <assert.h>
@@ -32,76 +42,75 @@ static const char rcsid[] = "$Id: GUI.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 
 #include "GUI.hh"
 
-#include "CoreFactory.hh"
-#include "ICore.hh"
-#include "IConfigurator.hh"
-
-#include "IBreakResponse.hh"
-#include "DailyLimitWindow.hh"
-#include "MainWindow.hh"
-#include "IBreakWindow.hh"
+// Backend includes.
 #include "IBreak.hh"
-#include "ITimer.hh"
+#include "IBreakResponse.hh"
+#include "IBreakWindow.hh"
+#include "IConfigurator.hh"
+#include "ICore.hh"
+#include "CoreFactory.hh"
+  
+#include "AppletControl.hh"
+#include "AppletWindow.hh"
 #include "BreakWindow.hh"
+#include "DailyLimitWindow.hh"
+#include "GUIConfig.hh"
+#include "MainWindow.hh"
+#include "Menus.hh"
 #include "MicroBreakWindow.hh"
 #include "PreludeWindow.hh"
 #include "RestBreakWindow.hh"
-#include "Util.hh"
-#include "WindowHints.hh"
+#include "SoundPlayer.hh"
+#include "StatusIcon.hh"
 #include "System.hh"
 #include "Text.hh"
-#include "AppletControl.hh"
-#include "AppletWindow.hh"
+#include "Util.hh"
+#include "WindowHints.hh"
+#include "Locale.hh"
 
-#ifdef WIN32
+#if defined(PLATFORM_OS_WIN32)
 #include "W32AppletWindow.hh"
-#include "StatusIcon.hh"
 #include <gdk/gdkwin32.h>
 #include <pbt.h>
 #endif
 
-#include "Menus.hh"
+#if defined(PLATFORM_OS_OSX)
+#include "OSXUtil.hh"
+#include <strings.h>
+#include <mach-o/dyld.h>
+#include <sys/param.h>
+#import <Cocoa/Cocoa.h>
+#import "AppController.h"
+#include <Carbon/Carbon.h>
+#endif
 
-#ifdef HAVE_GCONF
+#if defined(HAVE_GCONF)
 #include <gconf/gconf-client.h>
 #endif
 
-#include "SoundPlayer.hh"
-
-#ifdef HAVE_GNOME
-#include "RemoteControl.hh"
-#include <bonobo.h>
-#include <bonobo/bonobo-xobject.h>
-#ifdef HAVE_GNOMEMM
+#if defined(HAVE_GNOMEMM)
 #include "libgnomeuimm/wrap_init.h"
 #endif
-#endif
 
-#ifdef HAVE_KDE
+#if defined(HAVE_KDE)
 #include <dcopclient.h>
 #include <kapp.h>
 #endif
 
-#ifdef WIN32
-#include "crashlog.h"
-#include "W32Compat.hh"
+#if defined(HAVE_DBUS)
+#include "DBus.hh"
+#include "DBusException.hh"
 #endif
 
-#include <gtkmm/main.h>
-#include <gtkmm/messagedialog.h>
-
-#include <glibmm/refptr.h>
-
 GUI *GUI::instance = NULL;
 
-const string GUI::CFG_KEY_GUI_BLOCK_MODE =  "gui/breaks/block_mode";
 
 //! GUI Constructor.
 /*!
  *  \param argc number of command line parameters.
  *  \param argv all command line parameters.
  */
-GUI::GUI(int argc, char **argv)  :
+GUI::GUI(int argc, char **argv) :
   configurator(NULL),
   core(NULL),
   sound_player(NULL),
@@ -120,14 +129,11 @@ GUI::GUI(int argc, char **argv)  :
   num_heads(-1),
   screen_width(-1),
   screen_height(-1),
-#if !defined(HAVE_GTK_MULTIHEAD) && defined(WIN32)
-  current_monitor(0),
-#endif
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
   grab_wanted(false),
 #endif
   grab_handle(NULL),
-  status_icon(0),
+  status_icon(NULL),
   applet_control(NULL)
 {
   TRACE_ENTER("GUI:GUI");
@@ -164,11 +170,6 @@ GUI::~GUI()
 
   delete sound_player;
 
-#ifdef HAVE_GNOME
-  RemoteControl *control = RemoteControl::get_instance();
-  delete control;
-#endif
-
   TRACE_EXIT();
 }
 
@@ -187,46 +188,32 @@ GUI::main()
 {
   TRACE_ENTER("GUI::main");
 
-#ifdef WIN32
-  // Enable Windows structural exception handling.
-  __try1(exception_handler);
-#endif
-
-  Gtk::Main kit(argc, argv);
-
-#ifdef HAVE_GNOME
-  init_gnome();
-#endif
-#ifdef HAVE_KDE
-  init_kde();
-#endif
-
-#ifdef WIN32
-  // Win32 needs this....
+#if defined (PLATFORM_OS_WIN32) || defined(PLATFORM_OS_OSX)
+  // Win32/OSX need this....
   if (!g_thread_supported())
     {
       g_thread_init(NULL);
     }
 #endif
+  
+  Gtk::Main kit(argc, argv);
 
-#ifdef HAVE_X
-  char *display = gdk_get_display();
-  System::init(display);
-  g_free(display);
-#else
-  System::init();
-#endif
-
-  init_debug();
-  init_nls();
   init_core();
+  init_nls();
+  init_platform();
+  init_debug();
   init_sound_player();
   init_multihead();
+  init_dbus();
   init_gui();
-  init_remote_control();
-
+  
   on_timer();
 
+#if defined(PLATFORM_OS_WIN32)
+  // FIXME: debug, remove later
+  APPEND_TIME( "Workrave started and initialized", "Entering event loop." );
+#endif
+
   // Enter the event loop
   gdk_threads_enter();
   Gtk::Main::run();
@@ -238,11 +225,6 @@ GUI::main()
   delete applet_control;
   applet_control = NULL;
 
-#ifdef WIN32
-  // Disable Windows structural exception handling.
-  __except1;
-#endif
-
   TRACE_EXIT();
 }
 
@@ -253,7 +235,6 @@ GUI::terminate()
 {
   TRACE_ENTER("GUI::terminate");
 
-#ifdef WIN32
   // HACK: Without it status icon keeps on dangling in tray
   // Nicer solution: nicely cleanup complete gui ~GUI()
   if (status_icon)
@@ -261,7 +242,7 @@ GUI::terminate()
       delete status_icon;
       status_icon = 0;
     }
-#endif
+
   CoreFactory::get_configurator()->save();
 
   collect_garbage();
@@ -293,23 +274,12 @@ GUI::close_main_window()
 }
 
 
-//! Toggles the main window.
-void
-GUI::toggle_main_window()
-{
-  if (main_window != NULL)
-    {
-      main_window->toggle_window();
-    }
-}
-
-
 //! Periodic heartbeat.
 bool
 GUI::on_timer()
 {
   std::string tip = get_timers_tooltip();
-
+  
   if (core != NULL)
     {
       core->heartbeat();
@@ -326,12 +296,11 @@ GUI::on_timer()
       applet_control->set_timers_tooltip(tip);
     }
 
-#ifdef WIN32
   if (status_icon)
     {
       status_icon->set_timers_tooltip(tip);
     }
-#endif
+
   heartbeat_signal();
 
   collect_garbage();
@@ -339,37 +308,68 @@ GUI::on_timer()
   return true;
 }
 
-#ifdef NDEBUG
+#if defined(NDEBUG)
 static void my_log_handler(const gchar *log_domain, GLogLevelFlags log_level,
                            const gchar *message, gpointer user_data)
 {
 }
 #endif
 
+void
+GUI::init_platform()
+{
+#if defined(HAVE_GNOME)
+  init_gnome();
+#endif
+#if defined(HAVE_KDE)
+  init_kde();
+#endif
+#if defined(PLATFORM_OS_OSX)
+  [ [ AppController alloc ] init ];
+#endif
+  
+#if defined (PLATFORM_OS_WIN32) || defined(PLATFORM_OS_OSX)
+  // Win32/OSX need this....
+  if (!g_thread_supported())
+    {
+      g_thread_init(NULL);
+    }
+#endif
 
-#ifdef HAVE_GNOME
+#if defined(PLATFORM_OS_UNIX)
+  char *display = gdk_get_display();
+  System::init(display);
+  g_free(display);
+#else
+  System::init();
+#endif
+  
+  srand(time(NULL));
+}
+
+
+#if defined(HAVE_GNOME)
 void
 GUI::init_gnome()
 {
   TRACE_ENTER("GUI::init_gnome");
 
   gnome_init("workrave", VERSION, argc, argv);
-
-#ifdef HAVE_GNOMEMM
+#if defined(HAVE_GNOMEMM)
   Gnome::UI::wrap_init();
 
   Gnome::UI::Client *client = Gnome::UI::Client::master_client();
   if (client != NULL)
     {
-      client->signal_save_yourself().connect(MEMBER_SLOT(*this, &GUI::on_save_yourself));
-      client->signal_die().connect(MEMBER_SLOT(*this, &GUI::on_die));
+      client->signal_save_yourself().connect(sigc::mem_fun(*this, &GUI::on_save_yourself));
+      client->signal_die().connect(sigc::mem_fun(*this, &GUI::on_die));
     }
 
   TRACE_EXIT();
 #endif
 }
 
-#ifdef HAVE_GNOMEMM
+#if defined(HAVE_GNOMEMM)
 void
 GUI::on_die()
 {
@@ -436,7 +436,7 @@ GUI::on_save_yourself(int phase, Gnome::UI::SaveStyle save_style, bool shutdown,
 #endif // defined HAVE_GNOME
 
 
-#ifdef HAVE_KDE
+#if defined(HAVE_KDE)
 void
 GUI::init_kde()
 {
@@ -454,9 +454,9 @@ GUI::init_kde()
 void
 GUI::init_debug()
 {
-#ifdef NDEBUG
-  char *domains[] = { NULL, "Gtk", "GLib", "Gdk", "gtkmm", "GLib-GObject" };
-  for (int i = 0; i < sizeof(domains)/sizeof(char *); i++)
+#if defined(NDEBUG)
+  const char *domains[] = { NULL, "Gtk", "GLib", "Gdk", "gtkmm", "GLib-GObject" };
+  for (unsigned int i = 0; i < sizeof(domains)/sizeof(char *); i++)
     {
       g_log_set_handler(domains[i],
                         (GLogLevelFlags) (G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
@@ -471,14 +471,35 @@ GUI::init_debug()
 void
 GUI::init_nls()
 {
-#ifdef ENABLE_NLS
-#  ifndef HAVE_GNOME
+#if defined(ENABLE_NLS)
+  string language = GUIConfig::get_locale();
+  if (language != "")
+    {
+      g_setenv("LANGUAGE", language.c_str(), 1);
+    }
+
+#  if !defined(HAVE_GNOME)
   gtk_set_locale();
 #  endif
   const char *locale_dir;
-#ifdef WIN32
+
+#if defined(PLATFORM_OS_WIN32)
   string dir = Util::get_application_directory() + "\\lib\\locale";
   locale_dir = dir.c_str();
+#elif defined(PLATFORM_OS_OSX)
+  char locale_path[MAXPATHLEN * 4];
+  char execpath[MAXPATHLEN+1];
+  uint32_t pathsz = sizeof (execpath);
+
+  _NSGetExecutablePath(execpath, &pathsz);
+
+  gchar *dir_path = g_path_get_dirname(execpath);
+  strcpy(locale_path, dir_path);
+  g_free(dir_path);
+
+  // Locale
+  strcat(locale_path, "/../Resources/locale");
+  locale_dir = locale_path;
 #else
   locale_dir = GNOMELOCALEDIR;
 #  endif
@@ -487,9 +508,23 @@ GUI::init_nls()
   setlocale(LC_ALL, "");
 #  endif
 
+#if defined(PLATFORM_OS_WIN32)
+  bindtextdomain("gtk20", locale_dir);
+  bindtextdomain("iso_3166", locale_dir);
+  bindtextdomain("iso_639", locale_dir);
+  bindtextdomain("glib20", locale_dir);
+  bind_textdomain_codeset("gk20", "UTF-8");
+  bind_textdomain_codeset("glib20", "UTF-8");
+  bind_textdomain_codeset("iso_3166", "UTF-8");
+  bind_textdomain_codeset("iso_639", "UTF-8");
+
+  CoreFactory::get_configurator()->add_listener(GUIConfig::CFG_KEY_LOCALE, this);
+#endif
+  
   bindtextdomain(PACKAGE, locale_dir);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
+ 
 #endif
 }
 
@@ -498,20 +533,21 @@ GUI::init_nls()
 void
 GUI::init_core()
 {
-#ifdef HAVE_X
-  char *display_name = gdk_get_display();
-#else
-  char *display_name = NULL;
+  string display_name;
+  
+#if defined(PLATFORM_OS_UNIX)
+  char *display = gdk_get_display();
+  if (display != NULL)
+    {
+      display_name = display;
+    }
 #endif
 
   core = CoreFactory::get_core();
   core->init(argc, argv, this, display_name);
   core->set_core_events_listener(this);
 
-
-// #ifdef HAVE_X
-//    g_free(display_name);
-// #endif
+  GUIConfig::init();
 }
 
 
@@ -520,12 +556,7 @@ GUI::init_multihead()
 {
   TRACE_ENTER("GUI::init_multihead");
 
-#if defined(HAVE_GTK_MULTIHEAD)
   init_gtk_multihead();
-#elif defined(WIN32)
-  init_win32_multihead();
-#endif
-
   if (num_heads == -1)
     {
       init_multihead_mem(1);
@@ -658,7 +689,6 @@ GUI::init_multihead_desktop()
 }
 
 
-#ifdef HAVE_GTK_MULTIHEAD
 void
 GUI::init_gtk_multihead()
 {
@@ -675,28 +705,20 @@ GUI::init_gtk_multihead()
       for (int i = 0; i < num_screens; i++)
         {
           Glib::RefPtr<Gdk::Screen> screen = display->get_screen(i);
-#ifdef HAVE_GTKMM24
           if (screen)
-#else
-            if (!screen.is_null())
-#endif
-              {
-                new_num_heads += screen->get_n_monitors();
-                TRACE_MSG("num monitors on screen " << i << " = " << screen->get_n_monitors());
-              }
+            {
+              new_num_heads += screen->get_n_monitors();
+              TRACE_MSG("num monitors on screen " << i << " = " << screen->get_n_monitors());
+            }
         }
 
       for (int i = 0; i < num_screens; i++)
         {
           Glib::RefPtr<Gdk::Screen> screen = display->get_screen(i);
-#ifdef HAVE_GTKMM24
           if (screen)
-#else
-          if (!screen.is_null())
-#endif
-              {
-                TRACE_MSG("num monitors on screen " << i << " = " << screen->get_n_monitors());
-              }
+            {
+              TRACE_MSG("num monitors on screen " << i << " = " << screen->get_n_monitors());
+            }
         }
 
       init_multihead_mem(new_num_heads);
@@ -705,124 +727,56 @@ GUI::init_gtk_multihead()
       for (int i = 0; i < num_screens; i++)
         {
           Glib::RefPtr<Gdk::Screen> screen = display->get_screen(i);
-#ifdef HAVE_GTKMM24
           if (screen)
-#else
-          if (!screen.is_null())
-#endif
-              {
-                int num_monitors = screen->get_n_monitors();
-                TRACE_MSG("monitors = " << num_monitors);
-                for (int j = 0; j < num_monitors && count < new_num_heads; j++)
-                  {
-                    Gdk::Rectangle rect;
-                    screen->get_monitor_geometry(j, rect);
+            {
+              int num_monitors = screen->get_n_monitors();
+              TRACE_MSG("monitors = " << num_monitors);
+              for (int j = 0; j < num_monitors && count < new_num_heads; j++)
+                {
+                  Gdk::Rectangle rect;
+                  screen->get_monitor_geometry(j, rect);
 
-                    bool overlap = false;
-                    for (int k = 0; !overlap && k < count; k++)
-                      {
-                        Gdk::Rectangle irect = rect;
+                  bool overlap = false;
+                  for (int k = 0; !overlap && k < count; k++)
+                    {
+                      Gdk::Rectangle irect = rect;
 
-                        if (heads[k].screen->get_number() == i)
-                          {
-                            irect.intersect(heads[k].geometry, overlap);
-                          }
-                      }
+                      if (heads[k].screen->get_number() == i)
+                        {
+                          irect.intersect(heads[k].geometry, overlap);
+                        }
+                    }
 
-                    if (!overlap)
-                      {
-                        heads[count].screen = screen;
-                        heads[count].monitor = j;
-                        heads[count].valid = true;
-                        heads[count].count = count;
+                  if (!overlap)
+                    {
+                      heads[count].screen = screen;
+                      heads[count].monitor = j;
+                      heads[count].valid = true;
+                      heads[count].count = count;
 
-                        heads[count].geometry = rect;
-                        count++;
-                      }
+                      heads[count].geometry = rect;
+                      count++;
+                    }
 
-                    TRACE_MSG("Screen #" << i << " Monitor #" << j << "  "
-                              << rect.get_x() << " "
-                              << rect.get_y() << " "
-                              << rect.get_width() << " "
-                              << rect.get_height() << " "
-                              << " intersects " << overlap);
-                  }
-              }
+                  TRACE_MSG("Screen #" << i << " Monitor #" << j << "  "
+                            << rect.get_x() << " "
+                            << rect.get_y() << " "
+                            << rect.get_width() << " "
+                            << rect.get_height() << " "
+                            << " intersects " << overlap);
+                }
+            }
         }
       num_heads = count;
       TRACE_MSG("# Heads = " << num_heads);
     }
   TRACE_EXIT();
 }
-#endif
-
-
-#if !defined(HAVE_GTK_MULTIHEAD) && defined(WIN32)
-BOOL CALLBACK enum_monitor_callback(HMONITOR monitor,
-                                    HDC hdc,
-                                    LPRECT rc,
-                                    LPARAM dwData)
-{
-  GUI *gui = (GUI *) dwData;
-  return gui->enum_monitor_callback(monitor, hdc, rc);
-};
-
-BOOL CALLBACK
-GUI::enum_monitor_callback(HMONITOR monitor, HDC hdc, LPRECT rc)
-{
-  TRACE_ENTER("GUI::enum_monitor_callback");
-  TRACE_MSG(current_monitor << " " << num_heads);
-
-  TRACE_MSG(rc->left << " " << rc->top << " - " << rc->right << " " << rc->bottom);
-
-  if (current_monitor < num_heads)
-    {
-      Gdk::Rectangle &geometry = heads[current_monitor].geometry;
-
-      geometry.set_x(rc->left);
-      geometry.set_y(rc->top);
-      geometry.set_width(rc->right-rc->left+1);
-      geometry.set_height(rc->bottom-rc->top+1);
-
-      heads[current_monitor].valid = true;
-      heads[current_monitor].count = current_monitor;
-
-      current_monitor++;
-    }
-
-  return TRUE;
-};
-
-
-void
-GUI::init_win32_multihead()
-{
-  TRACE_ENTER("GUI::init_win32_multihead");
-
-  if (! W32Compat::IsWindows95())
-    {
-      int new_num_heads = GetSystemMetrics(SM_CMONITORS);
-
-      if (new_num_heads > 1)
-        {
-          init_multihead_mem(new_num_heads);
-
-          current_monitor = 0;
-          W32Compat::EnumDisplayMonitors(NULL, NULL, ::enum_monitor_callback, (LPARAM)this);
-        }
-    }
-}
-
-#endif
-
 
 //! Initializes the GUI
 void
 GUI::init_gui()
 {
-  // Setup the window hints module.
-  WindowHints::init();
-
   tooltips = manage(new Gtk::Tooltips());
   tooltips->enable();
 
@@ -831,79 +785,96 @@ GUI::init_gui()
   // The main status window.
   main_window = new MainWindow();
 
-#ifdef WIN32
-  // Status icon
-  status_icon = new StatusIcon(*main_window);
-#endif
-
   // The applet window.
   applet_control = new AppletControl();
   applet_control->init();
 
-#ifdef HAVE_GNOME
-  AppletWindow *applet_window = applet_control->get_applet_window(AppletControl::APPLET_GNOME);
-  menus->set_applet_window(applet_window);
+  AppletWindow *applet_window = NULL;
+#if defined(HAVE_GNOME)
+  applet_window = applet_control->get_applet_window(AppletControl::APPLET_GNOME);
 #endif
-#ifdef WIN32
-  AppletWindow *applet_window = applet_control->get_applet_window(AppletControl::APPLET_W32);
-  menus->set_applet_window(applet_window);
+#if defined(PLATFORM_OS_WIN32)
+  applet_window = applet_control->get_applet_window(AppletControl::APPLET_W32);
 #endif
 
-#ifdef WIN32
+  menus->init(main_window, applet_window);
+  menus->resync();
+
+  // Status icon
+  status_icon = new StatusIcon(*main_window);
+  
+#ifdef HAVE_DBUS
+  DBus *dbus = CoreFactory::get_dbus();
+
+  if (dbus != NULL && dbus->is_available())
+    {
+      dbus->connect("/org/workrave/Workrave/UI",
+                    "org.workrave.ControlInterface",
+                    menus);
+      
+    }
+#endif
+  
+#if defined(PLATFORM_OS_WIN32)
   win32_init_filter();
 #endif
 
   // Periodic timer.
-  Glib::signal_timeout().connect(MEMBER_SLOT(*this, &GUI::on_timer), 1000);
-
+  Glib::signal_timeout().connect(sigc::mem_fun(*this, &GUI::on_timer), 1000);
 }
 
 
-//! Initializes the CORBA remote control interface.
 void
-GUI::init_remote_control()
+GUI::init_dbus()
 {
-#ifdef HAVE_GNOME
-  if (!bonobo_init(&argc, argv))
-    {
-      g_error (_("I could not initialize Bonobo"));
-    }
+#if defined(HAVE_DBUS)
+  DBus *dbus = CoreFactory::get_dbus();
 
-  RemoteControl *control = RemoteControl::get_instance();
-  if (control == NULL)
+  if (dbus != NULL && dbus->is_available())
     {
-      Gtk::MessageDialog dialog("Could not initialize Workrave. Is Workrave already running?",
-                                false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
-      dialog.show();
-      dialog.run();
-      exit(1);
-    }
+      if (!dbus->is_owner())
+        {
+          Gtk::MessageDialog dialog("Could not initialize Workrave. Is Workrave already running?",
+                                    false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+          dialog.show();
+          dialog.run();
+          exit(1);
+        }
+      
+      try
+        {
+          dbus->register_service("org.workrave.Workrave.Activator");
+          
+          dbus->register_object_path("/org/workrave/Workrave/UI");
 
-  BonoboGenericFactory *factory;
-  factory = bonobo_generic_factory_new("OAFIID:GNOME_Workrave_Factory",
-                                       workrave_component_factory, NULL);
-  bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
+          extern void init_DBusGUI(DBus *dbus);
+          init_DBusGUI(dbus);
+        }
+      catch (DBusException &e)
+        {
+        }
+    }  
 #endif
 }
 
 
 //! Returns a break window for the specified break.
 IBreakWindow *
-GUI::create_break_window(HeadInfo &head, BreakId break_id, bool ignorable)
+GUI::create_break_window(HeadInfo &head, BreakId break_id, BreakWindow::BreakFlags break_flags)
 {
   IBreakWindow *ret = NULL;
-  BlockMode block_mode = get_block_mode();
+  GUIConfig::BlockMode block_mode = GUIConfig::get_block_mode();
   if (break_id == BREAK_ID_MICRO_BREAK)
     {
-      ret = new MicroBreakWindow(head, ignorable, block_mode);
+      ret = new MicroBreakWindow(head, break_flags, block_mode);
     }
   else if (break_id == BREAK_ID_REST_BREAK)
     {
-      ret = new RestBreakWindow(head, ignorable, block_mode);
+      ret = new RestBreakWindow(head, break_flags, block_mode);
     }
   else if (break_id == BREAK_ID_DAILY_LIMIT)
     {
-      ret = new DailyLimitWindow(head, ignorable, block_mode);
+      ret = new DailyLimitWindow(head, break_flags, block_mode);
     }
 
   return ret;
@@ -921,9 +892,9 @@ GUI::init_sound_player()
 void
 GUI::core_event_notify(CoreEvent event)
 {
-  TRACE_ENTER_MSG("GUI::core_event_notify", event)
+  TRACE_ENTER_MSG("GUI::core_event_notify", event);
   // FIXME: HACK
-  ISoundPlayer::Sound snd = (ISoundPlayer::Sound) event;
+  SoundPlayer::SoundEvent snd = (SoundPlayer::SoundEvent) event;
   if (sound_player != NULL)
     {
       TRACE_MSG("play");
@@ -935,13 +906,28 @@ GUI::core_event_notify(CoreEvent event)
 void
 GUI::core_event_operation_mode_changed(const OperationMode m)
 {
-#ifdef WIN32
   if (status_icon)
-    status_icon->set_operation_mode(m);
-#else
-  (void) m;
+    {
+      status_icon->set_operation_mode(m);
+    }
+
+  menus->resync();
+}
+
+void
+GUI::config_changed_notify(const std::string &key)
+{
+  (void) key;
+  
+#if defined(HAVE_LANGUAGE_SELECTION)
+  if (key == GUIConfig::CFG_KEY_LOCALE)
+    {
+      string locale = GUIConfig::get_locale();
+      Locale::set_locale(locale);
+
+      menus->locale_changed();
+    }
 #endif
-  menus->resync_applet();
 }
 
 
@@ -954,7 +940,7 @@ GUI::set_break_response(IBreakResponse *rep)
 
 
 void
-GUI::start_prelude_window(BreakId break_id)
+GUI::create_prelude_window(BreakId break_id)
 {
   hide_break_window();
   init_multihead();
@@ -963,12 +949,8 @@ GUI::start_prelude_window(BreakId break_id)
   active_break_id = break_id;
   for (int i = 0; i < num_heads; i++)
     {
-      PreludeWindow *prelude_window = new PreludeWindow(heads[i], break_id);
-
-      prelude_windows[i] = prelude_window;
-
-      prelude_window->set_response(response);
-      prelude_window->start();
+      prelude_windows[i] = new PreludeWindow(heads[i], break_id);
+      prelude_windows[i]->set_response(response);
     }
 
   active_prelude_count = num_heads;
@@ -976,27 +958,40 @@ GUI::start_prelude_window(BreakId break_id)
 
 
 void
-GUI::start_break_window(BreakId break_id, bool ignorable)
+GUI::create_break_window(BreakId break_id, bool user_initiated)
 {
   TRACE_ENTER_MSG("GUI::start_break_window", num_heads);
   hide_break_window();
   init_multihead();
   collect_garbage();
 
+  BreakWindow::BreakFlags break_flags = BreakWindow::BREAK_FLAGS_NONE;
+  bool ignorable = GUIConfig::get_ignorable(break_id);
+
+  if (user_initiated && !ignorable)
+    {
+      break_flags = BreakWindow::BREAK_FLAGS_POSTPONABLE;
+    }
+  else if (ignorable)
+    {
+      break_flags =  ( BreakWindow::BREAK_FLAGS_POSTPONABLE |
+                       BreakWindow::BREAK_FLAGS_SKIPPABLE);
+    }
+
   active_break_id = break_id;
 
   for (int i = 0; i < num_heads; i++)
     {
-      IBreakWindow *break_window = create_break_window(heads[i], break_id, ignorable);
+      IBreakWindow *break_window = create_break_window(heads[i], break_id, break_flags);
 
       break_windows[i] = break_window;
 
       break_window->set_response(response);
-      break_window->start();
+      break_window->init();
     }
 
   active_break_count = num_heads;
-  if (get_block_mode() != GUI::BLOCK_MODE_NONE)
+  if (GUIConfig::get_block_mode() != GUIConfig::BLOCK_MODE_NONE)
     {
       grab();
     }
@@ -1041,6 +1036,30 @@ GUI::hide_break_window()
 }
 
 
+void
+GUI::show_break_window()
+{
+  TRACE_ENTER("GUI::hide_break_window");
+
+  for (int i = 0; i < active_prelude_count; i++)
+    {
+      if (prelude_windows[i] != NULL)
+        {
+          prelude_windows[i]->start();
+        }
+    }
+  for (int i = 0; i < active_break_count; i++)
+    {
+      if (break_windows[i] != NULL)
+        {
+          break_windows[i]->start();
+        }
+    }
+
+  TRACE_EXIT();
+}
+
+
 void
 GUI::refresh_break_window()
 {
@@ -1152,23 +1171,6 @@ GUI::collect_garbage()
 }
 
 
-GUI::BlockMode
-GUI::get_block_mode()
-{
-  int mode;
-  CoreFactory::get_configurator()
-    ->get_value_default(CFG_KEY_GUI_BLOCK_MODE, &mode, BLOCK_MODE_INPUT);
-  return (BlockMode) mode;
-}
-
-void
-GUI::set_block_mode(BlockMode mode)
-{
-  CoreFactory::get_configurator()
-    ->set_value(CFG_KEY_GUI_BLOCK_MODE, int(mode));
-}
-
-
 //! Grabs the pointer and the keyboard.
 bool
 GUI::grab()
@@ -1183,17 +1185,17 @@ GUI::grab()
           windows[i] = window->gobj();
         }
 
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
       grab_wanted = true;
 #endif
       if (! grab_handle)
         {
           grab_handle = WindowHints::grab(active_break_count, windows);
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
           if (! grab_handle && !grab_retry_connection.connected())
             {
               grab_retry_connection =
-                Glib::signal_timeout().connect(MEMBER_SLOT(*this, &GUI::on_grab_retry_timer), 2000);
+                Glib::signal_timeout().connect(sigc::mem_fun(*this, &GUI::on_grab_retry_timer), 2000);
             }
 #endif
         }
@@ -1206,12 +1208,12 @@ GUI::grab()
 void
 GUI::ungrab()
 {
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
   grab_wanted = false;
 #endif
   if (grab_handle)
     {
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
       grab_retry_connection.disconnect();
 #endif
       WindowHints::ungrab(grab_handle);
@@ -1225,14 +1227,14 @@ GUI::interrupt_grab()
 {
   if (grab_handle)
     {
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
       grab_wanted = true;
 
       WindowHints::ungrab(grab_handle);
       grab_handle = NULL;
       if (!grab_retry_connection.connected())
         {
-          Glib::signal_timeout().connect(MEMBER_SLOT(*this, &GUI::on_grab_retry_timer), 2000);
+          Glib::signal_timeout().connect(sigc::mem_fun(*this, &GUI::on_grab_retry_timer), 2000);
         }
 #endif
     }
@@ -1240,7 +1242,7 @@ GUI::interrupt_grab()
 
 
 
-#ifdef HAVE_X
+#if defined(PLATFORM_OS_UNIX)
 //! Reattempt to get the grab
 bool
 GUI::on_grab_retry_timer()
@@ -1385,19 +1387,18 @@ GUI::get_timers_tooltip()
   ICore *core = CoreFactory::get_core();
   for (int count = 0; count < BREAK_ID_SIZEOF; count++)
     {
-      IBreak *break_data = core->get_break(BreakId(count));
-      ITimer *timer = break_data->get_timer();
-      bool on = break_data->get_break_enabled();
+      IBreak *b = core->get_break(BreakId(count));
+      bool on = b->is_enabled();
 
-      if (timer != NULL && on)
+      if (b != NULL && on)
         {
           // Collect some data.
-          time_t maxActiveTime = timer->get_limit();
-          time_t activeTime = timer->get_elapsed_time();
+          time_t maxActiveTime = b->get_limit();
+          time_t activeTime = b->get_elapsed_time();
           std::string text;
 
           // Set the text
-          if (timer->is_limit_enabled() && maxActiveTime != 0)
+          if (b->is_limit_enabled() && maxActiveTime != 0)
             {
               text = Text::time_to_string(maxActiveTime - activeTime);
             }
@@ -1406,7 +1407,7 @@ GUI::get_timers_tooltip()
               text = Text::time_to_string(activeTime);
             }
 
-#ifndef WIN32
+#if !defined(PLATFORM_OS_WIN32)
           // Win32 tip is limited in length
           if (tip == "")
             {
@@ -1427,7 +1428,7 @@ GUI::get_timers_tooltip()
 }
 
 
-#ifdef WIN32
+#if defined(PLATFORM_OS_WIN32)
 void
 GUI::win32_init_filter()
 {
@@ -1452,6 +1453,43 @@ GUI::win32_filter_func (void     *xevent,
     case WM_POWERBROADCAST:
       {
         TRACE_MSG("WM_POWERBROADCAST " << msg->wParam << " " << msg->lParam);
+
+#if defined(PLATFORM_OS_WIN32)
+// FIXME: debug, remove later
+switch (msg->wParam)
+{
+case PBT_APMBATTERYLOW:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMBATTERYLOW");
+break;
+case PBT_APMOEMEVENT:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMOEMEVENT");
+break;
+case PBT_APMPOWERSTATUSCHANGE:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMPOWERSTATUSCHANGE");
+break;
+case PBT_APMQUERYSUSPEND:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMQUERYSUSPEND");
+break;
+case PBT_APMQUERYSUSPENDFAILED:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMQUERYSUSPENDFAILED");
+break;
+case PBT_APMRESUMEAUTOMATIC:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMRESUMEAUTOMATIC");
+break;
+case PBT_APMRESUMECRITICAL:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMRESUMECRITICAL");
+break;
+case PBT_APMRESUMESUSPEND:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMRESUMESUSPEND");
+break;
+case PBT_APMSUSPEND:
+APPEND_TIME("WM_POWERBROADCAST", "PBT_APMSUSPEND");
+break;
+default:
+APPEND_TIME("WM_POWERBROADCAST", "<UNKNOWN MESSAGE> : " << hex << msg->wParam );
+}
+#endif
+
           switch (msg->wParam)
             {
             case PBT_APMQUERYSUSPEND:
@@ -1494,6 +1532,14 @@ GUI::win32_filter_func (void     *xevent,
       }
       break;
 
+    case WM_TIMECHANGE:
+      {
+        TRACE_MSG("WM_TIMECHANGE " << msg->wParam << " " << msg->lParam);
+        ICore *core = CoreFactory::get_core();
+        core->time_changed();
+      }
+      break;
+
     default:
       W32AppletWindow *applet_window =
         static_cast<W32AppletWindow*>
@@ -1504,12 +1550,10 @@ GUI::win32_filter_func (void     *xevent,
         }
     }
 
-#ifdef WIN32
   if (ret != GDK_FILTER_REMOVE && gui->status_icon)
     {
       ret = gui->status_icon->win32_filter_func(xevent, event);
     }
-#endif
 
   TRACE_EXIT();
   return ret;
