@@ -1,20 +1,23 @@
 // GnomeAppletWindow.cc --- Applet info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: GnomeAppletWindow.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: GnomeAppletWindow.cc 1513 2008-03-26 18:16:44Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -36,8 +39,6 @@ static const char rcsid[] = "$Id: GnomeAppletWindow.cc 1298 2007-08-30 14:18:04Z
 #include "Menus.hh"
 #include "System.hh"
 
-#include "RemoteControl.hh"
-
 #include "ICore.hh"
 #include "CoreFactory.hh"
 
@@ -47,6 +48,9 @@ static const char rcsid[] = "$Id: GnomeAppletWindow.cc 1298 2007-08-30 14:18:04Z
 #include <X11/X.h>
 #include <X11/Xlib.h>
 
+#include "DBusException.hh"
+#include "DBusGnomeApplet.hh"
+
 //! Constructor.
 /*!
  *  \param gui the main GUI entry point.
@@ -56,9 +60,9 @@ GnomeAppletWindow::GnomeAppletWindow(AppletControl *control) :
   view(NULL),
   plug(NULL),
   container(NULL),
-  applet_control(NULL),
   applet_orientation(ORIENTATION_UP),
   applet_size(0),
+  applet_control(NULL),
   control(control),
   applet_active(false)
 {
@@ -79,49 +83,52 @@ GnomeAppletWindow::~GnomeAppletWindow()
 AppletWindow::AppletState
 GnomeAppletWindow::activate_applet()
 {
-  TRACE_ENTER("GnomeAppletWindow::init_gnome_applet");
+  TRACE_ENTER("GnomeAppletWindow::activate_applet");
   bool ok = true;
 
   if (!applet_active)
     {
-      // Initialize bonobo activation.
-      bonobo_activate();
-
-      CORBA_Environment ev;
-      CORBA_exception_init (&ev);
-
-      // Connect to the applet.
-      // FIXME: leak
-      if (applet_control == NULL)
-        {
-          applet_control = bonobo_activation_activate_from_id("OAFIID:GNOME_Workrave_AppletControl",
-                                                              Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, &ev);
-        }
-
-      // Socket ID of the applet.
       long id = 0;
-      if (applet_control != NULL && !BONOBO_EX(&ev))
+      
+      try
         {
-          id = GNOME_Workrave_AppletControl_get_socket_id(applet_control, &ev);
-          ok = !BONOBO_EX(&ev);
-        }
+          DBus *dbus = CoreFactory::get_dbus();
 
-      if (ok)
+          if (dbus != NULL && dbus->is_available())
+            {
+              dbus->connect("/org/workrave/Workrave/UI",
+                            "org.workrave.GnomeAppletSupportInterface",
+                            this);
+            }
+      
+          applet_control = org_workrave_GnomeAppletInterface::instance(dbus,
+                                                                       "org.workrave.Workrave.GnomeApplet",
+                                                                       "/org/workrave/Workrave/GnomeApplet");
+          if (applet_control != NULL)
+            {
+              id = applet_control->GetSocketId();
+              applet_size = applet_control->GetSize();
+              applet_orientation =  (Orientation) applet_control->GetOrientation();
+
+#ifndef HAVE_EXERCISES
+              applet_control->SetMenuActive("/commands/Exercises", false);
+#endif
+#ifndef HAVE_DISTRIBUTION
+              applet_control->SetMenuActive("/commands/Network", false);
+#endif
+            }
+        }
+      catch (DBusException)
         {
-          // Retrieve applet size.
-          applet_size = GNOME_Workrave_AppletControl_get_size(applet_control, &ev);
-          ok = !BONOBO_EX(&ev);
+          if (applet_control != NULL)
+            {
+              delete applet_control;
+              applet_control = NULL;
+            }
+          
+          ok = false;
         }
 
-      if (ok)
-        {
-          // Retrieve applet orientation.
-          applet_orientation =  (Orientation)
-            GNOME_Workrave_AppletControl_get_orientation(applet_control, &ev);
-          ok = !BONOBO_EX(&ev);
-        }
-
-
       if (ok)
         {
           // Initialize applet GUI.
@@ -136,46 +143,31 @@ GnomeAppletWindow::activate_applet()
 
           plug->set_events(plug->get_events() | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
 
-          plug->signal_embedded().connect(MEMBER_SLOT(*this, &GnomeAppletWindow::on_embedded));
-          plug->signal_delete_event().connect(MEMBER_SLOT(*this, &GnomeAppletWindow::delete_event));
+          plug->signal_embedded().connect(sigc::mem_fun(*this, &GnomeAppletWindow::on_embedded));
+          plug->signal_delete_event().connect(sigc::mem_fun(*this, &GnomeAppletWindow::delete_event));
 
           // Gtkmm does not wrap this event....
           g_signal_connect(G_OBJECT(plug->gobj()), "destroy-event",
                            G_CALLBACK(GnomeAppletWindow::destroy_event), this);
 
-          view = new TimerBoxGtkView();
+          view = new TimerBoxGtkView(Menus::MENU_NONE);
           timer_box_view = view;
           timer_box_control = new TimerBoxControl("applet", *timer_box_view);
 
           view->set_geometry(applet_orientation, applet_size);
           view->show_all();
 
-          plug->signal_button_press_event().connect(MEMBER_SLOT(*this, &GnomeAppletWindow::on_button_press_event));
-          plug->signal_button_release_event().connect(MEMBER_SLOT(*this, &GnomeAppletWindow::on_button_press_event));
+          plug->signal_button_press_event().connect(sigc::mem_fun(*this, &GnomeAppletWindow::on_button_press_event));
+          plug->signal_button_release_event().connect(sigc::mem_fun(*this, &GnomeAppletWindow::on_button_press_event));
 
           container->add(*view);
           container->show_all();
           plug->show_all();
 
-          Menus *menus = Menus::get_instance();
-
-          // Tray menu
-          menus->create_menu(Menus::MENU_APPLET);
-
-#ifndef HAVE_EXERCISES
-          GNOME_Workrave_AppletControl_set_menu_active(applet_control, "/commands/Exercises", false, &ev);
-#endif
-#ifndef HAVE_DISTRIBUTION
-          GNOME_Workrave_AppletControl_set_menu_active(applet_control, "/commands/Network", false, &ev);
-#endif
         }
 
-      if (!ok)
-        {
-          applet_control = NULL;
-        }
-
-      CORBA_exception_free(&ev);
+      Menus *menus = Menus::get_instance();
+      menus->resync();
     }
 
   if (ok)
@@ -219,7 +211,8 @@ GnomeAppletWindow::deactivate_applet()
       timer_box_view = NULL;
       view = NULL;
 
-      applet_control = NULL; // FIXME: free memory.
+      delete applet_control;
+      applet_control = NULL;
     }
 
   applet_active = false;
@@ -249,53 +242,37 @@ GnomeAppletWindow::fire_gnome_applet()
 }
 
 
-//! Sets the applet control callback interface.
-void
-GnomeAppletWindow::set_applet_control(GNOME_Workrave_AppletControl applet_control)
-{
-  TRACE_ENTER("GnomeAppletWindow::set_applet_control");
-  if (this->applet_control != NULL)
-    {
-      // FIXME: free old interface
-    }
-
-  this->applet_control = applet_control;
-
-  control->show(AppletControl::APPLET_GNOME);
-  TRACE_EXIT();
-}
-
-
 //! Sets the state of a toggle menu item.
 void
 GnomeAppletWindow::set_menu_active(int menu, bool active)
 {
   TRACE_ENTER_MSG("GnomeAppletWindow::set_menu_active", menu << " " << active);
-  CORBA_Environment ev;
 
-  if (applet_control != NULL)
+  try
     {
-      CORBA_exception_init (&ev);
-      switch (menu)
+      if (applet_control != NULL)
         {
-        case Menus::MENUSYNC_MODE_NORMAL:
-          GNOME_Workrave_AppletControl_set_menu_status(applet_control, "/commands/Normal",
-                                                       active, &ev);
-          break;
-        case Menus::MENUSYNC_MODE_SUSPENDED:
-          GNOME_Workrave_AppletControl_set_menu_status(applet_control, "/commands/Suspended",
-                                                       active, &ev);
-          break;
-        case Menus::MENUSYNC_MODE_QUIET:
-          GNOME_Workrave_AppletControl_set_menu_status(applet_control, "/commands/Quiet",
-                                                       active, &ev);
-          break;
-        case Menus::MENUSYNC_SHOW_LOG:
-          GNOME_Workrave_AppletControl_set_menu_status(applet_control, "/commands/ShowLog", active, &ev);
-          break;
+          switch (menu)
+            {
+            case MENUSYNC_MODE_NORMAL:
+              applet_control->SetMenuStatus("/commands/Normal", active);
+              break;
+            case MENUSYNC_MODE_SUSPENDED:
+              applet_control->SetMenuStatus("/commands/Suspended", active);
+              break;
+            case MENUSYNC_MODE_QUIET:
+              applet_control->SetMenuStatus("/commands/Quiet", active);
+              break;
+            case MENUSYNC_SHOW_LOG:
+              applet_control->SetMenuStatus("/commands/ShowLog", active);
+              break;
+            }
         }
-      CORBA_exception_free(&ev);
     }
+  catch(DBusException)
+    {
+    }
+
   TRACE_EXIT();
 }
 
@@ -304,33 +281,34 @@ GnomeAppletWindow::set_menu_active(int menu, bool active)
 bool
 GnomeAppletWindow::get_menu_active(int menu)
 {
-  CORBA_Environment ev;
   bool ret = false;
 
-  if (applet_control != NULL)
+  try
     {
-      CORBA_exception_init (&ev);
-      switch (menu)
+      if (applet_control != NULL)
         {
-        case Menus::MENUSYNC_MODE_NORMAL:
-          ret = GNOME_Workrave_AppletControl_get_menu_status(applet_control, "/commands/Normal",
-                                                             &ev);
-          break;
-        case Menus::MENUSYNC_MODE_SUSPENDED:
-          ret = GNOME_Workrave_AppletControl_get_menu_status(applet_control, "/commands/Suspended",
-                                                             &ev);
-          break;
-        case Menus::MENUSYNC_MODE_QUIET:
-          ret = GNOME_Workrave_AppletControl_get_menu_status(applet_control, "/commands/Quiet",
-                                                             &ev);
-          break;
-        case Menus::MENUSYNC_SHOW_LOG:
-          ret = GNOME_Workrave_AppletControl_get_menu_status(applet_control, "/commands/ShowLog",
-                                                             &ev);
-          break;
+          switch (menu)
+            {
+            case MENUSYNC_MODE_NORMAL:
+              applet_control->GetMenuStatus("/commands/Normal", ret);
+              break;
+            case MENUSYNC_MODE_SUSPENDED:
+              applet_control->GetMenuStatus("/commands/Suspended", ret);
+              break;
+            case MENUSYNC_MODE_QUIET:
+              applet_control->GetMenuStatus("/commands/Quiet", ret);
+              break;
+            case MENUSYNC_SHOW_LOG:
+              applet_control->GetMenuStatus("/commands/ShowLog", ret);
+              break;
+            }
         }
-      CORBA_exception_free(&ev);
     }
+  catch(DBusException)
+    {
+      ret = false;
+    }
+
   return ret;
 }
 
@@ -572,5 +550,9 @@ GnomeAppletWindow::on_embedded()
   TRACE_ENTER("GnomeAppletWindow::on_embedded");
   control->set_applet_state(AppletControl::APPLET_GNOME,
                             AppletWindow::APPLET_STATE_VISIBLE);
+
+  Menus *menus = Menus::get_instance();
+  menus->resync();
+  
   TRACE_EXIT();
 }
