@@ -1,19 +1,22 @@
 // GtkUtil.cc --- Gtk utilities
 //
-// Copyright (C) 2003, 2004, 2005, 2007 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2003, 2004, 2005, 2007, 2008 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: GtkUtil.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: GtkUtil.cc 1519 2008-03-31 17:06:49Z rcaelers $
 //
 
 #ifdef HAVE_CONFIG_H
@@ -156,8 +159,8 @@ Gtk::Widget *
 GtkUtil::create_label_for_break(BreakId id)
 {
   // FIXME: duplicate:
-  char *icons[] = { "timer-micro-break.png", "timer-rest-break.png", "timer-daily.png" };
-  char *labels[] = { _("Micro-break"), _("Rest break"), _("Daily limit") };
+  const char *icons[] = { "timer-micro-break.png", "timer-rest-break.png", "timer-daily.png" };
+  const char *labels[] = { _("Micro-break"), _("Rest break"), _("Daily limit") };
 
   string icon = Util::complete_directory(string(icons[id]), Util::SEARCH_PATH_IMAGES);
 
@@ -243,17 +246,6 @@ GtkUtil::create_label(string text, bool bold)
 }
 
 
-
-void
-GtkUtil::set_wmclass(Gtk::Window &window, string class_postfix)
-{
-  string s = gdk_get_program_class();
-  s += class_postfix;
-
-  window.set_wmclass(g_get_prgname(), s);
-}
-
-
 /*
  * Returns a copy of pixbuf mirrored and or flipped.
  * TO do a 180 degree rotations set both mirror and flipped TRUE
@@ -343,30 +335,15 @@ GtkUtil::center_window(Gtk::Window &window, HeadInfo &head)
 
   if (head.valid)
     {
-#ifdef HAVE_GTKMM24
       Gtk::Requisition size;
       window.size_request(size);
-#else
-      GtkRequisition size;
-      window.size_request(&size);
-#endif
-
-#ifdef WIN32
-      TRACE_MSG(
-                head.geometry.get_width() << "x" << head.geometry.get_height() << " +" <<
-                head.geometry.get_x() << "+" << head.geometry.get_y() << " " <<
-                size.width << " " << size.height);
-#else
-      TRACE_MSG(
-                head.monitor << " : " <<
-                head.geometry.get_width() << "x" << head.geometry.get_height() << " +" <<
-                head.geometry.get_x() << "+" << head.geometry.get_y() << " " <<
-                size.width << " " << size.height);
-#endif
 
       int x = head.geometry.get_x() + (head.geometry.get_width() - size.width) / 2;
       int y = head.geometry.get_y() + (head.geometry.get_height() - size.height) / 2;
 
+      TRACE_MSG(head.geometry.get_x() << " "  << head.geometry.get_width() << " " << size.width);
+      TRACE_MSG(head.geometry.get_y() << " "  << head.geometry.get_height() << " " << size.height);
+
       window.set_position(Gtk::WIN_POS_NONE);
       window.move(x, y);
     }
