@@ -1,20 +1,23 @@
 // MainWindow.cc --- Main info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: MainWindow.cc 1311 2007-09-04 19:09:08Z rcaelers $";
+static const char rcsid[] = "$Id: MainWindow.cc 1537 2008-06-08 19:02:40Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -22,7 +25,7 @@ static const char rcsid[] = "$Id: MainWindow.cc 1311 2007-09-04 19:09:08Z rcaele
 #include "config.h"
 #endif
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 #include <gdk/gdkwin32.h>
 #include <shellapi.h>
 #endif
@@ -48,14 +51,10 @@ static const char rcsid[] = "$Id: MainWindow.cc 1311 2007-09-04 19:09:08Z rcaele
 #include "TimeBar.hh"
 #include "GUI.hh"
 #include "Util.hh"
-#ifdef WIN32
-#include "StatusIcon.hh"
-#endif
 #include "Text.hh"
 
 #include "CoreFactory.hh"
 #include "IConfigurator.hh"
-#include "ITimer.hh"
 #include "IStatistics.hh"
 
 #ifdef HAVE_DISTRIBUTION
@@ -63,7 +62,6 @@ static const char rcsid[] = "$Id: MainWindow.cc 1311 2007-09-04 19:09:08Z rcaele
 #include "NetworkLogDialog.hh"
 #endif
 
-
 #include "Menus.hh"
 
 #include <gtkmm/menu.h>
@@ -71,26 +69,18 @@ static const char rcsid[] = "$Id: MainWindow.cc 1311 2007-09-04 19:09:08Z rcaele
 
 #include "gtk/gtkmenu.h"
 
-
-const string MainWindow::CFG_KEY_MAIN_WINDOW = "gui/main_window";
+const string MainWindow::CFG_KEY_MAIN_WINDOW               = "gui/main_window";
 const string MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP = "gui/main_window/always_on_top";
+const string MainWindow::CFG_KEY_MAIN_WINDOW_START_IN_TRAY = "gui/main_window/start_in_tray";
+const string MainWindow::CFG_KEY_MAIN_WINDOW_X             = "gui/main_window/x";
+const string MainWindow::CFG_KEY_MAIN_WINDOW_Y             = "gui/main_window/y";
+const string MainWindow::CFG_KEY_MAIN_WINDOW_HEAD          = "gui/main_window/head";
 
-const string MainWindow::CFG_KEY_MAIN_WINDOW_START_IN_TRAY
-= "gui/main_window/start_in_tray";
-
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 const char *WIN32_MAIN_CLASS_NAME = "Workrave";
 const UINT MYWM_TRAY_MESSAGE = WM_USER +0x100;
 #endif
 
-const string MainWindow::CFG_KEY_MAIN_WINDOW_X
-= "gui/main_window/x";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_Y
-= "gui/main_window/y";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_HEAD
-= "gui/main_window/head";
-
-
 
 //! Constructor.
 /*!
@@ -108,7 +98,7 @@ MainWindow::MainWindow() :
   window_head_location(-1, -1),
   window_relocated_location(-1, -1)
 {
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   leader = NULL;
 #endif
   init();
@@ -120,10 +110,10 @@ MainWindow::~MainWindow()
 {
   TRACE_ENTER("MainWindow::~MainWindow");
   delete timer_box_control;
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   win32_exit();
 #endif
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   delete leader;
 #endif
 
@@ -141,9 +131,9 @@ MainWindow::init()
 
   list<Glib::RefPtr<Gdk::Pixbuf> > icons;
 
-  char *icon_files[] = { "workrave-icon-small.png",
-                         "workrave-icon-medium.png",
-                         "workrave-icon-large.png" };
+  const char *icon_files[] = { "workrave-icon-small.png",
+                               "workrave-icon-medium.png",
+                               "workrave-icon-large.png" };
 
   for (unsigned int i = 0; i < sizeof(icon_files) / sizeof(char *); i++)
     {
@@ -164,11 +154,7 @@ MainWindow::init()
 
   enabled = TimerBoxControl::is_enabled("main_window");
 
-  Menus *menus = Menus::get_instance();
-  menus->set_main_window(this);
-  menus->create_menu(Menus::MENU_MAINWINDOW);
-
-  timer_box_view = manage(new TimerBoxGtkView());
+  timer_box_view = manage(new TimerBoxGtkView(Menus::MENU_MAINWINDOW));
   timer_box_control = new TimerBoxControl("main_window", *timer_box_view);
   timer_box_view->set_geometry(ORIENTATION_LEFT, -1);
   timer_box_control->update();
@@ -190,7 +176,7 @@ MainWindow::init()
 
   // (end window decorators)
 
-#ifdef HAVE_X
+#ifdef PLATFORM_OS_UNIX
   // HACK. this sets a different group leader in the WM_HINTS....
   // Without this hack, metacity makes ALL windows on-top.
   leader = new Gtk::Window(Gtk::WINDOW_POPUP);
@@ -202,7 +188,7 @@ MainWindow::init()
   stick();
   setup();
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 
   win32_init();
   set_gravity(Gdk::GRAVITY_STATIC);
@@ -283,16 +269,19 @@ MainWindow::setup()
       if (enabled)
         {
           open_window();
- 
-
-          WindowHints::set_always_on_top(Gtk::Widget::gobj(), always_on_top);
         }
       else
         {
           close_window();
         }
     }
-  if (enabled && always_on_top)
+
+  if (is_visible())
+    {
+      WindowHints::set_always_on_top(this, always_on_top);
+    }
+  
+  if (is_visible() && always_on_top)
     {
       raise();
     }
@@ -323,8 +312,9 @@ MainWindow::open_window()
   TRACE_ENTER("MainWindow::open_window");
   if (timer_box_view->get_visible_count() > 0)
     {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
       win32_show(true);
+      show_all();
 #else
       show_all();
       deiconify();
@@ -343,6 +333,8 @@ MainWindow::open_window()
       TRACE_MSG("moving to " << x << " " << y);
       move(x, y);
 
+      bool always_on_top = get_always_on_top();
+      WindowHints::set_always_on_top(this, always_on_top);
     }
   TRACE_EXIT();
 }
@@ -353,8 +345,10 @@ MainWindow::open_window()
 void
 MainWindow::close_window()
 {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   win32_show(false);
+#elif defined(PLATFORM_OS_OSX)
+  hide_all();
 #else
   if (applet_active)
     {
@@ -368,21 +362,13 @@ MainWindow::close_window()
 }
 
 
-//! Toggles the main window.
-void
-MainWindow::toggle_window()
-{
-  TimerBoxControl::set_enabled("main_window", !enabled);
-}
-
-
 //! User has closed the main window.
 bool
 MainWindow::on_delete_event(GdkEventAny *)
 {
   TRACE_ENTER("MainWindow::on_delete_event");
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   win32_show(false);
 #else
   GUI *gui = GUI::get_instance();
@@ -405,10 +391,13 @@ MainWindow::on_delete_event(GdkEventAny *)
       close_window();
       TimerBoxControl::set_enabled("main_window", false);
     }
+#elif defined(PLATFORM_OS_OSX) 
+  close_window();
+  TimerBoxControl::set_enabled("main_window", false);
 #else
   gui->terminate();
 #endif // HAVE_GNOME || HAVE_KDE
-#endif // WIN32
+#endif // PLATFORM_OS_WIN32
 
   TRACE_EXIT();
   return true;
@@ -423,39 +412,25 @@ MainWindow::on_button_press_event(GdkEventButton *event)
   TRACE_ENTER("MainWindow::on_button_press_event");
   bool ret = false;
 
+  (void) event;
+  
+#ifndef PLATFORM_OS_OSX
+  // No popup menu on OS X
+  
   if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
     {
       Menus::get_instance()->popup(Menus::MENU_MAINWINDOW,
                                    event->button, event->time);
       ret = true;
     }
-
+#endif
+  
   TRACE_EXIT();
   return ret;
 }
 
-
-
-bool
-MainWindow::on_window_state_event(GdkEventWindowState *event)
-{
-  TRACE_ENTER("MainWindow::on_window_state_event");
-  if (event != NULL)
-    {
-      if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
-        {
-          bool iconified = event->new_window_state & GDK_WINDOW_STATE_ICONIFIED;
-          TimerBoxControl::set_enabled("main_window", !iconified);
-        }
-    }
-
-  TRACE_EXIT();
-  return true;
-}
-
-
 void
-MainWindow::config_changed_notify(string key)
+MainWindow::config_changed_notify(const string &key)
 {
   TRACE_ENTER_MSG("MainWindow::config_changed_notify", key);
   if (key != CFG_KEY_MAIN_WINDOW_HEAD
@@ -467,14 +442,15 @@ MainWindow::config_changed_notify(string key)
   TRACE_EXIT();
 }
 
+
 bool
 MainWindow::get_always_on_top()
 {
   bool rc;
   CoreFactory::get_configurator()
-    ->get_value_default(MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP,
-                        &rc,
-                        false);
+    ->get_value_with_default(MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP,
+                             rc,
+                             false);
   return rc;
 }
 
@@ -492,7 +468,7 @@ MainWindow::get_start_in_tray()
 {
   bool rc;
   CoreFactory::get_configurator()
-    ->get_value_default(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, &rc, false);
+    ->get_value_with_default(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, rc, false);
   return rc;
 }
 
@@ -505,7 +481,7 @@ MainWindow::set_start_in_tray(bool b)
 }
 
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 void
 MainWindow::win32_show(bool b)
 {
@@ -611,9 +587,9 @@ MainWindow::get_start_position(int &x, int &y, int &head)
   TRACE_ENTER("MainWindow::get_start_position");
   // FIXME: Default to right-bottom instead of 256x256
   IConfigurator *cfg = CoreFactory::get_configurator();
-  cfg->get_value_default(CFG_KEY_MAIN_WINDOW_X, &x, 256);
-  cfg->get_value_default(CFG_KEY_MAIN_WINDOW_Y, &y, 256);
-  cfg->get_value_default(CFG_KEY_MAIN_WINDOW_HEAD, &head, 0);
+  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_X, x, 256);
+  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_Y, y, 256);
+  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_HEAD, head, 0);
   if (head < 0)
     {
       head = 0;
@@ -671,23 +647,21 @@ void
 MainWindow::set_applet_active(bool a)
 {
   TRACE_ENTER_MSG("MainWindow::set_applet_active", a);
-  if (applet_active != a)
-    {
-      applet_active = a;
+  applet_active = a;
 
-      if (!enabled)
+  if (!enabled)
+    {
+      if (applet_active)
         {
-          if (applet_active)
-            {
-              hide_all();
-            }
-          else
-            {
-              iconify();
-              show_all();
-            }
+          hide_all();
+        }
+      else
+        {
+          iconify();
+          show_all();
         }
     }
+
   TRACE_EXIT();
 }
 
@@ -712,7 +686,7 @@ MainWindow::locate_window(GdkEventConfigure *event)
 
   (void) event;
 
-#ifndef WIN32
+#ifndef PLATFORM_OS_WIN32
   // Returns bogus results on windows...sometime.
   if (event != NULL)
     {
@@ -726,6 +700,8 @@ MainWindow::locate_window(GdkEventConfigure *event)
   else
 #endif
     {
+      (void) event;
+
       get_position(x, y);
 
       GtkRequisition req;
@@ -751,6 +727,8 @@ MainWindow::locate_window(GdkEventConfigure *event)
     {
       window_location.set_x(x);
       window_location.set_y(y);
+      window_relocated_location.set_x(x);
+      window_relocated_location.set_y(y);
 
       int head = GUI::get_instance()->map_to_head(x, y);
       TRACE_MSG("main window head = " << x << " " << y);
@@ -831,7 +809,7 @@ MainWindow::relocate_window(int width, int height)
   TRACE_EXIT();
 }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 
 LRESULT CALLBACK
 MainWindow::win32_window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
