@@ -1,20 +1,23 @@
 // MicroBreakWindow.cc --- window for the microbreak
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: MicroBreakWindow.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: MicroBreakWindow.cc 1513 2008-03-26 18:16:44Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,25 +32,26 @@ static const char rcsid[] = "$Id: MicroBreakWindow.cc 1298 2007-08-30 14:18:04Z
 #include "nls.h"
 #include "debug.hh"
 
+#include "IBreak.hh"
+#include "GUI.hh"
 #include "CoreFactory.hh"
 #include "MicroBreakWindow.hh"
 #include "WindowHints.hh"
 #include "TimeBar.hh"
-#include "ITimer.hh"
 #include "IBreakResponse.hh"
 #include "Util.hh"
 #include "GtkUtil.hh"
 #include "Text.hh"
 #include "Hig.hh"
-#include "IActivityMonitor.hh"
 #include "Frame.hh"
 
 //! Construct a new Microbreak window.
-MicroBreakWindow::MicroBreakWindow(HeadInfo &head, bool ignorable, GUI::BlockMode mode) :
-  BreakWindow(BREAK_ID_MICRO_BREAK, head, ignorable, mode),
+MicroBreakWindow::MicroBreakWindow(HeadInfo &head, BreakFlags break_flags, GUIConfig::BlockMode mode) :
+  BreakWindow(BREAK_ID_MICRO_BREAK, head, break_flags, mode),
   progress_value(0),
   progress_max_value(0),
-  is_flashing(false)
+  is_flashing(false),
+  fixed_size(false)
 {
   set_title(_("Micro-break"));
 }
@@ -57,7 +61,7 @@ MicroBreakWindow::create_gui()
 {
   // Time bar
   time_bar = manage(new TimeBar);
-  time_bar->set_text("Microbreak 0:32"); // FIXME
+  time_bar->set_text("Microbreak 0:32"); // FIXME:
 
   // Label
   label = manage(new Gtk::Label());
@@ -79,26 +83,33 @@ MicroBreakWindow::create_gui()
 
   // Button box at the bottom.
   ICore *core = CoreFactory::get_core();
-  ITimer *restbreak_timer =  core->get_timer(BREAK_ID_REST_BREAK);
-  bool has_rb = restbreak_timer->get_state() != ITimer::STATE_INVALID;
-  if (ignorable_break || has_rb)
+  IBreak *restbreak =  core->get_break(BREAK_ID_REST_BREAK);
+  if ((break_flags != BREAK_FLAGS_NONE) || restbreak->is_enabled())
     {
       Gtk::HBox *button_box;
-      if (ignorable_break)
+      if (break_flags != BREAK_FLAGS_NONE)
         {
           button_box = manage(new Gtk::HBox(false, 6));
 
           Gtk::HBox *bbox = manage(new Gtk::HBox(true, 6));
-          Gtk::Button *postpone_button = create_postpone_button();
-          bbox->pack_end(*postpone_button, Gtk::PACK_EXPAND_WIDGET, 0);
-          Gtk::Button *skip_button = create_skip_button();
-          bbox->pack_end(*skip_button, Gtk::PACK_EXPAND_WIDGET, 0);
+
+          if ((break_flags & BREAK_FLAGS_POSTPONABLE) != 0)
+            {
+              Gtk::Button *postpone_button = create_postpone_button();
+              bbox->pack_end(*postpone_button, Gtk::PACK_EXPAND_WIDGET, 0);
+            }
+
+          if ((break_flags & BREAK_FLAGS_SKIPPABLE) != 0)
+            {
+              Gtk::Button *skip_button = create_skip_button();
+              bbox->pack_end(*skip_button, Gtk::PACK_EXPAND_WIDGET, 0);
+            }
 
           Gtk::Alignment *bboxa =
             manage(new Gtk::Alignment(1.0, 0.0, 0.0, 0.0));
           bboxa->add(*bbox);
 
-          if (has_rb)
+          if (restbreak->is_enabled())
             {
               button_box->pack_start
                 (*manage(create_restbreaknow_button(false)),
@@ -116,6 +127,8 @@ MicroBreakWindow::create_gui()
       box->pack_start(*button_box, Gtk::PACK_EXPAND_WIDGET, 0);
     }
 
+  fixed_size = false;
+  
   return box;
 }
 
@@ -129,14 +142,6 @@ MicroBreakWindow::~MicroBreakWindow()
 
 
 
-//! Updates the main window.
-void
-MicroBreakWindow::heartbeat()
-{
-  refresh();
-}
-
-
 Gtk::Button *
 MicroBreakWindow::create_restbreaknow_button(bool label)
 {
@@ -145,7 +150,7 @@ MicroBreakWindow::create_restbreaknow_button(bool label)
                                             "timer-rest-break.png",
                                             label));
   ret->signal_clicked()
-    .connect(MEMBER_SLOT(*this,
+    .connect(sigc::mem_fun(*this,
                          &MicroBreakWindow::on_restbreaknow_button_clicked));
   GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
   return ret;
@@ -163,7 +168,7 @@ MicroBreakWindow::on_restbreaknow_button_clicked()
 
 
 void
-MicroBreakWindow::refresh_time_bar()
+MicroBreakWindow::update_time_bar()
 {
   TRACE_ENTER("MicroBreakWindow::refresh_time_bar");
 
@@ -176,18 +181,17 @@ MicroBreakWindow::refresh_time_bar()
   time_bar->set_text(s);
 
   ICore *core = CoreFactory::get_core();
-  IActivityMonitor *monitor = core->get_activity_monitor();
-  ActivityState state = monitor->get_current_state();
+  bool user_active = core->is_user_active();
   if (frame != NULL)
     {
-      if (state == ACTIVITY_ACTIVE && !is_flashing)
+      if (user_active && !is_flashing)
         {
           frame->set_frame_color(Gdk::Color("orange"));
           frame->set_frame_visible(true);
           frame->set_frame_flashing(500);
           is_flashing = true;
         }
-      else if (state == ACTIVITY_IDLE && is_flashing)
+      else if (!user_active && is_flashing)
         {
           frame->set_frame_flashing(0);
           frame->set_frame_visible(false);
@@ -201,26 +205,26 @@ MicroBreakWindow::refresh_time_bar()
 
 
 void
-MicroBreakWindow::refresh_label()
+MicroBreakWindow::update_label()
 {
   TRACE_ENTER("MicroBreakWindow::refresh_label");
 
   ICore *core = CoreFactory::get_core();
 
-  ITimer *restbreak_timer =  core->get_timer(BREAK_ID_REST_BREAK);
-  ITimer *daily_timer =  core->get_timer(BREAK_ID_DAILY_LIMIT);
+  IBreak *restbreak_timer =  core->get_break(BREAK_ID_REST_BREAK);
+  IBreak *daily_timer =  core->get_break(BREAK_ID_DAILY_LIMIT);
 
   BreakId show_next = BREAK_ID_NONE;
 
   time_t rb = restbreak_timer->get_limit() - restbreak_timer->get_elapsed_time();
   time_t dl = daily_timer->get_limit() - daily_timer->get_elapsed_time();
 
-  if (restbreak_timer->get_state() != ITimer::STATE_INVALID)
+  if (restbreak_timer->is_enabled())
     {
       show_next = BREAK_ID_REST_BREAK;
     }
 
-  if (daily_timer->get_state() != ITimer::STATE_INVALID)
+  if (daily_timer->is_enabled())
     {
       if (show_next == BREAK_ID_NONE || dl < rb)
         {
@@ -274,12 +278,20 @@ MicroBreakWindow::refresh_label()
 
 //! Refresh window.
 void
-MicroBreakWindow::refresh()
+MicroBreakWindow::update_break_window()
 {
-  BreakWindow::refresh();
+  update_time_bar();
+  update_label();
 
-  refresh_time_bar();
-  refresh_label();
+  if (!fixed_size)
+    {
+      // Make sure the label doesn't resize anymore.
+      // There has to be a better way to do this...
+      Gtk::Requisition size = label->size_request();
+      label->set_size_request(size.width, size.height);
+      fixed_size = true;
+      center();
+    }
 }
 
 
