@@ -1,20 +1,23 @@
 // PreludeWindow.cc
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: PreludeWindow.cc 1324 2007-09-10 19:39:34Z rcaelers $";
+static const char rcsid[] = "$Id: PreludeWindow.cc 1540 2008-06-18 20:12:19Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,6 +30,9 @@ static const char rcsid[] = "$Id: PreludeWindow.cc 1324 2007-09-10 19:39:34Z rca
 #include <gtkmm/box.h>
 
 #include "debug.hh"
+#if defined(PLATFORM_OS_WIN32)
+#include "w32debug.hh"
+#endif
 #include "nls.h"
 
 #include "Text.hh"
@@ -43,7 +49,7 @@ static const char rcsid[] = "$Id: PreludeWindow.cc 1324 2007-09-10 19:39:34Z rca
 #include "Hig.hh"
 #include "GtkUtil.hh"
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 #include <gdk/gdkwin32.h>
 #endif
 
@@ -65,9 +71,6 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
     prelude_response(NULL)
 {
   Gtk::Window::set_border_width(0);
-#ifdef HAVE_X
-  GtkUtil::set_wmclass(*this, "Prelude");
-#endif
 
   init_avoid_pointer();
   realize();
@@ -97,7 +100,7 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
   frame->set_frame_width(6);
   frame->set_border_width(6);
   frame->add(*hbox);
-  frame->signal_flash().connect(MEMBER_SLOT(*this, &PreludeWindow::on_frame_flash));
+  frame->signal_flash().connect(sigc::mem_fun(*this, &PreludeWindow::on_frame_flash));
   flash_visible = true;
   color_warn = Gdk::Color("orange");
   color_alert = Gdk::Color("red");
@@ -126,20 +129,64 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
   show_all_children();
   stick();
 
+  // trace window handles:
+  // FIXME: debug, remove later
+#ifdef PLATFORM_OS_WIN32
+  HWND _hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
+  HWND _scope = (HWND) GDK_WINDOW_HWND( GTK_WIDGET( this->gobj() )->window );
+  HWND _hRoot = GetAncestor( _hwnd, GA_ROOT );
+  HWND _hParent = GetAncestor( _hwnd, GA_PARENT );
+  //HWND _hParent2 = (HWND)GetWindowLong( _hwnd, GWL_HWNDPARENT );
+  HWND _hDesktop = GetDesktopWindow();
+  
+  APPEND_TIME( "PreludeWindow created", hex << _hwnd );
+  
+  if( _hwnd != _scope )
+    {
+      APPEND( "!!!!!!!!!!!!!!!", "Scope issue: " << hex << _scope );
+      APPEND_ENDL();
+    }
+  
+  if( _hwnd != _hRoot )
+    {
+      APPEND( "GetDesktopWindow()", hex << _hDesktop );
+      APPEND( "!!!!!!!!!!!!!!!", "PreludeWindow GA_ROOT: " << hex << _hRoot );
+      APPEND_ENDL();
+    }
+  
+  if( _hParent != _hDesktop )
+    {
+      APPEND( "GetDesktopWindow()", hex << _hDesktop );
+      APPEND( "!!!!!!!!!!!!!!!", "PreludeWindow GA_PARENT: " << hex << _hParent );
+      
+      HWND _hTemp;
+      while( IsWindow( _hParent ) && _hParent != _hDesktop )
+        {
+          _hTemp = _hParent;
+          _hParent = GetAncestor( _hTemp, GA_PARENT );
+          HWND _hParent2 = (HWND)GetWindowLong( _hTemp, GWL_HWNDPARENT );
+          if( _hParent == _hTemp )
+            break;
+          APPEND( "!!!!!!!!!!!!!!!", hex << _hTemp << " GA_PARENT: " << hex << _hParent );
+          APPEND( "!!!!!!!!!!!!!!!", hex << _hTemp << " GWL_HWNDPARENT: " << hex << _hParent2 );
+        }
+      APPEND_ENDL();
+    }
+  
+#endif
+
   this->head = head;
-#ifdef HAVE_GTK_MULTIHEAD
   if (head.valid)
     {
       Gtk::Window::set_screen(head.screen);
     }
-#endif
 }
 
 
 //! Destructor.
 PreludeWindow::~PreludeWindow()
 {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   if (avoid_signal.connected())
     {
       avoid_signal.disconnect();
@@ -160,22 +207,20 @@ PreludeWindow::start()
   realize_if_needed();
 
   // Set some window hints.
-  WindowHints::set_skip_winlist(Gtk::Widget::gobj(), true);
+  set_skip_pager_hint(true);
+  set_skip_taskbar_hint(true);
 
-#ifdef WIN32
-  SetWindowPos( (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window ),
-      HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
-#else
-  WindowHints::set_always_on_top(Gtk::Widget::gobj(), true);
-#endif
+  WindowHints::set_always_on_top(this, true);
 
   refresh();
+
   GtkUtil::center_window(*this, head);
   show_all();
 
-
+  WindowHints::set_always_on_top(this, true);
+  
   time_bar->set_bar_color(TimeBar::COLOR_ID_OVERDUE);
-
+  
   TRACE_EXIT();
 }
 
@@ -239,7 +284,8 @@ PreludeWindow::refresh()
     }
   time_bar->set_text(s);
   time_bar->update();
-#ifdef WIN32
+  
+#if defined(PLATFORM_OS_WIN32)
 // Vista GTK phantom toplevel parent kludge:
   HWND hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
   if( hwnd )
@@ -337,15 +383,16 @@ PreludeWindow::on_frame_flash(bool frame_visible)
   TRACE_EXIT();
 }
 
+
 // Sets whether the window should run away for the mouse pointer.
 void
 PreludeWindow::init_avoid_pointer()
 {
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   if (! avoid_signal.connected())
     {
       avoid_signal = Glib::signal_timeout()
-        .connect(MEMBER_SLOT(*this, &PreludeWindow::on_avoid_pointer_timer),
+        .connect(sigc::mem_fun(*this, &PreludeWindow::on_avoid_pointer_timer),
                  150);
     }
 #else
@@ -360,7 +407,7 @@ PreludeWindow::init_avoid_pointer()
   did_avoid = false;
 }
 
-#ifdef HAVE_X
+#ifndef PLATFORM_OS_WIN32
 
 //! GDK EventNotifyEvent notification.
 bool
@@ -369,6 +416,7 @@ PreludeWindow::on_enter_notify_event(GdkEventCrossing *event)
   avoid_pointer((int)event->x, (int)event->y);
   return Gtk::Window::on_enter_notify_event(event);
 }
+
 #endif
 
 
@@ -384,8 +432,8 @@ PreludeWindow::avoid_pointer(int px, int py)
 
   TRACE_MSG("geom" << winx << " " << winy << " " << width << " " << height << " ");
 
-#ifdef WIN32
-  // This is only necessary for WIN32, since HAVE_X uses GdkEventCrossing.
+#ifdef PLATFORM_OS_WIN32
+  // This is only necessary for PLATFORM_OS_WIN32, since PLATFORM_OS_UNIX uses GdkEventCrossing.
   // Set gravitiy, otherwise, get_position() returns weird winy.
   set_gravity(Gdk::GRAVITY_STATIC);
   get_position(winx, winy);
@@ -395,6 +443,7 @@ PreludeWindow::avoid_pointer(int px, int py)
   px += winx;
   py += winy;
 #endif
+
   TRACE_MSG("geom2" << winx << " " << winy << " " << width << " " << height << " ");
 
   int screen_height = head.get_height();
@@ -425,7 +474,7 @@ PreludeWindow::avoid_pointer(int px, int py)
   did_avoid = true;
 }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 bool
 PreludeWindow::on_avoid_pointer_timer()
 {
