@@ -1,21 +1,24 @@
 // RestBreakWindow.cc --- window for the microbreak
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: RestBreakWindow.cc 1316 2007-09-08 17:40:55Z rcaelers $";
+static const char rcsid[] = "$Id: RestBreakWindow.cc 1485 2008-03-13 20:52:02Z rcaelers $";
 
 const int TIMEOUT = 1000;
 
@@ -41,7 +44,6 @@ const int TIMEOUT = 1000;
 #include "Util.hh"
 #include "WindowHints.hh"
 
-#include "ITimer.hh"
 #include "IBreak.hh"
 #include "IBreakResponse.hh"
 #include "GtkUtil.hh"
@@ -50,7 +52,6 @@ const int TIMEOUT = 1000;
 #include "ICore.hh"
 #include "IConfigurator.hh"
 #include "CoreFactory.hh"
-#include "IActivityMonitor.hh"
 
 #ifdef HAVE_EXERCISES
 #include "Exercise.hh"
@@ -65,9 +66,9 @@ const int MARGINY = 8;
 /*!
  *  \param control The controller.
  */
-RestBreakWindow::RestBreakWindow(HeadInfo &head, bool ignorable,
-                                 GUI::BlockMode mode) :
-  BreakWindow(BREAK_ID_REST_BREAK, head, ignorable, mode),
+RestBreakWindow::RestBreakWindow(HeadInfo &head, BreakFlags break_flags,
+                                 GUIConfig::BlockMode mode) :
+  BreakWindow(BREAK_ID_REST_BREAK, head, break_flags, mode),
   timebar(NULL),
   progress_value(0),
   progress_max_value(0),
@@ -136,7 +137,8 @@ RestBreakWindow::start()
 #else
   set_ignore_activity(false);
 #endif
-  refresh();
+  
+  update_break_window();
 
   BreakWindow::start();
 
@@ -146,10 +148,8 @@ RestBreakWindow::start()
 
 //! Period timer callback.
 void
-RestBreakWindow::refresh()
+RestBreakWindow::update_break_window()
 {
-  BreakWindow::refresh();
-
   draw_time_bar();
 }
 
@@ -175,18 +175,17 @@ RestBreakWindow::draw_time_bar()
   timebar->set_text(s);
 
   ICore *core = CoreFactory::get_core();
-  IActivityMonitor *monitor = core->get_activity_monitor();
-  ActivityState state = monitor->get_current_state();
+  bool user_active = core->is_user_active();
   if (frame != NULL)
     {
-      if (state == ACTIVITY_ACTIVE && !is_flashing)
+      if (user_active && !is_flashing)
         {
           frame->set_frame_color(Gdk::Color("orange"));
           frame->set_frame_visible(true);
           frame->set_frame_flashing(500);
           is_flashing = true;
         }
-      else if (state == ACTIVITY_IDLE && is_flashing)
+      else if (!user_active && is_flashing)
         {
           frame->set_frame_flashing(0);
           frame->set_frame_visible(false);
@@ -240,10 +239,7 @@ RestBreakWindow::get_exercise_count()
 
   if (Exercise::has_exercises())
     {
-      ICore *core = CoreFactory::get_core();
-      assert(core != NULL);
-
-      ret = core->get_break(BREAK_ID_REST_BREAK)->get_break_exercises();
+      ret = GUIConfig::get_number_of_exercises(BREAK_ID_REST_BREAK);
     }
   return ret;
 }
@@ -263,22 +259,41 @@ RestBreakWindow::install_exercises_panel()
       pluggable_panel->pack_start(*exercises_panel, false, false, 0);
       exercises_panel->set_exercise_count(get_exercise_count());
       exercises_panel->signal_stop().connect
-        (MEMBER_SLOT(*this, &RestBreakWindow::install_info_panel));
+        (sigc::mem_fun(*this, &RestBreakWindow::install_info_panel));
       pluggable_panel->show_all();
       pluggable_panel->queue_resize();
-      center();
     }
+  center();
 }
 
 void
 RestBreakWindow::install_info_panel()
 {
+  Gtk::Requisition old_size = size_request();
+  
   set_ignore_activity(false);
   clear_pluggable_panel();
   pluggable_panel->pack_start(*(create_info_panel()), false, false, 0);
   pluggable_panel->show_all();
   pluggable_panel->queue_resize();
-  center();
+
+  GUIConfig::BlockMode block_mode = GUIConfig::get_block_mode();
+  if (block_mode == GUIConfig::BLOCK_MODE_NONE &&
+      head.count == 0)
+    {
+      Gtk::Requisition new_size = size_request();
+
+      int width_delta = (new_size.width - old_size.width) / 2;
+      int height_delta = (new_size.height -  old_size.height) / 2;
+
+      int x, y;
+      get_position(x, y);
+      move(x - width_delta, y - height_delta);
+    }
+  else
+    {
+      center();
+    }
 }
 
 #endif
@@ -289,9 +304,10 @@ RestBreakWindow::set_ignore_activity(bool i)
   ICore *core = CoreFactory::get_core();
   assert(core != NULL);
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   bool force_focus = false;
-  CoreFactory::get_configurator()->get_value_default("advanced/force_focus", &force_focus, false);
+
+  CoreFactory::get_configurator()->get_value_with_default("advanced/force_focus", force_focus, false);
   if (force_focus)
     {
       i = true;
@@ -299,8 +315,6 @@ RestBreakWindow::set_ignore_activity(bool i)
 #endif
 
   core->set_insist_policy(i ?
-                        ICore::INSIST_POLICY_IGNORE :
-                        (block_mode != GUI::BLOCK_MODE_NONE
-                         ? ICore::INSIST_POLICY_HALT
-                         : ICore::INSIST_POLICY_RESET));
+                          ICore::INSIST_POLICY_IGNORE :
+                          ICore::INSIST_POLICY_HALT);
 }
