@@ -1,20 +1,23 @@
 // StatusIcon.cc --- Status icon
 //
-// Copyright (C) 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: StatusIcon.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: StatusIcon.cc 1463 2008-03-08 17:35:31Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -49,12 +52,11 @@ StatusIcon::StatusIcon(MainWindow& mw)
       mode_icons[i] = Gdk::Pixbuf::create_from_file(file);
     }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   wm_taskbarcreated = RegisterWindowMessage("TaskbarCreated");
 #endif
 
   insert_icon();
-  Menus::get_instance()->create_menu(Menus::MENU_APPLET);
 }
 
 StatusIcon::~StatusIcon()
@@ -69,6 +71,10 @@ StatusIcon::insert_icon()
   OperationMode mode = core->get_operation_mode();
   status_icon = Gtk::StatusIcon::create(mode_icons[mode]);
 
+#ifdef HAVE_STATUSICON_SIGNAL 
+  status_icon->signal_activate().connect(sigc::mem_fun(*this, &StatusIcon::on_activate));
+  status_icon->signal_popup_menu().connect(sigc::mem_fun(*this, &StatusIcon::on_popup_menu));
+#else
   // Hook up signals, missing from gtkmm
   GtkStatusIcon *gobj = status_icon->gobj();
 
@@ -76,7 +82,7 @@ StatusIcon::insert_icon()
                    reinterpret_cast<GCallback>(activate_callback), this);
   g_signal_connect(gobj, "popup-menu",
                    reinterpret_cast<GCallback>(popup_menu_callback), this);
-
+#endif
 }
 
 void StatusIcon::set_operation_mode(OperationMode m)
@@ -93,10 +99,11 @@ void StatusIcon::on_popup_menu(guint button, guint activate_time)
 {
   (void) button;
 
-  // Note the 1 is a hack. It used to be button. See bugzilla 598
-  Menus::get_instance()->popup(Menus::MENU_APPLET, 1 , activate_time);
+  // Note the 1 is a hack. It used to be 'button'. See bugzilla 598
+  Menus::get_instance()->popup(Menus::MENU_APPLET, 1, activate_time);
 }
 
+#ifndef HAVE_STATUSICON_SIGNAL 
 void StatusIcon::activate_callback(GtkStatusIcon *,
                                    gpointer callback_data)
 {
@@ -110,13 +117,14 @@ void StatusIcon::popup_menu_callback(GtkStatusIcon *,
 {
   static_cast<StatusIcon*>(callback_data)->on_popup_menu(button, activate_time);
 }
+#endif
 
 void StatusIcon::set_timers_tooltip(std::string& tip)
 {
   status_icon->set_tooltip(tip);
 }
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
 GdkFilterReturn
 StatusIcon::win32_filter_func (void     *xevent,
                                GdkEvent *event)
