@@ -1,21 +1,24 @@
 // TimeBar.cc --- Time Bar
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
 
-static const char rcsid[] = "$Id: TimeBar.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: TimeBar.cc 1430 2008-01-28 22:44:29Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,6 +41,8 @@ const int MARGINY = 3;
 const int MIN_HORIZONTAL_BAR_WIDTH = 12;
 const int MIN_HORIZONTAL_BAR_HEIGHT = 20; // stolen from gtk's progress bar
 
+using namespace std;
+
 Gdk::Color TimeBar::bar_colors[TimeBar::COLOR_ID_SIZEOF] =
   {
     Gdk::Color("lightblue"),
@@ -489,6 +494,6 @@ TimeBar::draw_bar(Glib::RefPtr<Gdk::Window> &window,
     }
   else
     {
-      window->draw_rectangle(gc, filled, y, winw - x, height, width);
+      window->draw_rectangle(gc, filled, y, winw - x - width, height, width);
     }
 }
