@@ -1,19 +1,22 @@
 // TimeEntry.cc --- Entry widget for time
 //
-// Copyright (C) 2002, 2003, 2004, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimeEntry.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimeEntry.cc 1351 2007-10-14 20:56:54Z rcaelers $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -42,8 +45,8 @@ TimeEntry::TimeEntry(bool millis)
 
   secs = manage(new Gtk::SpinButton(secs_adjustment));
   secs->set_numeric(true);
-  secs->signal_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_changed));
-  secs->signal_value_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_value_changed));
+  secs->signal_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_changed));
+  secs->signal_value_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_value_changed));
 
   if (millis)
     {
@@ -64,15 +67,15 @@ TimeEntry::TimeEntry(bool millis)
       hrs->set_numeric(true);
       hrs->set_wrap(true);
       hrs->set_width_chars(2);
-      hrs->signal_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_changed));
-      hrs->signal_value_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_value_changed));
+      hrs->signal_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_changed));
+      hrs->signal_value_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_value_changed));
 
       mins = manage(new Gtk::SpinButton(mins_adjustment));
       mins->set_numeric(true);
       mins->set_wrap(true);
       mins->set_width_chars(2);
-      mins->signal_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_changed));
-      mins->signal_value_changed().connect(MEMBER_SLOT(*this, &TimeEntry::on_value_changed));
+      mins->signal_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_changed));
+      mins->signal_value_changed().connect(sigc::mem_fun(*this, &TimeEntry::on_value_changed));
   
       Gtk::Label *semi1 = manage(new Gtk::Label(":"));
       Gtk::Label *semi2 = manage(new Gtk::Label(":"));
