@@ -1,19 +1,22 @@
 // TimerBoxPreferencePage.cc --- Preferences widgets for a timer
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimerBoxPreferencePage.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimerBoxPreferencePage.cc 1434 2008-02-01 22:20:17Z rcaelers $
 //
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +38,7 @@
 #include "IConfigurator.hh"
 #include "CoreFactory.hh"
 #include "ICore.hh"
+#include "CoreConfig.hh"
 #include "GtkUtil.hh"
 #include "Hig.hh"
 #include "MainWindow.hh"
@@ -62,10 +66,7 @@ TimerBoxPreferencePage::TimerBoxPreferencePage(string n)
       ICore *core = CoreFactory::get_core();
       assert(core != NULL);
 
-      IBreak *b = core->get_break(BreakId(i));
-      config->add_listener("gui/breaks/"
-                           + b->get_name()
-                           + "/enabled", this);
+      config->add_listener(CoreConfig::CFG_KEY_BREAK_ENABLED % BreakId(i), this);
     }
 
   TRACE_EXIT();
@@ -108,7 +109,7 @@ TimerBoxPreferencePage::create_page()
   cycle_entry->set_numeric(true);
   cycle_entry->set_width_chars(3);
   cycle_entry->signal_changed().connect
-    (MEMBER_SLOT(*this, &TimerBoxPreferencePage::on_cycle_time_changed));
+    (sigc::mem_fun(*this, &TimerBoxPreferencePage::on_cycle_time_changed));
 
   // Timer display
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
@@ -138,7 +139,7 @@ TimerBoxPreferencePage::create_page()
       ontop_cb = manage
         (new Gtk::CheckButton
          (_("The status window stays always on top of other windows")));
-      ontop_cb->signal_toggled().connect(MEMBER_SLOT(*this, &TimerBoxPreferencePage::on_always_on_top_toggled));
+      ontop_cb->signal_toggled().connect(sigc::mem_fun(*this, &TimerBoxPreferencePage::on_always_on_top_toggled));
       ontop_cb->set_active(MainWindow::get_always_on_top());
     }
   else if (name == "applet")
@@ -230,14 +231,14 @@ void
 TimerBoxPreferencePage::init_page_callbacks()
 {
   place_button->signal_changed().connect
-    (MEMBER_SLOT(*this, &TimerBoxPreferencePage::on_place_changed));
+    (sigc::mem_fun(*this, &TimerBoxPreferencePage::on_place_changed));
 
-  enabled_cb->signal_toggled().connect(MEMBER_SLOT(*this, &TimerBoxPreferencePage::on_enabled_toggled));
+  enabled_cb->signal_toggled().connect(sigc::mem_fun(*this, &TimerBoxPreferencePage::on_enabled_toggled));
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
       timer_display_button[i]->signal_changed().connect
-        (bind(MEMBER_SLOT(*this, &TimerBoxPreferencePage::on_display_changed), i));
+        (bind(sigc::mem_fun(*this, &TimerBoxPreferencePage::on_display_changed), i));
     }
 }
 
@@ -344,7 +345,7 @@ TimerBoxPreferencePage::enable_buttons(void)
 
           IBreak *b = core->get_break(BreakId(i));
 
-          bool timer_on = b->get_break_enabled();
+          bool timer_on = b->is_enabled();
           timer_display_button[i]->set_sensitive(on && timer_on);
         }
       cycle_entry->set_sensitive(on && count != 3);
@@ -358,7 +359,7 @@ TimerBoxPreferencePage::enable_buttons(void)
           assert(core != NULL);
 
           IBreak *b = core->get_break(BreakId(i));
-          timer_display_button[i]->set_sensitive(b->get_break_enabled());
+          timer_display_button[i]->set_sensitive(b->is_enabled());
         }
       if (count == 3)
         {
@@ -393,7 +394,7 @@ TimerBoxPreferencePage::on_always_on_top_toggled()
 
 
 void
-TimerBoxPreferencePage::config_changed_notify(string key)
+TimerBoxPreferencePage::config_changed_notify(const string &key)
 {
   TRACE_ENTER("TimerBoxPreferencePage::config_changed_notify");
   (void)key;
