@@ -1,19 +1,22 @@
 // TimerPreferencesPanel.cc --- Preferences widgets for a timer
 //
-// Copyright (C) 2002, 2003, 2004, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimerPreferencesPanel.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimerPreferencesPanel.cc 1490 2008-03-17 18:57:38Z rcaelers $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -31,14 +34,19 @@
 
 #include "CoreFactory.hh"
 #include "ICore.hh"
+#include "IConfigurator.hh"
 #include "IBreak.hh"
-#include "ITimer.hh"
 
 #include "TimeEntry.hh"
 #include "TimerPreferencesPanel.hh"
 #include "GtkUtil.hh"
 #include "Hig.hh"
 
+#include "GUIConfig.hh"
+#include "CoreConfig.hh"
+#include "DataConnector.hh"
+
+using namespace std;
 
 TimerPreferencesPanel::TimerPreferencesPanel
 (BreakId t,
@@ -50,19 +58,17 @@ TimerPreferencesPanel::TimerPreferencesPanel
   ,exercises_adjustment(0, 0, 10)
 #endif
 {
+  connector = new DataConnector();
   break_id = t;
 
-  ICore *core = CoreFactory::get_core();
-  break_data = core->get_break(BreakId(break_id));
-
   Gtk::HBox *box = manage(new Gtk::HBox(false, 6));
 
   // Enabled/Disabled checkbox
   Gtk::Label *enabled_lab = manage(GtkUtil::create_label(_("Enable timer"), true));
   enabled_cb = manage(new Gtk::CheckButton());
   enabled_cb->add(*enabled_lab);
-  enabled_cb->set_active(break_data->get_break_enabled());
-  enabled_cb->signal_toggled().connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_enabled_toggled));
+  enabled_cb->signal_toggled().connect(sigc::mem_fun(*this, &TimerPreferencesPanel::on_enabled_toggled));
+
 
   HigCategoriesPanel *categories = manage(new HigCategoriesPanel());;
 
@@ -75,7 +81,6 @@ TimerPreferencesPanel::TimerPreferencesPanel
   categories->add(*opts_frame);
 
   enable_buttons();
-  set_prelude_sensitivity();
 
   // Overall box
   box->pack_start(*categories, false, false, 0);
@@ -84,17 +89,18 @@ TimerPreferencesPanel::TimerPreferencesPanel
   pack_start(*enabled_cb, false, false, 0);
   pack_start(*box, false, false, 0);
 
+  connector->connect(CoreConfig::CFG_KEY_BREAK_ENABLED % break_id, dc::wrap(enabled_cb));
+  
   set_border_width(12);
 }
 
 
 TimerPreferencesPanel::~TimerPreferencesPanel()
 {
-  // FIXME: disconnect signals?
+  delete connector;
 }
 
 
-
 Gtk::Widget *
 TimerPreferencesPanel::create_prelude_panel()
 {
@@ -102,34 +108,28 @@ TimerPreferencesPanel::create_prelude_panel()
   HigCategoryPanel *hig = manage(new HigCategoryPanel(_("Break prompting")));
 
   prelude_cb = manage(new Gtk::CheckButton(_("Prompt before breaking")));
-  int max_preludes = break_data->get_break_max_preludes();
-  prelude_cb->set_active(max_preludes != 0);
-
-  has_max_prelude_cb = manage(new Gtk::CheckButton
-                              (_("Maximum number of prompts:")));
-  has_max_prelude_cb->set_active(max_preludes > 0);
-
-  max_prelude_adjustment.set_value(max_preludes > 0 ? max_preludes : 1);
-  max_prelude_spin = manage(new Gtk::SpinButton(max_prelude_adjustment));
-
-  set_prelude_sensitivity();
-
-  prelude_cb->signal_toggled()
-    .connect(MEMBER_SLOT(*this,
-                        &TimerPreferencesPanel::on_preludes_active_toggled));
-  has_max_prelude_cb->signal_toggled()
-    .connect(MEMBER_SLOT(*this,
-                        &TimerPreferencesPanel::on_preludes_maximum_toggled));
-  max_prelude_adjustment.signal_value_changed()
-    .connect(MEMBER_SLOT(*this,
-                        &TimerPreferencesPanel::on_preludes_maximum_changed));
-
   hig->add(*prelude_cb);
+
   Gtk::HBox *max_box = manage(new Gtk::HBox());
+  has_max_prelude_cb = manage(new Gtk::CheckButton(_("Maximum number of prompts:")));
+  max_prelude_spin = manage(new Gtk::SpinButton(max_prelude_adjustment));
   max_box->pack_start(*has_max_prelude_cb, false, false, 0);
   max_box->pack_start(*max_prelude_spin, false, false, 0);
   hig->add(*max_box);
 
+  connector->connect(CoreConfig::CFG_KEY_BREAK_MAX_PRELUDES % break_id,
+                     dc::wrap(prelude_cb),
+                     sigc::mem_fun(*this, &TimerPreferencesPanel::on_preludes_changed));
+  
+  connector->connect(CoreConfig::CFG_KEY_BREAK_MAX_PRELUDES % break_id,
+                     dc::wrap(has_max_prelude_cb),
+                     sigc::mem_fun(*this, &TimerPreferencesPanel::on_preludes_changed),
+                     dc::NO_CONFIG);
+
+  connector->connect(CoreConfig::CFG_KEY_BREAK_MAX_PRELUDES % break_id,
+                     dc::wrap(max_prelude_spin),
+                     sigc::mem_fun(*this, &TimerPreferencesPanel::on_preludes_changed),
+                     dc::NO_CONFIG);
   return hig;
 }
 
@@ -139,21 +139,13 @@ TimerPreferencesPanel::create_options_panel()
   HigCategoryPanel *hig = manage(new HigCategoryPanel(_("Options")));
 
   // Ignorable
-  bool ignorable = break_data->get_break_ignorable();
   ignorable_cb = manage(new Gtk::CheckButton
                         (_("Show 'Postpone' and 'Skip' button")));
-  ignorable_cb->set_active(ignorable);
-  ignorable_cb->signal_toggled()
-    .connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_ignorable_toggled));
   hig->add(*ignorable_cb);
 
   // Sensitive for activity
-  bool sensitive = break_data->get_timer_activity_sensitive();
   activity_sensitive_cb = manage(new Gtk::CheckButton
                         (_("Suspend timer when inactive")));
-  activity_sensitive_cb->set_active(sensitive);
-  activity_sensitive_cb->signal_toggled()
-    .connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_activity_sensitive_toggled));
   hig->add(*activity_sensitive_cb);
 
   // Break specific options
@@ -167,11 +159,6 @@ TimerPreferencesPanel::create_options_panel()
     {
       monitor_cb
         = manage(new Gtk::CheckButton(_("Regard micro-breaks as activity")));
-      string monitor_name;
-      monitor_name = break_data->get_timer_monitor();
-      monitor_cb->set_active(monitor_name != "");
-      monitor_cb->signal_toggled()
-        .connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_monitor_toggled));
       hig->add(*monitor_cb);
     }
 #endif
@@ -181,14 +168,21 @@ TimerPreferencesPanel::create_options_panel()
     {
       exercises_spin = manage(new Gtk::SpinButton(exercises_adjustment));
       hig->add(_("Number of exercises:"), *exercises_spin);
-      exercises_adjustment.set_value(break_data->get_break_exercises());
-      exercises_adjustment.signal_value_changed()
-        .connect(MEMBER_SLOT(*this,
-                            &TimerPreferencesPanel::on_exercises_changed));
     }
 #endif
 
+  connector->connect(CoreConfig::CFG_KEY_TIMER_ACTIVITY_SENSITIVE % break_id,
+                     dc::wrap(activity_sensitive_cb));
 
+  connector->connect(GUIConfig::CFG_KEY_BREAK_IGNORABLE % break_id,
+                     dc::wrap(ignorable_cb));
+  
+  connector->connect(GUIConfig::CFG_KEY_BREAK_EXERCISES % break_id,
+                     dc::wrap(exercises_spin));
+  
+  connector->connect(CoreConfig::CFG_KEY_MONITOR % break_id,
+                     dc::wrap(monitor_cb),
+                     sigc::mem_fun(*this, &TimerPreferencesPanel::on_monitor_changed));
   return hig;
 }
 
@@ -201,28 +195,19 @@ TimerPreferencesPanel::create_timers_panel
 
   // Limit time
   limit_tim = manage(new TimeEntry());
-  limit_tim->set_value(break_data->get_timer_limit());
-  limit_tim->signal_value_changed()
-    .connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_limit_changed));
   Gtk::Label *limit_lab = hig->add(break_id == BREAK_ID_DAILY_LIMIT
            ? _("Time before end:")
            : _("Time between breaks:"), *limit_tim);
   hsize_group->add_widget(*limit_lab);
 
+
   // Auto-reset time
   if (break_id != BREAK_ID_DAILY_LIMIT)
     {
-      const char *auto_reset_txt;
-      time_t auto_reset_value;
-
-      auto_reset_txt = _("Break duration:");
-      auto_reset_value = break_data->get_timer_auto_reset();
+      const char *auto_reset_txt = _("Break duration:");
 
       auto_reset_tim = manage(new TimeEntry());
-      auto_reset_tim->set_value (auto_reset_value);
-      auto_reset_tim->signal_value_changed()
-        .connect(MEMBER_SLOT(*this,
-                            &TimerPreferencesPanel::on_auto_reset_changed));
+
       Gtk::Label *auto_reset_lab = manage(new Gtk::Label(auto_reset_txt));
       hsize_group->add_widget(*auto_reset_lab);
       hig->add(*auto_reset_lab, *auto_reset_tim);
@@ -234,18 +219,19 @@ TimerPreferencesPanel::create_timers_panel
 
   // Snooze time
   snooze_tim = manage(new TimeEntry());
-  snooze_tim->set_value (break_data->get_timer_snooze());
-  snooze_tim->signal_value_changed()
-    .connect(MEMBER_SLOT(*this, &TimerPreferencesPanel::on_snooze_changed));
   Gtk::Label *snooze_lab = hig->add(_("Postpone time:"), *snooze_tim);
   hsize_group->add_widget(*snooze_lab);
 
   vsize_group->add_widget(*hig);
+
+  connector->connect(CoreConfig::CFG_KEY_TIMER_LIMIT % break_id, dc::wrap(limit_tim));
+  connector->connect(CoreConfig::CFG_KEY_TIMER_AUTO_RESET % break_id, dc::wrap(auto_reset_tim));
+  connector->connect(CoreConfig::CFG_KEY_TIMER_SNOOZE % break_id, dc::wrap(snooze_tim));
+
   return hig;
 }
 
 
-
 void
 TimerPreferencesPanel::set_prelude_sensitivity()
 {
@@ -256,128 +242,115 @@ TimerPreferencesPanel::set_prelude_sensitivity()
   max_prelude_spin->set_sensitive(has_preludes && has_max && on);
 }
 
-void
-TimerPreferencesPanel::on_preludes_active_toggled()
+bool
+TimerPreferencesPanel::on_preludes_changed(const std::string &key, bool write)
 {
-  int mp;
-  if (prelude_cb->get_active())
+  static bool inside = false;
+
+  if (inside)
+    return true;
+
+  inside = true;
+  
+  IConfigurator *config = CoreFactory::get_configurator();
+  if (write)
     {
-      if (has_max_prelude_cb->get_active())
+      int mp;
+      if (prelude_cb->get_active())
         {
-          mp = (int) max_prelude_adjustment.get_value();
+          if (has_max_prelude_cb->get_active())
+            {
+              mp = (int) max_prelude_adjustment.get_value();
+            }
+          else
+            {
+              mp = -1;
+            }
         }
       else
         {
-          mp = -1;
+          mp = 0;
+        }
+      config->set_value(key, mp);
+      set_prelude_sensitivity();
+    }
+  else
+    {
+      int value;
+      bool ok = config->get_value(key, value);
+      if (ok)
+        {
+          if (value == -1)
+            {
+              prelude_cb->set_active(true);
+              has_max_prelude_cb->set_active(false);
+            }
+          else if (value == 0)
+            {
+              prelude_cb->set_active(false);
+              has_max_prelude_cb->set_active(false);
+            }
+          else
+            {
+              prelude_cb->set_active(true);
+              has_max_prelude_cb->set_active(true);
+              max_prelude_adjustment.set_value(value);
+            }
+
+          set_prelude_sensitivity();
         }
     }
-  else
-    {
-      mp = 0;
-    }
-  break_data->set_break_max_preludes(mp);
-  set_prelude_sensitivity();
+
+  inside = false;
+  
+  return true;
 }
 
-void
-TimerPreferencesPanel::on_preludes_maximum_changed()
-{
-  int mp = (int) max_prelude_adjustment.get_value();
-  break_data->set_break_max_preludes(mp);
-}
-
-#ifdef HAVE_EXERCISES
-void
-TimerPreferencesPanel::on_exercises_changed()
-{
-  int ex = (int) exercises_adjustment.get_value();
-  break_data->set_break_exercises(ex);
-}
-#endif
-
-void
-TimerPreferencesPanel::on_preludes_maximum_toggled()
-{
-  int mp;
-  if (has_max_prelude_cb->get_active())
-    {
-      mp = (int)max_prelude_adjustment.get_value();
-    }
-  else
-    {
-      mp = -1;
-    }
-  break_data->set_break_max_preludes(mp);
-  set_prelude_sensitivity();
-}
-
-
-
-
-void
-TimerPreferencesPanel::on_auto_reset_changed()
-{
-  TRACE_ENTER("TimerPreferencesPanel::on_auto_reset_changed");
-  if (auto_reset_tim != NULL)
-    {
-      time_t val = auto_reset_tim->get_value();
-      break_data->set_timer_auto_reset(val);
-    }
-  TRACE_EXIT();
-}
-
-void
-TimerPreferencesPanel::on_snooze_changed()
-{
-  TRACE_ENTER("TimerPreferencesPanel::on_snooze_changed");
-  break_data->set_timer_snooze(snooze_tim->get_value());
-  TRACE_EXIT();
-}
-
-void
-TimerPreferencesPanel::on_limit_changed()
-{
-  TRACE_ENTER("TimerPreferencesPanel::on_limit_changed");
-  break_data->set_timer_limit(limit_tim->get_value());
-  TRACE_EXIT();
-}
 
 #ifdef HAVE_MICRO_BREAK_ACTIVITY
-void
-TimerPreferencesPanel::on_monitor_toggled()
+bool
+TimerPreferencesPanel::on_monitor_changed(const string &key, bool write)
 {
-  string val;
+  IConfigurator *config = CoreFactory::get_configurator();
 
-  if (monitor_cb->get_active())
+  if (write)
     {
-      ICore *core = CoreFactory::get_core();
-      IBreak *mp_break = core->get_break(BREAK_ID_MICRO_BREAK);
-      val = mp_break->get_name();
+      string val;
+
+      if (monitor_cb->get_active())
+        {
+          ICore *core = CoreFactory::get_core();
+          IBreak *mp_break = core->get_break(BREAK_ID_MICRO_BREAK);
+          val = mp_break->get_name();
+        }
+
+      config->set_value(key, val);
+    }
+  else
+    {
+      string monitor_name;
+      bool ok = config->get_value(key, monitor_name);
+      if (ok)
+        {
+          bool s = monitor_cb->is_sensitive();
+          monitor_cb->set_active(monitor_name != "");
+          monitor_cb->set_sensitive(s);
+        }
     }
 
-  break_data->set_timer_monitor(val);
+  return true;
 }
 #endif
 
-void
-TimerPreferencesPanel::on_ignorable_toggled()
-{
-  break_data->set_break_ignorable(ignorable_cb->get_active());
-}
-
-void
-TimerPreferencesPanel::on_activity_sensitive_toggled()
-{
-  break_data->set_timer_activity_sensitive(activity_sensitive_cb->get_active());
-}
 
 void
 TimerPreferencesPanel::on_enabled_toggled()
 {
-  break_data->set_break_enabled(enabled_cb->get_active());
   enable_buttons();
+  set_prelude_sensitivity();
 }
 
+
 //! Enable widgets
 void
 TimerPreferencesPanel::enable_buttons()
@@ -400,5 +373,5 @@ TimerPreferencesPanel::enable_buttons()
   if (auto_reset_tim != NULL)
     auto_reset_tim->set_sensitive(on);
   snooze_tim->set_sensitive(on);
-  max_prelude_spin->set_sensitive(on);
+  // max_prelude_spin->set_sensitive(on);
 }
