@@ -1,20 +1,23 @@
 // X11SystrayAppletWindow.cc --- Applet info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: X11SystrayAppletWindow.cc 1324 2007-09-10 19:39:34Z rcaelers $";
+static const char rcsid[] = "$Id: X11SystrayAppletWindow.cc 1537 2008-06-08 19:02:40Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -35,7 +38,7 @@ static const char rcsid[] = "$Id: X11SystrayAppletWindow.cc 1324 2007-09-10 19:3
 #include "Menus.hh"
 #include "System.hh"
 
-#include "eggtrayicon.h"
+#include "gtktrayicon.h"
 
 //! Constructor.
 /*!
@@ -49,6 +52,7 @@ X11SystrayAppletWindow::X11SystrayAppletWindow(AppletControl *control) :
   applet_orientation(ORIENTATION_UP),
   applet_size(0),
   applet_active(false),
+  embedded(false),
   control(control),
   tray_icon(NULL)
 {
@@ -80,9 +84,9 @@ void
 X11SystrayAppletWindow::notify_callback()
 {
   TRACE_ENTER("X11SystrayAppletWindow::notify_callback");
-  if (tray_icon != NULL)
+  if (tray_icon != NULL && embedded)
     {
-      GtkOrientation o = egg_tray_icon_get_orientation(tray_icon);
+      GtkOrientation o = wrgtk_tray_icon_get_orientation(tray_icon);
       Orientation orientation;
 
       if (o != GTK_ORIENTATION_VERTICAL)
@@ -116,7 +120,7 @@ X11SystrayAppletWindow::activate_applet()
       return APPLET_STATE_VISIBLE;
     }
 
-  tray_icon = egg_tray_icon_new("Workrave Tray Icon");
+  tray_icon = wrgtk_tray_icon_new("Workrave Tray Icon");
   AppletState ret =  APPLET_STATE_DISABLED;
 
   if (tray_icon != NULL)
@@ -127,51 +131,44 @@ X11SystrayAppletWindow::activate_applet()
 
       plug = Glib::wrap(GTK_PLUG(tray_icon));
 
-
       Gtk::EventBox *eventbox = new Gtk::EventBox;
       eventbox->set_visible_window(false);
       eventbox->set_events(eventbox->get_events() | Gdk::BUTTON_PRESS_MASK);
-      eventbox->signal_button_press_event().connect(MEMBER_SLOT(*this,
-                                                                &X11SystrayAppletWindow::on_button_press_event));
+      eventbox->signal_button_press_event().connect(sigc::mem_fun(*this,
+                                                                  &X11SystrayAppletWindow::on_button_press_event));
       container = eventbox;
 
-      view = new TimerBoxGtkView();
+      view = new TimerBoxGtkView(Menus::MENU_APPLET);
       timer_box_view = view;
       timer_box_control = new TimerBoxControl("applet", *timer_box_view);
 
+      Gtk::VBox *box = manage(new Gtk::VBox());
+      box->set_spacing(1);
+      box->pack_start(*view, true, true, 0);
+      
       if (System::is_kde())
         {
           timer_box_control->set_force_empty(true);
         }
 
-      container->add(*view);
+      container->add(*box);
 
-      plug->signal_embedded().connect(MEMBER_SLOT(*this, &X11SystrayAppletWindow::on_embedded));
-      plug->signal_delete_event().connect(MEMBER_SLOT(*this, &X11SystrayAppletWindow::delete_event));
+      plug->signal_embedded().connect(sigc::mem_fun(*this, &X11SystrayAppletWindow::on_embedded));
+      plug->signal_delete_event().connect(sigc::mem_fun(*this, &X11SystrayAppletWindow::on_delete_event));
+      plug->signal_size_allocate().connect(sigc::mem_fun(*this, &X11SystrayAppletWindow::on_size_allocate));
 
       plug->add(*container);
       plug->show_all();
 
-      // Tray menu
-      Menus *menus = Menus::get_instance();
-      menus->create_menu(Menus::MENU_APPLET);
-
-      ret = AppletWindow::APPLET_STATE_VISIBLE;
       applet_orientation = ORIENTATION_UP;
 
-#ifdef HAVE_GTKMM24
       Gtk::Requisition req;
       plug->size_request(req);
       applet_size = req.height;
-#else
-      GtkRequisition req;
-      plug->size_request(&req);
-      applet_size = req.height;
-#endif
-      view->set_geometry(applet_orientation, 24);
+
+      view->set_geometry(applet_orientation, applet_size);
 
       applet_active = true;
-      ret = APPLET_STATE_VISIBLE;
     }
 
   TRACE_EXIT();
@@ -219,7 +216,7 @@ X11SystrayAppletWindow::deactivate_applet()
 
 //! Applet window is deleted. Destroy applet.
 bool
-X11SystrayAppletWindow::delete_event(GdkEventAny *event)
+X11SystrayAppletWindow::on_delete_event(GdkEventAny *event)
 {
   (void) event;
   deactivate_applet();
@@ -237,15 +234,7 @@ X11SystrayAppletWindow::on_embedded()
 
   if (applet_active)
     {
-#ifdef HAVE_GTKMM24
-      Gtk::Requisition req;
-      plug->size_request(req);
-#else
-      GtkRequisition req;
-      plug->size_request(&req);
-#endif
-
-      GtkOrientation o = egg_tray_icon_get_orientation(tray_icon);
+      GtkOrientation o = wrgtk_tray_icon_get_orientation(tray_icon);
       Orientation orientation;
 
       if (o == GTK_ORIENTATION_VERTICAL)
@@ -257,20 +246,10 @@ X11SystrayAppletWindow::on_embedded()
           orientation = ORIENTATION_LEFT;
         }
 
-      if (orientation == ORIENTATION_UP ||
-          orientation == ORIENTATION_DOWN)
-        {
-          applet_size = req.width;
-        }
-      else
-        {
-          applet_size = req.height;
-        }
-
-      TRACE_MSG("Size = " << req.width << " " << req.height << " " << applet_orientation);
+      embedded = true;
+      applet_size = 24;
+      
       view->set_geometry(applet_orientation, applet_size);
-
-      TRACE_MSG(applet_size);
     }
 
   control->set_applet_state(AppletControl::APPLET_TRAY,
@@ -286,7 +265,8 @@ X11SystrayAppletWindow::on_button_press_event(GdkEventButton *event)
   bool ret = false;
 
   if (applet_active &&
-      event->type == GDK_BUTTON_PRESS)
+      event->type == GDK_BUTTON_PRESS &&
+      embedded)
     {
       if (event->button == 3)
         {
@@ -311,3 +291,50 @@ X11SystrayAppletWindow::button_clicked(int button)
   (void) button;
   timer_box_control->force_cycle();
 }
+
+
+void
+X11SystrayAppletWindow::on_size_allocate(Gtk::Allocation& allocation)
+{
+  TRACE_ENTER("X11SystrayAppletWindow::on_size_allocate");
+
+  if (embedded)
+    {
+      TRACE_MSG(allocation.get_x() << " " <<
+                allocation.get_y() << " " <<
+                allocation.get_width() << " " <<
+                allocation.get_height());
+      GtkOrientation o = wrgtk_tray_icon_get_orientation(tray_icon);
+      Orientation orientation;
+  
+      if (o == GTK_ORIENTATION_VERTICAL)
+        {
+          orientation = ORIENTATION_UP;
+        }
+      else
+        {
+          orientation = ORIENTATION_LEFT;
+        }
+  
+      if (orientation == ORIENTATION_UP ||
+          orientation == ORIENTATION_DOWN)
+        {
+          if (applet_size != allocation.get_width())
+            {
+              applet_size = allocation.get_width();
+              TRACE_MSG("New size = " << allocation.get_width() << " " << allocation.get_height() << " " << applet_orientation);
+              view->set_geometry(applet_orientation, applet_size);
+            }
+        }
+      else
+        {
+          if (applet_size != allocation.get_height())
+            {
+              applet_size = allocation.get_height();
+              TRACE_MSG("New size = " << allocation.get_width() << " " << allocation.get_height() << " " << applet_orientation);
+              view->set_geometry(applet_orientation, applet_size);
+            }
+        }
+    }
+  TRACE_EXIT();
+}
