@@ -1,21 +1,22 @@
 // preinclude.h --- Patch includes
 //
-// Copyright (C) 2002, 2003, 2004 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// Time-stamp: <2007-08-30 16:03:01 robc>
-//
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: preinclude.h 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: preinclude.h 1351 2007-10-14 20:56:54Z rcaelers $
 //
 
 #ifndef PREINCLUDE_HH
@@ -25,13 +26,6 @@
 #define NOMINMAX
 #endif
 
-#ifdef HAVE_GTKMM24
-#define MEMBER_SLOT sigc::mem_fun
-#else
-#define MEMBER_SLOT SigC::slot
-#endif
-
-
 #undef THREAD_PRIORITY_NORMAL
 #undef THREAD_PRIORITY_LOW
 #undef THREAD_PRIORITY_HIGH
