@@ -1,19 +1,22 @@
 // AppletWindow.cc --- Applet info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-static const char rcsid[] = "$Id: W32AppletWindow.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: W32AppletWindow.cc 1466 2008-03-08 21:06:41Z rcaelers $";
 
 #include "preinclude.h"
 
@@ -146,12 +149,17 @@ W32AppletWindow::update_view()
 void
 W32AppletWindow::update_menu()
 {
+  TRACE_ENTER("W32AppletWindow::update_menu");
   if (menu_sent)
     return;
 
+  TRACE_MSG("have to send");
+  
   HWND hwnd = get_applet_window();
   if (hwnd != NULL)
     {
+      TRACE_MSG("sending");
+
       COPYDATASTRUCT msg;
       msg.dwData = APPLET_MESSAGE_MENU;
       msg.cbData = sizeof(AppletMenuData);
@@ -160,6 +168,7 @@ W32AppletWindow::update_menu()
 
       menu_sent = true;
     }
+  TRACE_EXIT();
 }
 
 void
@@ -211,7 +220,11 @@ W32AppletWindow::init_menu(HWND hwnd)
 {
   menu_data.num_items = 0;
   menu_sent = false;
-  menu_data.command_window = hwnd;
+  /*
+    As noted in frontend/win32/applet/include/applet.hh: 
+    We pass the command_window HWND as a LONG for compatibility.
+  */
+  menu_data.command_window = HandleToLong( hwnd );
 }
 
 void
@@ -247,7 +260,7 @@ bool
 W32AppletWindow::on_applet_command(int command)
 {
   Menus *menus = Menus::get_instance();
-  menus->on_applet_command(command);
+  menus->applet_command(command);
 
   return false;
 }
