@@ -1,19 +1,22 @@
 // Exercise.cc --- Exercises
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: Exercise.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: Exercise.cc 1500 2008-03-18 21:58:38Z rcaelers $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,6 +32,8 @@
 #include <unistd.h>
 #include <assert.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 
 #include <glib.h>
 
@@ -43,7 +48,7 @@ struct ExerciseParser
   std::string cdata;
   ExerciseParser(std::list<Exercise> &exe);
 
-  const GList *i18n_languages;
+  const gchar * const *i18n_languages;  
 };
 
 static const gchar *
@@ -144,7 +149,7 @@ exercise_parser_start_element  (GMarkupParseContext *,
 
 /* Updates language dependent attribute */
 static void
-exercise_parse_update_i18n_attribute(const GList *languages,
+exercise_parse_update_i18n_attribute(const gchar * const *languages,
                                      std::string &cur_value, int &cur_rank,
                                      const std::string &new_value,
                                      const std::string &new_lang)
@@ -161,21 +166,17 @@ exercise_parse_update_i18n_attribute(const GList *languages,
           nl_len = 2;
         }
 
-      const GList *langs = languages;
-      r = 0;
-      while (langs)
+      for (r = 0; languages[r] != NULL; r++)
         {
-          const gchar *lang = (const gchar *) langs->data;
+          const gchar *lang = (const gchar *) languages[r];
 
           if (! strncmp(lang, nl, nl_len))
             {
               break;
             }
-          langs = langs->next;
-          r++;
         }
 
-      if (langs == NULL)
+      if (languages[r] == NULL)
         {
           // Language not found...
           if (cur_rank < 0)
@@ -190,7 +191,6 @@ exercise_parse_update_i18n_attribute(const GList *languages,
               cur_value = new_value;
               cur_rank = 9998;
             }
-
         }
       else
         {
@@ -262,11 +262,20 @@ exercise_parser_text (GMarkupParseContext *,
 
 ExerciseParser::ExerciseParser(std::list<Exercise> &exe)
 {
+  TRACE_ENTER("ExerciseParser::ExerciseParser");
   exercises = &exe;
   exercise = NULL;
   lang = "";
 
-  i18n_languages = nls_get_language_list("LC_MESSAGES");
+  i18n_languages =  g_get_language_names(); // nls_get_language_list("LC_MESSAGES");
+
+  // FIXME: remove this debugging
+  int i = 0;
+  for (i = 0; i18n_languages[i] != NULL; i++)
+    {
+      TRACE_MSG("lang " << i18n_languages[i]);
+    }
+  TRACE_EXIT();
 }
 
 
