@@ -1,19 +1,22 @@
 // ExercisesPanel.cc --- Exercises panel
 //
-// Copyright (C) 2002, 2003, 2004, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: ExercisesPanel.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: ExercisesPanel.cc 1549 2008-06-28 21:51:43Z rcaelers $
 //
 
 #include "config.h"
@@ -22,6 +25,9 @@
 
 #include "preinclude.h"
 
+#include <algorithm>
+
+#include <string.h>
 #include <gtkmm/stock.h>
 
 #include "ExercisesPanel.hh"
@@ -30,7 +36,7 @@
 #include "Util.hh"
 #include "Hig.hh"
 #include "nls.h"
-#include "ISoundPlayer.hh"
+#include "SoundPlayer.hh"
 #include "debug.hh"
 
 // This code can be removed once the following bug is closed:
@@ -211,7 +217,10 @@ ExercisesPanel::ExercisesPanel(Gtk::HButtonBox *dialog_action_area)
          exercises(Exercise::get_exercises())
 {
   standalone = dialog_action_area != NULL;
-
+        
+  copy(exercises.begin(), exercises.end(), back_inserter(shuffled_exercises));  
+  random_shuffle(shuffled_exercises.begin(), shuffled_exercises.end());
+  
   progress_bar.set_orientation(Gtk::PROGRESS_BOTTOM_TO_TOP);
 
   description_scroll.add(description_text);
@@ -245,7 +254,7 @@ ExercisesPanel::ExercisesPanel(Gtk::HButtonBox *dialog_action_area)
       stop_button =  manage(GtkUtil::create_custom_stock_button
                                          (NULL, Gtk::Stock::CLOSE));
       stop_button->signal_clicked()
-        .connect(MEMBER_SLOT(*this, &ExercisesPanel::on_stop));
+        .connect(sigc::mem_fun(*this, &ExercisesPanel::on_stop));
 
       Gtk::HBox *button_box = manage(new Gtk::HBox());
       Gtk::Label *browse_label = manage(new Gtk::Label());
@@ -276,13 +285,13 @@ ExercisesPanel::ExercisesPanel(Gtk::HButtonBox *dialog_action_area)
   // (end of ugly)
 
   back_button->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &ExercisesPanel::on_go_back));
+    .connect(sigc::mem_fun(*this, &ExercisesPanel::on_go_back));
 
   forward_button->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &ExercisesPanel::on_go_forward));
+    .connect(sigc::mem_fun(*this, &ExercisesPanel::on_go_forward));
 
   pause_button->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &ExercisesPanel::on_pause));
+    .connect(sigc::mem_fun(*this, &ExercisesPanel::on_pause));
 
 #ifdef FIXME // Causes fuzzy translations...
   tooltips = manage( new Gtk::Tooltips() );
@@ -298,7 +307,7 @@ ExercisesPanel::ExercisesPanel(Gtk::HButtonBox *dialog_action_area)
   pack_start(*description_widget, false, false, 0);
 
   heartbeat_signal = GUI::get_instance()->signal_heartbeat()
-    .connect(MEMBER_SLOT(*this, &ExercisesPanel::heartbeat));
+    .connect(sigc::mem_fun(*this, &ExercisesPanel::heartbeat));
 
   exercise_count = 0;
   reset();
@@ -328,7 +337,7 @@ void
 ExercisesPanel::reset()
 {
   int i = adjust_exercises_pointer(1);
-  exercise_iterator = exercises.begin();
+  exercise_iterator = shuffled_exercises.begin();
   while (i > 0)
     {
       exercise_iterator++;
@@ -344,7 +353,7 @@ ExercisesPanel::reset()
 void
 ExercisesPanel::start_exercise()
 {
-  if (exercises.size() > 0)
+  if (shuffled_exercises.size() > 0)
     {
       const Exercise &exercise = *exercise_iterator;
 
@@ -397,6 +406,11 @@ ExercisesPanel::refresh_sequence()
           image_iterator = exercise.sequence.begin();
         }
       show_image();
+      if (exercise_time != 0)
+        {
+          SoundPlayer *snd = GUI::get_instance()->get_sound_player();
+          snd->play_sound(SoundPlayer::SOUND_EXERCISE_STEP);
+        }
     }
 
   TRACE_EXIT();
@@ -424,9 +438,9 @@ void
 ExercisesPanel::on_go_back()
 {
   adjust_exercises_pointer(-1);
-  if (exercise_iterator == exercises.begin())
+  if (exercise_iterator == shuffled_exercises.begin())
     {
-      exercise_iterator = --(exercises.end());
+      exercise_iterator = --(shuffled_exercises.end());
     }
   else
     {
@@ -440,9 +454,9 @@ ExercisesPanel::on_go_forward()
 {
   adjust_exercises_pointer(1);
   exercise_iterator++;
-  if (exercise_iterator == exercises.end())
+  if (exercise_iterator == shuffled_exercises.end())
     {
-      exercise_iterator = exercises.begin();;
+      exercise_iterator = shuffled_exercises.begin();;
     }
   start_exercise();
 }
@@ -476,7 +490,7 @@ ExercisesPanel::heartbeat()
   if (paused || stopped)
     return;
 
-  if (exercises.size() == 0)
+  if (shuffled_exercises.size() == 0)
     return;
 
   const Exercise &exercise = *exercise_iterator;
@@ -484,15 +498,15 @@ ExercisesPanel::heartbeat()
   if (exercise_time >= exercise.duration)
     {
       on_go_forward();
-      ISoundPlayer *snd = GUI::get_instance()->get_sound_player();
+      SoundPlayer *snd = GUI::get_instance()->get_sound_player();
       exercise_num++;
       if (exercise_num == exercise_count)
         {
           on_stop();
         }
       snd->play_sound(stopped
-                      ? ISoundPlayer::SOUND_EXERCISES_ENDED
-                      : ISoundPlayer::SOUND_EXERCISE_ENDED);
+                      ? SoundPlayer::SOUND_EXERCISES_ENDED
+                      : SoundPlayer::SOUND_EXERCISE_ENDED);
     }
   else
     {
