@@ -1,19 +1,22 @@
 // StatisticsDialog.cc --- Statistics dialog
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006 Rob Caelers <robc@krandor.org>
+// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: StatisticsDialog.cc 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: StatisticsDialog.cc 1532 2008-04-08 20:08:30Z rcaelers $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -64,6 +67,11 @@ StatisticsDialog::StatisticsDialog()
   ICore *core = CoreFactory::get_core();
   statistics = core->get_statistics();
 
+  for (int i = 0; i < 5; i++)
+    {
+      activity_labels[i] = NULL;
+    }
+  
   init_gui();
   display_calendar_date();
 }
@@ -79,7 +87,7 @@ int
 StatisticsDialog::run()
 {
   // Periodic timer.
-  Glib::signal_timeout().connect(MEMBER_SLOT(*this, &StatisticsDialog::on_timer), 1000);
+  Glib::signal_timeout().connect(sigc::mem_fun(*this, &StatisticsDialog::on_timer), 1000);
 
   show_all();
   return 0;
@@ -89,13 +97,17 @@ StatisticsDialog::run()
 void
 StatisticsDialog::init_gui()
 {
+#if !defined(PLATFORM_OS_OSX)
   Gtk::Notebook *tnotebook = manage(new Gtk::Notebook());
   tnotebook->set_tab_pos(Gtk::POS_TOP);
-
+#else
+  Gtk::HBox *tnotebook= manage(new Gtk::HBox(false, 6));
+#endif
+  
   // Calendar
   calendar = manage(new Gtk::Calendar());
-  calendar->signal_month_changed().connect(MEMBER_SLOT(*this, &StatisticsDialog::on_calendar_month_changed));
-  calendar->signal_day_selected().connect(MEMBER_SLOT(*this, &StatisticsDialog::on_calendar_day_selected));
+  calendar->signal_month_changed().connect(sigc::mem_fun(*this, &StatisticsDialog::on_calendar_month_changed));
+  calendar->signal_day_selected().connect(sigc::mem_fun(*this, &StatisticsDialog::on_calendar_day_selected));
   calendar->display_options(Gtk::CALENDAR_SHOW_WEEK_NUMBERS
                             |Gtk::CALENDAR_SHOW_DAY_NAMES
                             |Gtk::CALENDAR_SHOW_HEADING);
@@ -105,19 +117,19 @@ StatisticsDialog::init_gui()
   first_btn
     = manage(GtkUtil::create_custom_stock_button(NULL,Gtk::Stock::GOTO_FIRST));
   first_btn->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &StatisticsDialog::on_history_goto_first));
+    .connect(sigc::mem_fun(*this, &StatisticsDialog::on_history_goto_first));
   last_btn
     = manage(GtkUtil::create_custom_stock_button(NULL,Gtk::Stock::GOTO_LAST));
   last_btn->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &StatisticsDialog::on_history_goto_last));
+    .connect(sigc::mem_fun(*this, &StatisticsDialog::on_history_goto_last));
   back_btn
     = manage(GtkUtil::create_custom_stock_button(NULL,Gtk::Stock::GO_BACK));
   back_btn->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &StatisticsDialog::on_history_go_back));
+    .connect(sigc::mem_fun(*this, &StatisticsDialog::on_history_go_back));
   forward_btn
     = manage(GtkUtil::create_custom_stock_button(NULL,Gtk::Stock::GO_FORWARD));
   forward_btn->signal_clicked()
-    .connect(MEMBER_SLOT(*this, &StatisticsDialog::on_history_go_forward));
+    .connect(sigc::mem_fun(*this, &StatisticsDialog::on_history_go_forward));
 
   btnbox->pack_start(*first_btn, true, true, 0);
   btnbox->pack_start(*back_btn, true, true, 0);
@@ -145,17 +157,18 @@ StatisticsDialog::init_gui()
   hbox->pack_start(*navbox, true, true, 0);
 
   create_break_page(tnotebook);
+#if !defined(PLATFORM_OS_OSX)
+  // No details activity statistics on OS X..
   create_activity_page(tnotebook);
-
+#endif
 
   tnotebook->show_all();
+
+#if !defined(PLATFORM_OS_OSX)
   tnotebook->set_current_page(0);
-
-  get_vbox()->pack_start(*hbox, true, true, 0);
-
-#ifdef HAVE_X
-  GtkUtil::set_wmclass(*this, "Statistics");
 #endif
+  
+  get_vbox()->pack_start(*hbox, true, true, 0);
 
   // Dialog
   add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
@@ -168,7 +181,7 @@ StatisticsDialog::init_gui()
 
 
 void
-StatisticsDialog::create_break_page(Gtk::Notebook *tnotebook)
+StatisticsDialog::create_break_page(Gtk::Widget *tnotebook)
 {
   Gtk::HBox *box = manage(new Gtk::HBox(false, 3));
   Gtk::Label *lab = manage(new Gtk::Label(_("Breaks")));
@@ -273,12 +286,17 @@ StatisticsDialog::create_break_page(Gtk::Notebook *tnotebook)
     }
 
   box->show_all();
-  tnotebook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
+  
+#if !defined(PLATFORM_OS_OSX)
+  ((Gtk::Notebook *)tnotebook)->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
+#else
+  ((Gtk::HBox *)tnotebook)->pack_start(*table, true, true, 0);
+#endif
 }
 
 
 void
-StatisticsDialog::create_activity_page(Gtk::Notebook *tnotebook)
+StatisticsDialog::create_activity_page(Gtk::Widget *tnotebook)
 {
   Gtk::HBox *box = manage(new Gtk::HBox(false, 3));
   Gtk::Label *lab = manage(new Gtk::Label(_("Activity")));
@@ -326,7 +344,7 @@ StatisticsDialog::create_activity_page(Gtk::Notebook *tnotebook)
     }
 
   box->show_all();
-  tnotebook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
+  ((Gtk::Notebook *)tnotebook)->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
 }
 
 
@@ -410,32 +428,36 @@ StatisticsDialog::display_statistics(IStatistics::DailyStats *stats)
 
   stringstream ss;
 
-  value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_MOVEMENT_TIME];
-  if (value > 24 * 60 * 60) {
-    value = 0;
-  }
-  activity_labels[0]->set_text(Text::time_to_string(value));
+  if (activity_labels[0] != NULL)
+    {
+      // Label not available is OS X
+      
+      value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_MOVEMENT_TIME];
+      if (value > 24 * 60 * 60) {
+        value = 0;
+      }
+      activity_labels[0]->set_text(Text::time_to_string(value));
 
-  value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_MOUSE_MOVEMENT];
-  ss.str("");
-  stream_distance(ss, value);
-  activity_labels[1]->set_text(ss.str());
+      value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_MOUSE_MOVEMENT];
+      ss.str("");
+      stream_distance(ss, value);
+      activity_labels[1]->set_text(ss.str());
 
-  value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_CLICK_MOVEMENT];
-  ss.str("");
-  stream_distance(ss, value);
-  activity_labels[2]->set_text(ss.str());
+      value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_CLICK_MOVEMENT];
+      ss.str("");
+      stream_distance(ss, value);
+      activity_labels[2]->set_text(ss.str());
 
-  value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_CLICKS];
-  ss.str("");
-  ss << value;
-  activity_labels[3]->set_text(ss.str());
-
-  value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_KEYSTROKES];
-  ss.str("");
-  ss << value;
-  activity_labels[4]->set_text(ss.str());
+      value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_CLICKS];
+      ss.str("");
+      ss << value;
+      activity_labels[3]->set_text(ss.str());
 
+      value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_KEYSTROKES];
+      ss.str("");
+      ss << value;
+      activity_labels[4]->set_text(ss.str());
+    }
 }
 
 
@@ -455,7 +477,10 @@ StatisticsDialog::clear_display_statistics()
     }
   for (int i = 0; i <= 4; i++)
     {
-      activity_labels[i]->set_text("");
+      if (activity_labels[i] != NULL)
+        {
+          activity_labels[i]->set_text("");
+        }
     }
 }
 
