@@ -1,20 +1,23 @@
 // BreakWindow.cc --- base class for the break windows
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: BreakWindow.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: BreakWindow.cc 1351 2007-10-14 20:56:54Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,7 +30,7 @@ static const char rcsid[] = "$Id: BreakWindow.cc 1298 2007-08-30 14:18:04Z rcael
 #include <math.h>
 
 #include "BreakWindow.hh"
-#include "BreakResponseInterface.hh"
+#include "IBreakResponse.hh"
 #include "System.hh"
 #include "Util.hh"
 
@@ -51,7 +54,7 @@ BreakWindow::~BreakWindow()
 
 //! Break response
 void
-BreakWindow::set_response(BreakResponseInterface *bri)
+BreakWindow::set_response(IBreakResponse *bri)
 {
   break_response = bri;
 
@@ -92,7 +95,7 @@ BreakWindow::stop()
 //! Self-Destruct
 /*!
  *  This method MUST be used to destroy the objects through the
- *  BreakWindowInterface. it is NOT possible to do a delete on
+ *  IBreakWindow. it is NOT possible to do a delete on
  *  this interface...
  */
 void
