@@ -1,20 +1,23 @@
 // TextGUI.cc --- The WorkRave GUI
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2008 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
 
-static const char rcsid[] = "$Id: GUI.cc 1298 2007-08-30 14:18:04Z rcaelers $";
+static const char rcsid[] = "$Id: GUI.cc 1506 2008-03-21 08:30:45Z rcaelers $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -34,22 +37,25 @@ static const char rcsid[] = "$Id: GUI.cc 1298 2007-08-30 14:18:04Z rcaelers $";
 #include "GUI.hh"
 #include "PreludeWindow.hh"
 #include "BreakWindow.hh"
-#include "BreakWindowInterface.hh"
+#include "IBreakWindow.hh"
 #include "MainWindow.hh"
 
 #include "CoreFactory.hh"
-#include "CoreInterface.hh"
-#include "ConfiguratorInterface.hh"
+#include "ICore.hh"
+#include "IConfigurator.hh"
 
 #include "System.hh"
-#include "BreakResponseInterface.hh"
+#include "IBreakResponse.hh"
 #include "SoundPlayer.hh"
 
-#ifdef WIN32
+#include "Util.hh"
+
+#ifdef PLATFORM_OS_WIN32
 #include "crashlog.h"
-#include "W32Compat.hh"
 #endif
 
+#include <glib-object.h>
+
 GUI *GUI::instance = NULL;
 
 const string GUI::CFG_KEY_GUI_BLOCK_MODE =  "gui/breaks/block_mode";
@@ -126,12 +132,18 @@ GUI::main()
 {
   TRACE_ENTER("GUI::main");
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   // Enable Windows structural exception handling.
   __try1(exception_handler);
 #endif
 
+#ifdef PLATFORM_OS_WIN32
+  System::init();
+#else
   System::init(NULL);
+#endif
+
+  g_type_init();
 
   init_debug();
   init_core();
@@ -140,9 +152,13 @@ GUI::main()
   // The main status window.
   main_window = new MainWindow();
 
-  GMainLoop *main_loop = g_main_loop_new(NULL, FALSE);
+  main_loop = g_main_loop_new(NULL, FALSE);
 
-  g_timeout_add(1000, static_on_timer, this);
+  const char *env = getenv("WORKRAVE_TEST");
+  if (env == NULL)
+    {
+      g_timeout_add(1000, static_on_timer, this);
+    }
 
   g_main_loop_run(main_loop);
   g_main_loop_unref(main_loop);
@@ -150,7 +166,7 @@ GUI::main()
   delete main_window;
   main_window = NULL;
 
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   // Disable Windows structural exception handling.
   __except1;
 #endif
@@ -169,7 +185,7 @@ GUI::terminate()
 
   collect_garbage();
 
-  // g_main_loop_quit(main_loop);
+  g_main_loop_quit(main_loop);
   TRACE_EXIT();
 }
 
@@ -232,7 +248,7 @@ GUI::init_nls()
 {
 #ifdef ENABLE_NLS
   const char *locale_dir;
-#ifdef WIN32
+#ifdef PLATFORM_OS_WIN32
   string dir = Util::get_application_directory() + "\\lib\\locale";
   locale_dir = dir.c_str();
 #else
@@ -249,7 +265,7 @@ GUI::init_nls()
 void
 GUI::init_core()
 {
-  char *display_name = NULL;
+  string display_name = "";
 
   core = CoreFactory::get_core();
   core->init(argc, argv, this, display_name);
@@ -271,7 +287,7 @@ GUI::core_event_notify(CoreEvent event)
 {
   TRACE_ENTER_MSG("GUI::core_event_notify", event)
   // FIXME: HACK
-  SoundPlayerInterface::Sound snd = (SoundPlayerInterface::Sound) event;
+  ISoundPlayer::Sound snd = (ISoundPlayer::Sound) event;
   if (sound_player != NULL)
     {
       TRACE_MSG("play");
@@ -281,12 +297,22 @@ GUI::core_event_notify(CoreEvent event)
 }
 
 
-//! Returns a break window for the specified break.
-BreakWindowInterface *
-GUI::create_break_window(BreakId break_id, bool ignorable)
+void
+GUI::core_event_operation_mode_changed(const OperationMode m)
 {
-  BreakWindowInterface *ret = NULL;
+  (void) m;
+}
+
+//! Returns a break window for the specified break.
+IBreakWindow *
+GUI::new_break_window(BreakId break_id, bool user_initiated)
+{
+  IBreakWindow *ret = NULL;
   BlockMode block_mode = get_block_mode();
+  bool ignorable = true;
+
+  (void) user_initiated;
+  
   if (break_id == BREAK_ID_MICRO_BREAK)
     {
       ret = new BreakWindow(break_id, ignorable, block_mode);
@@ -305,14 +331,14 @@ GUI::create_break_window(BreakId break_id, bool ignorable)
 
 
 void
-GUI::set_break_response(BreakResponseInterface *rep)
+GUI::set_break_response(IBreakResponse *rep)
 {
   response = rep;
 }
 
 
 void
-GUI::start_prelude_window(BreakId break_id)
+GUI::create_prelude_window(BreakId break_id)
 {
   hide_break_window();
   collect_garbage();
@@ -321,12 +347,11 @@ GUI::start_prelude_window(BreakId break_id)
 
   prelude_window = new PreludeWindow(break_id);
   prelude_window->set_response(response);
-  prelude_window->start();
 }
 
 
 void
-GUI::start_break_window(BreakId break_id, bool ignorable)
+GUI::create_break_window(BreakId break_id, bool ignorable)
 {
   TRACE_ENTER("GUI::start_break_window");
 
@@ -335,9 +360,8 @@ GUI::start_break_window(BreakId break_id, bool ignorable)
 
   active_break_id = break_id;
 
-  break_window = create_break_window(break_id, ignorable);
+  break_window = new_break_window(break_id, ignorable);
   break_window->set_response(response);
-  break_window->start();
 
   if (get_block_mode() != GUI::BLOCK_MODE_NONE)
     {
@@ -370,6 +394,25 @@ GUI::hide_break_window()
   TRACE_EXIT();
 }
 
+void
+GUI::show_break_window()
+{
+  TRACE_ENTER("GUI::hide_break_window");
+  active_break_id = BREAK_ID_NONE;
+
+  if (prelude_window != NULL)
+    {
+      prelude_window->start();
+    }
+
+  if (break_window != NULL)
+    {
+      break_window->start();
+    }
+
+  TRACE_EXIT();
+}
+
 
 void
 GUI::refresh_break_window()
@@ -454,7 +497,7 @@ GUI::get_block_mode()
   bool b;
   int mode;
   b = CoreFactory::get_configurator()
-    ->get_value(CFG_KEY_GUI_BLOCK_MODE, &mode);
+    ->get_value(CFG_KEY_GUI_BLOCK_MODE, mode);
   if (! b)
     {
       mode = BLOCK_MODE_INPUT;
@@ -469,3 +512,5 @@ GUI::set_block_mode(BlockMode mode)
   CoreFactory::get_configurator()
     ->set_value(CFG_KEY_GUI_BLOCK_MODE, int(mode));
 }
+
+
