@@ -1,19 +1,22 @@
 // DeskBand.cpp --- CDeskBand implementation
 //
-// Copyright (C) 2004, 2005, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: DeskBand.cpp 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: DeskBand.cpp 1373 2007-11-27 18:09:26Z rcaelers $
 
 #include <windowsx.h>
 
@@ -339,6 +342,32 @@ CDeskBand::GetBandInfo(DWORD dwBandID, DWORD dwViewMode, DESKBANDINFO* pdbi)
   return E_INVALIDARG;
 }
 
+STDMETHODIMP
+CDeskBand::CanRenderComposited( BOOL *pfCanRenderComposited )
+{
+  if( !pfCanRenderComposited )
+      return E_INVALIDARG;
+  
+  *pfCanRenderComposited = TRUE;
+  return S_OK;
+}
+
+STDMETHODIMP
+CDeskBand::GetCompositionState( BOOL *pfCompositionEnabled )
+{
+  if( !pfCompositionEnabled )
+      return E_INVALIDARG;
+  
+  *pfCompositionEnabled = TRUE;
+  return S_OK;
+}
+
+STDMETHODIMP
+CDeskBand::SetCompositionState( BOOL fCompositionEnabled )
+{
+  return S_OK;
+}
+
 STDMETHODIMP
 CDeskBand::GetClassID(LPCLSID pClassID)
 {
@@ -377,7 +406,7 @@ CDeskBand::QueryContextMenu( HMENU hMenu,
                                           UINT idCmdLast,
                                           UINT uFlags)
 {
-  if ((!m_HasAppletMenu) || (CMF_DEFAULTONLY & uFlags) || !IsWindow(m_AppletMenu.command_window))
+  if ((!m_HasAppletMenu) || (CMF_DEFAULTONLY & uFlags) || !IsWindow( get_command_window() ))
     return MAKE_HRESULT(SEVERITY_SUCCESS, 0, USHORT(0));
 
   int m = 0;
@@ -441,9 +470,9 @@ CDeskBand::InvokeCommand(LPCMINVOKECOMMANDINFO lpcmi)
   int cmd = LOWORD(lpcmi->lpVerb);
   HRESULT ret;
 
-  if (m_HasAppletMenu && cmd >= 0 && cmd < m_AppletMenu.num_items && IsWindow(m_AppletMenu.command_window))
+  if (m_HasAppletMenu && cmd >= 0 && cmd < m_AppletMenu.num_items && IsWindow( get_command_window() ))
     {
-      SendMessage(m_AppletMenu.command_window, WM_USER, m_AppletMenu.items[cmd].command, NULL);
+      SendMessage( get_command_window(), WM_USER, m_AppletMenu.items[cmd].command, NULL );
       ret = NOERROR;
     }
   else
@@ -454,7 +483,7 @@ CDeskBand::InvokeCommand(LPCMINVOKECOMMANDINFO lpcmi)
 }
 
 STDMETHODIMP
-CDeskBand::GetCommandString( UINT idCommand,
+CDeskBand::GetCommandString( UINT_PTR idCommand,
                                           UINT uFlags,
                                           LPUINT lpReserved,
                                           LPSTR lpszName,
@@ -467,15 +496,17 @@ CDeskBand::GetCommandString( UINT idCommand,
 LRESULT CALLBACK
 CDeskBand::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
-  CDeskBand  *pThis = (CDeskBand*)GetWindowLong(hWnd, GWL_USERDATA);
+  CDeskBand  *pThis = (CDeskBand*)GetWindowLongPtr(hWnd, GWLP_USERDATA);
 
   switch (uMessage)
     {
     case WM_NCCREATE:
       {
         LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
-        pThis = (CDeskBand*)(lpcs->lpCreateParams);
-        SetWindowLong(hWnd, GWL_USERDATA, (LONG)pThis);
+        pThis = (CDeskBand *)( lpcs->lpCreateParams );
+        SetWindowLongPtr( hWnd, GWLP_USERDATA, (LONG_PTR)pThis );
+        SetWindowPos( hWnd, NULL, 0, 0, 0, 0, 
+            SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED );
 
         //set the window handle
         pThis->m_hWnd = hWnd;
@@ -506,7 +537,8 @@ CDeskBand::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
       break;
 
     case WM_LBUTTONUP:
-      SendMessage(pThis->m_AppletMenu.command_window, WM_USER + 1, 0, NULL);
+      SendMessage( (HWND)LongToHandle( pThis->m_AppletMenu.command_window ),
+          WM_USER + 1, 0, NULL );
       break;
     }
   return DefWindowProc(hWnd, uMessage, wParam, lParam);
