@@ -1,19 +1,22 @@
 // DeskBand.h --- CDeskBand definitions
 //
-// Copyright (C) 2004, 2005 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: DeskBand.h 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: DeskBand.h 1373 2007-11-27 18:09:26Z rcaelers $
 
 #include <windows.h>
 #include <shlobj.h>
@@ -34,7 +37,7 @@
 
 class TimerBox;
 
-class CDeskBand : public IDeskBand,
+class CDeskBand : public IDeskBand2,
                   public IInputObject,
                   public IObjectWithSite,
                   public IPersistStream,
@@ -64,6 +67,11 @@ public:
   //IDeskBand methods
   STDMETHOD (GetBandInfo) (DWORD, DWORD, DESKBANDINFO*);
 
+  //IDeskBand2 methods
+  STDMETHOD ( CanRenderComposited )( BOOL * );
+  STDMETHOD ( GetCompositionState )( BOOL * );
+  STDMETHOD ( SetCompositionState )( BOOL );
+
   //IInputObject methods
   STDMETHOD (UIActivateIO) (BOOL, LPMSG);
   STDMETHOD (HasFocusIO) (void);
@@ -83,7 +91,7 @@ public:
   //IContextMenu methods
   STDMETHOD (QueryContextMenu)(HMENU, UINT, UINT, UINT, UINT);
   STDMETHOD (InvokeCommand)(LPCMINVOKECOMMANDINFO);
-  STDMETHOD (GetCommandString)(UINT, UINT, LPUINT, LPSTR, UINT);
+  STDMETHOD (GetCommandString)(UINT_PTR, UINT, LPUINT, LPSTR, UINT);
 
   HWND get_command_window() const;
 
@@ -115,7 +123,7 @@ private:
 inline HWND
 CDeskBand::get_command_window() const
 {
-  return m_AppletMenu.command_window;
+  return (HWND)LongToHandle( m_AppletMenu.command_window );
 }
 
 #endif   //DESKBAND_H
