@@ -1,19 +1,22 @@
 // Icon.cpp --- Icon
 //
-// Copyright (C) 2004, 2005 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: Icon.cpp 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: Icon.cpp 1373 2007-11-27 18:09:26Z rcaelers $
 
 #include <stdio.h>
 
@@ -27,9 +30,8 @@ Icon::Icon(HWND parent, HINSTANCE hinst, const char *resource, CDeskBand *deskba
 {
   init(hinst);
   icon = LoadIcon(hinst, resource);
-  hwnd = CreateWindowEx(0, ICON_CLASS_NAME, "",
-                        WS_CHILD | WS_CLIPSIBLINGS, 0, 0, 16, 16, parent, NULL, hinst, NULL);
-  SetWindowLong(hwnd, GWL_USERDATA, (LONG)this);
+  hwnd = CreateWindowEx( 0, ICON_CLASS_NAME, "",
+      WS_CHILD | WS_CLIPSIBLINGS, 0, 0, 16, 16, parent, NULL, hinst, (LPVOID)this );
 }
 
 Icon::~Icon()
@@ -40,10 +42,20 @@ Icon::~Icon()
 LRESULT CALLBACK
 Icon::wnd_proc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
-  Icon  *pThis = (Icon*)GetWindowLong(hWnd, GWL_USERDATA);
+  Icon  *pThis = (Icon*)GetWindowLongPtr(hWnd, GWLP_USERDATA);
 
   switch (uMessage)
     {
+    case WM_NCCREATE:
+      {
+        LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
+        pThis = (Icon *)( lpcs->lpCreateParams );
+        SetWindowLongPtr( hWnd, GWLP_USERDATA, (LONG_PTR)pThis );
+        SetWindowPos( hWnd, NULL, 0, 0, 0, 0, 
+            SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED );
+      }
+      break;
+    
     case WM_PAINT:
       return pThis->on_paint();
 
