@@ -1,19 +1,22 @@
 // TimeBar.cpp --- Time bar
 //
-// Copyright (C) 2004, 2005, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimeBar.cpp 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimeBar.cpp 1373 2007-11-27 18:09:26Z rcaelers $
 
 #include <stdio.h>
 
@@ -45,8 +48,7 @@ TimeBar::TimeBar(HWND parent, HINSTANCE hinst, CDeskBand *deskband)
   bar_color = ITimeBar::COLOR_ID_ACTIVE;
 
   hwnd = CreateWindowEx(0, TIME_BAR_CLASS_NAME, "",
-                        WS_CHILD | WS_CLIPSIBLINGS, 0, 0, 56, 16, parent, NULL, hinst, NULL);
-  SetWindowLong(hwnd, GWL_USERDATA, (LONG)this);
+      WS_CHILD | WS_CLIPSIBLINGS, 0, 0, 56, 16, parent, NULL, hinst, (LPVOID)this );
 
   compute_size(width, height);
   SetWindowPos(hwnd, NULL, 0, 0, width, height, SWP_NOZORDER|SWP_NOMOVE|SWP_NOACTIVATE);
@@ -60,10 +62,20 @@ TimeBar::~TimeBar()
 LRESULT CALLBACK
 TimeBar::wnd_proc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
-  TimeBar  *pThis = (TimeBar*)GetWindowLong(hWnd, GWL_USERDATA);
+  TimeBar  *pThis = (TimeBar*)GetWindowLongPtr(hWnd, GWLP_USERDATA);
 
   switch (uMessage)
     {
+    case WM_NCCREATE:
+      {
+        LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
+        pThis = (TimeBar *)( lpcs->lpCreateParams );
+        SetWindowLongPtr( hWnd, GWLP_USERDATA, (LONG_PTR)pThis );
+        SetWindowPos( hWnd, NULL, 0, 0, 0, 0, 
+            SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED );
+      }
+      break;
+
     case WM_PAINT:
       return pThis->on_paint();
 
@@ -96,7 +108,7 @@ TimeBar::compute_size(int &w, int &h)
   rect.top = 0;
   rect.bottom = 0;
   rect.right = 0;
-  h= DrawText(dc, buf, strlen(buf), &rect, DT_CALCRECT);
+  h= DrawText(dc, buf, (int)strlen( buf ), &rect, DT_CALCRECT);
   if (! h)
     {
       w = 32;
@@ -226,7 +238,7 @@ TimeBar::on_paint(void)
   SetBkMode(dc, TRANSPARENT);
   r.right -= border_size + MARGINX;
   r.left += border_size + MARGINX;
-  DrawText(dc, bar_text, strlen(bar_text), &r, DT_SINGLELINE|DT_VCENTER|DT_RIGHT);
+  DrawText(dc, bar_text, (int)strlen( bar_text ), &r, DT_SINGLELINE|DT_VCENTER|DT_RIGHT);
 
   ReleaseDC(hwnd, dc);
   EndPaint(hwnd, &ps);
@@ -268,10 +280,26 @@ TimeBar::init(HINSTANCE hinst)
       bar_colors[ITimeBar::COLOR_ID_INACTIVE_OVER_OVERDUE] = light_green;
       bar_colors[ITimeBar::COLOR_ID_BG] = bg;
 
-      NONCLIENTMETRICS ncm;
-      ncm.cbSize = sizeof(NONCLIENTMETRICS);
+      NONCLIENTMETRICS_PRE_VISTA_STRUCT ncm;
+	  LOGFONT lfDefault =
+	  // the default status font info on my system:
+	    {
+          -12, 0, 0, 0, 400, 0, 0, 0, '\1', 0, 0, 0, 0, TEXT( "Tahoma" )
+		  //0, 0x00146218, 0, 0x001461F0, 0, '@', 0, 0, 0, 0, 0, 0, 0, TEXT( "~" )
+	    };
 
-      SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0);
+      ZeroMemory( &ncm, sizeof( ncm ) );
+      ncm.cbSize = sizeof( ncm );
+      ncm.lfStatusFont = lfDefault;
+	  
+	  if( !SystemParametersInfo( SPI_GETNONCLIENTMETRICS, sizeof( ncm ), &ncm, 0 ) )
+	  // If SystemParametersInfo fails, use my default.
+	  // Now that we're filling a pre-vista NCM struct, there 
+	  // shouldn't be any problem though, regardless of target.
+	    {
+		  ncm.lfStatusFont = lfDefault;
+		}
+      
       bar_font = CreateFontIndirect(&ncm.lfStatusFont);
 
     }
@@ -285,7 +313,6 @@ TimeBar::init(HINSTANCE hinst)
 void
 TimeBar::time_to_string(time_t time, char *buf, int len)
 {
-  // FIXME: Should validate len.
   char t[2];
 
   if (time < 0)
@@ -298,17 +325,17 @@ TimeBar::time_to_string(time_t time, char *buf, int len)
     {
       t[0] = 0;
     }
-  int hrs = time/3600;
-  int min = (time / 60) % 60;
-  int sec = time % 60;
+  int hrs = (int)( time / 3600 );
+  int min = (int)( (time / 60) % 60 );
+  int sec = (int)( time % 60 );
 
   if (hrs > 0)
     {
-      sprintf(buf, "%s%d:%02d:%02d", t, hrs, min, sec);
+      _snprintf_s( buf, len, _TRUNCATE, "%s%d:%02d:%02d", t, hrs, min, sec );
     }
   else
     {
-      sprintf(buf, "%s%d:%02d", t, min, sec);
+      _snprintf_s( buf, len, _TRUNCATE, "%s%d:%02d", t, min, sec );
     }
 }
 
@@ -330,9 +357,7 @@ TimeBar::set_secondary_progress(int value, int max_value)
 void
 TimeBar::set_text(const char *text)
 {
-  strncpy(bar_text, text, APPLET_BAR_TEXT_MAX_LENGTH-1);
-  bar_text[APPLET_BAR_TEXT_MAX_LENGTH-1] = '\0';
-
+  strncpy_s( bar_text, APPLET_BAR_TEXT_MAX_LENGTH, text, _TRUNCATE );
 }
 
 void
