@@ -1,19 +1,22 @@
 // TimeBar.h --- Time bar
 //
-// Copyright (C) 2004, 2005, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimeBar.h 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimeBar.h 1373 2007-11-27 18:09:26Z rcaelers $
 
 #ifndef TIMEBAR_H
 #define TIMEBAR_H
@@ -66,5 +69,31 @@ private:
   void time_to_string(time_t time, char *buf, int len);
 };
 
+
+struct NONCLIENTMETRICS_PRE_VISTA_STRUCT 
+{
+    UINT    cbSize;
+    int     iBorderWidth;
+    int     iScrollWidth;
+    int     iScrollHeight;
+    int     iCaptionWidth;
+    int     iCaptionHeight;
+    LOGFONT lfCaptionFont;
+    int     iSmCaptionWidth;
+    int     iSmCaptionHeight;
+    LOGFONT lfSmCaptionFont;
+    int     iMenuWidth;
+    int     iMenuHeight;
+    LOGFONT lfMenuFont;
+    LOGFONT lfStatusFont;
+    LOGFONT lfMessageFont;
+	/*
+	This is a pre-vista structure for compatibility across platforms.
+	Normally, when Vista is the target build (WINVER 0x0600), 
+	NONCLIENTMETRICS structs contain an ifdef WINVER >= 0x0600:
+	int     iPaddedBorderWidth;
+	*/
+};
+
 #endif // TIMEBAR_H
 
