@@ -1,19 +1,22 @@
 // TimerBox.h --- Timer box
 //
-// Copyright (C) 2004, 2005, 2006 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2004, 2005, 2006, 2007 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
-// This program is free software; you can redistribute it and/or modify
+// This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: TimerBox.h 1298 2007-08-30 14:18:04Z rcaelers $
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Id: TimerBox.h 1354 2007-10-15 19:38:16Z rcaelers $
 
 #ifndef TIMERBOX_H
 #define TIMERBOX_H
@@ -28,6 +31,8 @@ class TimeBar;
 class Icon;
 class CDeskBand;
 
+using namespace workrave;
+
 class TimerBox
 {
  public:
